/**
 * @file          commands/falcon/adk/clone.ts
 * @copyright     Vivek M. Chawla - 2018
 * @author        Vivek M. Chawla <@VivekMChawla>
 * @summary       Implements the CLI command "falcon:adk:clone"
 * @description   Salesforce CLI Plugin command (falcon:adk:clone) that allows a Salesforce DX
 *                developer to clone an existing AppExchange Demo Kit (ADK) based project. After
 *                the ADK project code is cloned, the user is taken through an interview to help
 *                set up developer-specific project variables.
 * @version       1.0.0
 * @license       MIT
 */
import { flags } from '@salesforce/command';
import { SfdxError } from '@salesforce/core';
import { AnyJson } from '@salesforce/ts-types';
import { SfdxFalconError } from '../../../modules/sfdx-falcon-error';
import { SfdxFalconYeomanCommand } from '../../../modules/sfdx-falcon-yeoman-command';
/**
 * @class       FalconAdkClone
 * @extends     SfdxFalconYeomanCommand
 * @summary     Implements the CLI Command "falcon:adk:clone"
 * @description The command "falcon:adk:clone" allows a Salesforce DX developer to clone an
 *              existing project that's based on the AppExchange Demo Kit (ADK) template.  After
 *              the project is cloned, the user is taken through an interview to help set up
 *              developer-specific project variables.
 * @public
 */
export default class FalconAdkClone extends SfdxFalconYeomanCommand {
    static description: string;
    static hidden: boolean;
    static examples: string[];
    static args: {
        name: string;
        description: string;
        required: boolean;
        hidden: boolean;
    }[];
    protected static flagsConfig: {
        falcondebug: flags.Discriminated<flags.Array<string>>;
        falcondebugerror: flags.Discriminated<flags.Boolean<boolean>>;
        falcondebugsuccess: flags.Discriminated<flags.Boolean<boolean>>;
        falcondebugdepth: flags.Discriminated<flags.Number>;
        outputdir: flags.Discriminated<flags.Option<string>>;
    };
    protected static requiresProject: boolean;
    protected static requiresUsername: boolean;
    protected static requiresDevhubUsername: boolean;
    protected static supportsUsername: boolean;
    protected static supportsDevhubUsername: boolean;
    /**
     * @method      run
     * @returns     {Promise<AnyJson>}  Resolves with a JSON object that the CLI
     *              will pass to the user as stdout if the --json flag was set.
     * @description Entrypoint function for "sfdx falcon:adk:clone".
     * @public @async
     */
    run(): Promise<AnyJson>;
    /**
     * @method      buildFinalError
     * @param       {SfdxFalconError} cmdError  Required. Error object used as
     *              the basis for the "friendly error message" being created
     *              by this method.
     * @returns     {SfdxError}
     * @description Builds a user-friendly error message that is appropriate to
     *              the CLI command that's being implemented by this class. The
     *              output of this method will always be used by the onError()
     *              method from the base class to communicate the end-of-command
     *              error state.
     * @protected
     */
    protected buildFinalError(cmdError: SfdxFalconError): SfdxError;
}
