"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
//─────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @file          commands/falcon/adk/clone.ts
 * @copyright     Vivek M. Chawla - 2018
 * @author        Vivek M. Chawla <@VivekMChawla>
 * @summary       Implements the CLI command "falcon:adk:clone"
 * @description   Salesforce CLI Plugin command (falcon:adk:clone) that allows a Salesforce DX
 *                developer to clone an existing AppExchange Demo Kit (ADK) based project. After
 *                the ADK project code is cloned, the user is taken through an interview to help
 *                set up developer-specific project variables.
 * @version       1.0.0
 * @license       MIT
 */
//─────────────────────────────────────────────────────────────────────────────────────────────────┘
// Import External Modules
const command_1 = require("@salesforce/command"); // Allows creation of flags for CLI commands.
const core_1 = require("@salesforce/core"); // Messages library that simplifies using external JSON for string reuse.
// Import Local Modules
const sfdx_falcon_error_1 = require("../../../modules/sfdx-falcon-error"); // Extends SfdxError to provide specialized error structures for SFDX-Falcon modules.
const git_1 = require("../../../modules/sfdx-falcon-util/git"); // Function. Validates a Git URI.
const sfdx_falcon_yeoman_command_1 = require("../../../modules/sfdx-falcon-yeoman-command"); // Base class that CLI commands in this project that use Yeoman should use.
// Import Internal Types
const sfdx_falcon_command_1 = require("../../../modules/sfdx-falcon-command"); // Enum. Represents the types of SFDX-Falcon Commands.
// Set the File Local Debug Namespace
const dbgNs = 'COMMAND:falcon-adk-clone:';
// Use SfdxCore's Messages framework to get the message bundle for this command.
core_1.Messages.importMessagesDirectory(__dirname);
const baseMessages = core_1.Messages.loadMessages('sfdx-falcon', 'sfdxFalconCommand');
const commandMessages = core_1.Messages.loadMessages('sfdx-falcon', 'falconAdkClone');
//─────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @class       FalconAdkClone
 * @extends     SfdxFalconYeomanCommand
 * @summary     Implements the CLI Command "falcon:adk:clone"
 * @description The command "falcon:adk:clone" allows a Salesforce DX developer to clone an
 *              existing project that's based on the AppExchange Demo Kit (ADK) template.  After
 *              the project is cloned, the user is taken through an interview to help set up
 *              developer-specific project variables.
 * @public
 */
//─────────────────────────────────────────────────────────────────────────────────────────────────┘
class FalconAdkClone extends sfdx_falcon_yeoman_command_1.SfdxFalconYeomanCommand {
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      run
     * @returns     {Promise<AnyJson>}  Resolves with a JSON object that the CLI
     *              will pass to the user as stdout if the --json flag was set.
     * @description Entrypoint function for "sfdx falcon:adk:clone".
     * @public @async
     */
    //───────────────────────────────────────────────────────────────────────────┘
    async run() {
        // Initialize the SfdxFalconCommand base (DO NOT REMOVE THIS LINE OF CODE!)
        this.sfdxFalconCommandInit('falcon:adk:clone', sfdx_falcon_command_1.SfdxFalconCommandType.APPX_DEMO);
        // Validate the gitRemoteUri passed in by the CLI Command
        if (git_1.isGitUriValid(this.gitRemoteUri) === false) {
            throw new sfdx_falcon_error_1.SfdxFalconError(`The value '${this.gitRemoteUri}' is not a valid Git Remote URI.`, `InvalidGitUri`, `${dbgNs}run`);
        }
        // Make sure the gitRemoteUri uses the https protocol.
        // Makes it less likey the user will hang on SSH messages.
        if (this.gitRemoteUri.substr(0, 8) !== 'https://') {
            throw new sfdx_falcon_error_1.SfdxFalconError(`Git Remote URI must use the https protocol (ex. 'https://github.com/GitHubUser/my-repository.git')`, `InvalidGitUriProtocol`, `${dbgNs}run`);
        }
        // Run a Yeoman Generator to interact with and run tasks for the user.
        await super.runYeomanGenerator({
            generatorType: 'clone-appx-demo-project',
            gitRemoteUri: this.gitRemoteUri,
            outputDir: this.outputDirectory,
            gitCloneDir: this.gitCloneDirectory,
            options: []
        })
            .then(generatorResult => this.onSuccess(generatorResult)) // Implemented by parent class
            .catch(generatorResult => this.onError(generatorResult)); // Implemented by parent class
        // Return the JSON Response that was created by onSuccess()
        return this.falconJsonResponse;
    }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      buildFinalError
     * @param       {SfdxFalconError} cmdError  Required. Error object used as
     *              the basis for the "friendly error message" being created
     *              by this method.
     * @returns     {SfdxError}
     * @description Builds a user-friendly error message that is appropriate to
     *              the CLI command that's being implemented by this class. The
     *              output of this method will always be used by the onError()
     *              method from the base class to communicate the end-of-command
     *              error state.
     * @protected
     */
    //───────────────────────────────────────────────────────────────────────────┘
    buildFinalError(cmdError) {
        // If not implementing anything special here, simply return cmdError.
        return cmdError;
    }
}
// Define the basic properties of this CLI command.
FalconAdkClone.description = commandMessages.getMessage('commandDescription');
FalconAdkClone.hidden = false;
FalconAdkClone.examples = [
    `$ sfdx falcon:adk:clone https://github.com/GitHubUser/my-repository.git`,
    `$ sfdx falcon:adk:clone https://github.com/GitHubUser/my-repository.git MyRepoDirName`,
    `$ sfdx falcon:adk:clone https://github.com/GitHubUser/my-repository.git MyRepoDirName \\\n` +
        `                       --outputdir ~/demos/appexchange-demo-kit-projects`
];
//───────────────────────────────────────────────────────────────────────────┐
// Define the ARGUMENTS used by this command.
// Position 1 (GIT_REMOTE_URI)  - URI of the git repository being cloned.
// Position 2 (GIT_CLONE_DIR)   - Name of the locally cloned repo directory.
//───────────────────────────────────────────────────────────────────────────┘
FalconAdkClone.args = [
    {
        name: 'GIT_REMOTE_URI',
        description: baseMessages.getMessage('gitRemoteUri_ArgDescription'),
        required: true,
        hidden: false
    },
    {
        name: 'GIT_CLONE_DIR',
        description: baseMessages.getMessage('gitCloneDir_ArgDescription'),
        required: false,
        hidden: false
    }
];
//───────────────────────────────────────────────────────────────────────────┐
// Define the FLAGS used by this command.
// -d --OUTPUTDIR   Directory where AppX Demo Kit (ADK) project will be cloned
//                  to.  Defaults to . (current directory) is not specified.
//───────────────────────────────────────────────────────────────────────────┘
FalconAdkClone.flagsConfig = Object.assign({ outputdir: command_1.flags.directory({
        char: 'd',
        required: false,
        description: commandMessages.getMessage('outputdir_FlagDescription'),
        default: '.',
        hidden: false
    }) }, sfdx_falcon_yeoman_command_1.SfdxFalconYeomanCommand.falconBaseflagsConfig);
// Identify the core SFDX arguments/features required by this command.
FalconAdkClone.requiresProject = false; // True if an SFDX Project workspace is REQUIRED.
FalconAdkClone.requiresUsername = false; // True if an org username is REQUIRED.
FalconAdkClone.requiresDevhubUsername = false; // True if a hub org username is REQUIRED.
FalconAdkClone.supportsUsername = false; // True if an org username is OPTIONAL.
FalconAdkClone.supportsDevhubUsername = false; // True if a hub org username is OPTIONAL.
exports.default = FalconAdkClone;
//# sourceMappingURL=clone.js.map