/**
 * @file          commands/falcon/adk/create.ts
 * @copyright     Vivek M. Chawla - 2018
 * @author        Vivek M. Chawla <@VivekMChawla>
 * @summary       Implements the CLI command "falcon:adk:create"
 * @description   Salesforce CLI Plugin command (falcon:adk:create) that allows a Salesforce DX
 *                developer to create an empty project based on the AppExchange Demo Kit (ADK)
 *                template.  Once the ADK project is created, the user is guided through an
 *                interview where they define key ADK project settings which are then used to
 *                customize the ADK project scaffolding that gets created on their local machine.
 * @version       1.0.0
 * @license       MIT
 */
import { flags } from '@salesforce/command';
import { SfdxError } from '@salesforce/core';
import { AnyJson } from '@salesforce/ts-types';
import { SfdxFalconError } from '../../../modules/sfdx-falcon-error';
import { SfdxFalconYeomanCommand } from '../../../modules/sfdx-falcon-yeoman-command';
/**
 * @class       FalconAdkCreate
 * @extends     SfdxFalconYeomanCommand
 * @summary     Implements the CLI Command "falcon:adk:create"
 * @description The command "falcon:adk:create" creates a local AppExchange Demo Kit (ADK)
 *              the project at https://github.com/sfdx-isv/sfdx-falcon-appx-demo-kit as a template.
 *              Uses Yeoman to create customized ADK project scaffolding on the user's machine.
 * @public
 */
export default class FalconAdkCreate extends SfdxFalconYeomanCommand {
    static description: string;
    static hidden: boolean;
    static examples: string[];
    protected static flagsConfig: {
        falcondebug: flags.Discriminated<flags.Array<string>>;
        falcondebugerror: flags.Discriminated<flags.Boolean<boolean>>;
        falcondebugsuccess: flags.Discriminated<flags.Boolean<boolean>>;
        falcondebugdepth: flags.Discriminated<flags.Number>;
        outputdir: flags.Discriminated<flags.Option<string>>;
    };
    protected static requiresProject: boolean;
    protected static requiresUsername: boolean;
    protected static requiresDevhubUsername: boolean;
    protected static supportsUsername: boolean;
    protected static supportsDevhubUsername: boolean;
    /**
     * @function    run
     * @returns     {Promise<AnyJson>}  Resolves with a JSON object that the CLI
     *              will pass to the user as stdout if the --json flag was set.
     * @description Entrypoint function for "sfdx falcon:adk:create".
     * @public @async
     */
    run(): Promise<AnyJson>;
    /**
     * @method      buildFinalError
     * @param       {SfdxFalconError} cmdError  Required. Error object used as
     *              the basis for the "friendly error message" being created
     *              by this method.
     * @returns     {SfdxError}
     * @description Builds a user-friendly error message that is appropriate to
     *              the CLI command that's being implemented by this class. The
     *              output of this method will always be used by the onError()
     *              method from the base class to communicate the end-of-command
     *              error state.
     * @protected
     */
    protected buildFinalError(cmdError: SfdxFalconError): SfdxError;
}
