/**
 * @file          commands/falcon/adk/install.ts
 * @copyright     Vivek M. Chawla - 2018
 * @author        Vivek M. Chawla <@VivekMChawla>
 * @summary       Implements the CLI command "falcon:adk:install"
 * @description   Salesforce CLI Plugin command (falcon:adk:install) that is expected to run inside
 *                of a fully-configured AppExchange Demo Kit (ADK) project.  Takes project and local
 *                settings from various JSON config files and uses them to power an Org Build
 *                based on the SFDX Falcon Recipe selected by the user.
 * @version       1.0.0
 * @license       MIT
 */
import { flags } from '@salesforce/command';
import { SfdxError } from '@salesforce/core';
import { AnyJson } from '@salesforce/ts-types';
import { SfdxFalconCommand } from '../../../modules/sfdx-falcon-command';
import { SfdxFalconError } from '../../../modules/sfdx-falcon-error';
/**
 * @class       FalconDemoInstall
 * @extends     SfdxFalconCommand
 * @summary     Implements the CLI Command "falcon:adk:install"
 * @description Reads an SFDX-Falcon Recipe (either the one specified as the project default inside
 *              sfdx-project.json or one specified at the command line) and uses the resulting
 *              compiled Recipe to perform a set of tasks that should result in a demo org being
 *              built to the specifications of the SFDX Falcon Recipe.
 * @version     1.0.0
 * @public
 */
export default class FalconDemoInstall extends SfdxFalconCommand {
    static description: string;
    static hidden: boolean;
    static examples: string[];
    protected static flagsConfig: {
        falcondebug: flags.Discriminated<flags.Array<string>>;
        falcondebugerror: flags.Discriminated<flags.Boolean<boolean>>;
        falcondebugsuccess: flags.Discriminated<flags.Boolean<boolean>>;
        falcondebugdepth: flags.Discriminated<flags.Number>;
        projectdir: flags.Discriminated<flags.Option<string>>;
        recipefile: flags.Discriminated<flags.Option<string>>;
        extendedoptions: flags.Discriminated<flags.Option<string>>;
    };
    protected static requiresProject: boolean;
    protected static requiresUsername: boolean;
    protected static requiresDevhubUsername: boolean;
    protected static supportsUsername: boolean;
    protected static supportsDevhubUsername: boolean;
    /**
     * @function    run
     * @returns     {Promise<AnyJson>}  This should resolve by returning a JSON object
     *              that the CLI will then forward to the user if the --json flag
     *              was set when this command was called.
     * @description Entrypoint function used by the CLI when the user wants to
     *              run the command 'sfdx falcon:adk:install'.
     * @version     1.0.0
     * @public @async
     */
    run(): Promise<AnyJson>;
    /**
     * @method      buildFinalError
     * @param       {SfdxFalconError} cmdError  Required. Error object used as
     *              the basis for the "friendly error message" being created
     *              by this method.
     * @returns     {SfdxError}
     * @description Builds a user-friendly error message that is appropriate to
     *              the CLI command that's being implemented by this class. The
     *              output of this method will always be used by the onError()
     *              method from the base class to communicate the end-of-command
     *              error state.
     * @protected
     */
    protected buildFinalError(cmdError: SfdxFalconError): SfdxError;
}
