"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
//─────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @file          commands/falcon/adk/install.ts
 * @copyright     Vivek M. Chawla - 2018
 * @author        Vivek M. Chawla <@VivekMChawla>
 * @summary       Implements the CLI command "falcon:adk:install"
 * @description   Salesforce CLI Plugin command (falcon:adk:install) that is expected to run inside
 *                of a fully-configured AppExchange Demo Kit (ADK) project.  Takes project and local
 *                settings from various JSON config files and uses them to power an Org Build
 *                based on the SFDX Falcon Recipe selected by the user.
 * @version       1.0.0
 * @license       MIT
 */
//─────────────────────────────────────────────────────────────────────────────────────────────────┘
// Import External Modules
const command_1 = require("@salesforce/command"); // Allows creation of flags for CLI commands.
const core_1 = require("@salesforce/core"); // Generalized SFDX error which also contains an action.
const core_2 = require("@salesforce/core"); // Messages library that simplifies using external JSON for string reuse.
const lodash_1 = require("lodash"); // Useful function for detecting empty objects.
const path = require("path"); // Helps resolve local paths at runtime.
// Import Local Modules
const sfdx_falcon_command_1 = require("../../../modules/sfdx-falcon-command"); // Class. Base class used by all SFDX-Falcon commands.
const sfdx_falcon_project_1 = require("../../../modules/sfdx-falcon-project"); // Class. Represents an SFDX-Falcon project, including locally stored project data.
// Import Internal Types
const sfdx_falcon_command_2 = require("../../../modules/sfdx-falcon-command"); // Enum. Represents the types of SFDX-Falcon Commands.
// Set the File Local Debug Namespace
//const dbgNs     = 'COMMAND:falcon-demo-install:';
// Use SfdxCore's Messages framework to get the message bundles for this command.
core_2.Messages.importMessagesDirectory(__dirname);
const baseMessages = core_2.Messages.loadMessages('sfdx-falcon', 'sfdxFalconCommand');
const commandMessages = core_2.Messages.loadMessages('sfdx-falcon', 'falconAdkInstall');
//─────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @class       FalconDemoInstall
 * @extends     SfdxFalconCommand
 * @summary     Implements the CLI Command "falcon:adk:install"
 * @description Reads an SFDX-Falcon Recipe (either the one specified as the project default inside
 *              sfdx-project.json or one specified at the command line) and uses the resulting
 *              compiled Recipe to perform a set of tasks that should result in a demo org being
 *              built to the specifications of the SFDX Falcon Recipe.
 * @version     1.0.0
 * @public
 */
//─────────────────────────────────────────────────────────────────────────────────────────────────┘
class FalconDemoInstall extends sfdx_falcon_command_1.SfdxFalconCommand {
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @function    run
     * @returns     {Promise<AnyJson>}  This should resolve by returning a JSON object
     *              that the CLI will then forward to the user if the --json flag
     *              was set when this command was called.
     * @description Entrypoint function used by the CLI when the user wants to
     *              run the command 'sfdx falcon:adk:install'.
     * @version     1.0.0
     * @public @async
     */
    //───────────────────────────────────────────────────────────────────────────┘
    async run() {
        // Initialize the SfdxFalconCommand base (DO NOT REMOVE THIS LINE OF CODE!)
        this.sfdxFalconCommandInit('falcon:adk:install', sfdx_falcon_command_2.SfdxFalconCommandType.APPX_DEMO);
        // Resolve the Project Directory (specified by the user) to a Project Path.
        const projectPath = path.resolve(this.projectDirectory);
        // Instantiate the SFDX-Falcon Project residing at the Project Path.
        const sfdxFalconProject = await sfdx_falcon_project_1.SfdxFalconProject.resolve(projectPath);
        // If the user passed any Extended Options, use them as Compile Options.
        const compileOptions = this.extendedOptions;
        if (this.recipeFile) {
            // Run the specific Recipe supplied by the user.
            await sfdxFalconProject.runSpecifiedRecipe(this.recipeFile)
                .then(async (falconRecipeResult) => { await this.onSuccess(falconRecipeResult); }) // Implemented by parent class
                .catch(async (falconRecipeResult) => { await this.onError(falconRecipeResult); }); // Implemented by parent class
        }
        else {
            // Run the Default Recipe as specified in sfdx-project.json.
            await sfdxFalconProject.runDefaultRecipe(compileOptions)
                .then(async (falconRecipeResult) => { await this.onSuccess(falconRecipeResult); }) // Implemented by parent class
                .catch(async (falconRecipeResult) => { await this.onError(falconRecipeResult); }); // Implemented by parent class
        }
        // Return the JSON Response that was populated by onSuccess().
        return this.falconJsonResponse;
    }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      buildFinalError
     * @param       {SfdxFalconError} cmdError  Required. Error object used as
     *              the basis for the "friendly error message" being created
     *              by this method.
     * @returns     {SfdxError}
     * @description Builds a user-friendly error message that is appropriate to
     *              the CLI command that's being implemented by this class. The
     *              output of this method will always be used by the onError()
     *              method from the base class to communicate the end-of-command
     *              error state.
     * @protected
     */
    //───────────────────────────────────────────────────────────────────────────┘
    buildFinalError(cmdError) {
        let actionError = cmdError;
        // Find the first Error Object that has an associated ACTION Result.
        while (actionError && (lodash_1.isEmpty(actionError) === false)) {
            if (actionError.data && actionError.data.sfdxFalconResult) {
                if (actionError.data.sfdxFalconResult.type === "ACTION" /* ACTION */) {
                    // Found what we were looking for.
                    const actionResult = actionError.data.sfdxFalconResult;
                    const actionDetail = actionResult.detail;
                    let errorMessage = '';
                    try {
                        const stepName = actionDetail.actionContext.listrExecOptions.listrTask._task.title;
                        errorMessage = `The step "${stepName}" has failed. Its action, "${actionDetail.actionName}", returned the following error:`;
                    }
                    catch (err) {
                        errorMessage = `A step with the action "${actionDetail.actionName}" has failed with the following error:`;
                    }
                    // Build the FINAL error that we'll throw to the CLI.
                    const finalError = new core_1.SfdxError(`${errorMessage}\n`
                        + `${actionError.rootCause.name}: ${actionError.rootCause.message}`, `FailedCommand`, actionError.rootCause.actions, 1, cmdError);
                    finalError.commandName = `falcon:adk:install`;
                    return finalError;
                }
            }
            // Get the next child error object.
            actionError = actionError.cause;
        }
        // We could not find an ACTION error. Just reflect the COMMAND error.
        return cmdError;
    }
} // End of Class FalconDemoInstall
// Define the basic properties of this CLI command.
FalconDemoInstall.description = commandMessages.getMessage('commandDescription');
FalconDemoInstall.hidden = false;
FalconDemoInstall.examples = [
    `$ sfdx falcon:adk:install`,
    `$ sfdx falcon:adk:install --projectdir ~/demos/adk-projects/my-adk-project`,
    `$ sfdx falcon:adk:install --projectdir ~/demos/adk-projects/my-adk-project \\\n` +
        `                         --configfile my-alternate-demo-config.json`
];
//───────────────────────────────────────────────────────────────────────────┐
// -d --PROJECTDIR  Directory where a fully configured AppX Demo Kit (ADK)
//                  project exists. All commands for deployment must be
//                  defined inside this directory.
// -f --CONFIGFILE  Name of the config file to override the normal demo
//                  install process with.
//───────────────────────────────────────────────────────────────────────────┘
FalconDemoInstall.flagsConfig = Object.assign({ projectdir: command_1.flags.directory({
        char: 'd',
        required: false,
        description: baseMessages.getMessage('projectdir_FlagDescription'),
        default: '.',
        hidden: false
    }), recipefile: command_1.flags.string({
        char: 'f',
        required: false,
        description: baseMessages.getMessage('recipefile_FlagDescription'),
        default: '',
        hidden: false
    }), extendedoptions: command_1.flags.string({
        char: 'x',
        required: false,
        description: baseMessages.getMessage('extendedoptions_FlagDescription'),
        default: '{}',
        hidden: false
    }) }, sfdx_falcon_command_1.SfdxFalconCommand.falconBaseflagsConfig);
// Identify the core SFDX arguments/features required by this command.
FalconDemoInstall.requiresProject = false; // True if an SFDX Project workspace is REQUIRED.
FalconDemoInstall.requiresUsername = false; // True if an org username is REQUIRED.
FalconDemoInstall.requiresDevhubUsername = false; // True if a hub org username is REQUIRED.
FalconDemoInstall.supportsUsername = false; // True if an org username is OPTIONAL.
FalconDemoInstall.supportsDevhubUsername = false; // True if a hub org username is OPTIONAL.
exports.default = FalconDemoInstall;
//# sourceMappingURL=install.js.map