"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
//─────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @file          commands/falcon/apk/create.ts
 * @copyright     Vivek M. Chawla - 2018
 * @author        Vivek M. Chawla <@VivekMChawla>
 * @summary       Implements the CLI command "falcon:apk:create"
 * @description   Salesforce CLI Plugin command (falcon:apk:create) that allows a Salesforce DX
 *                developer to create an empty project based on the  SFDX-Falcon template.  Before
 *                the project is created, the user is guided through an interview where they define
 *                key project settings which are then used to customize the project scaffolding
 *                that gets created on their local machine.
 * @version       1.0.0
 * @license       MIT
 */
//─────────────────────────────────────────────────────────────────────────────────────────────────┘
// Import External Modules
const command_1 = require("@salesforce/command"); // Allows creation of flags for CLI commands.
const core_1 = require("@salesforce/core"); // Messages library that simplifies using external JSON for string reuse.
const sfdx_falcon_yeoman_command_1 = require("../../../modules/sfdx-falcon-yeoman-command"); // Base class that CLI commands in this project that use Yeoman should use.
// Import Internal Types
const sfdx_falcon_command_1 = require("../../../modules/sfdx-falcon-command"); // Enum. Represents the types of SFDX-Falcon Commands.
// Set the File Local Debug Namespace
//const dbgNs     = 'COMMAND:falcon-apk-create:';
// Use SfdxCore's Messages framework to get the message bundle for this command.
core_1.Messages.importMessagesDirectory(__dirname);
const commandMessages = core_1.Messages.loadMessages('sfdx-falcon', 'falconApkCreate');
//─────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @class       FalconApkCreate
 * @extends     SfdxFalconYeomanCommand
 * @summary     Implements the CLI Command "falcon:apk:create"
 * @description The command "falcon:apk:create" creates a local SFDX project using the
 *              scaffolding found at https://github.com/sfdx-isv/sfdx-falcon-appx-package-kit.
 *              Uses Yeoman to create customized project scaffolding on the user's machine.
 * @public
 */
//─────────────────────────────────────────────────────────────────────────────────────────────────┘
class FalconApkCreate extends sfdx_falcon_yeoman_command_1.SfdxFalconYeomanCommand {
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @function    run
     * @returns     {Promise<AnyJson>}  Resolves with a JSON object that the CLI
     *              will pass to the user as stdout if the --json flag was set.
     * @description Entrypoint function for "sfdx falcon:apk:create".
     * @public @async
     */
    //───────────────────────────────────────────────────────────────────────────┘
    async run() {
        // Initialize the SfdxFalconCommand (required by ALL classes that extend SfdxFalconCommand).
        this.sfdxFalconCommandInit('falcon:apk:create', sfdx_falcon_command_1.SfdxFalconCommandType.APPX_PACKAGE);
        // Run a Yeoman Generator to interact with and run tasks for the user.
        await super.runYeomanGenerator({
            generatorType: 'create-appx-package-project',
            outputDir: this.outputDirectory,
            options: []
        })
            .then(generatorResult => this.onSuccess(generatorResult)) // Implemented by parent class
            .catch(generatorResult => this.onError(generatorResult)); // Implemented by parent class
        // Return the JSON Response that was created by onSuccess()
        return this.falconJsonResponse;
    }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      buildFinalError
     * @param       {SfdxFalconError} cmdError  Required. Error object used as
     *              the basis for the "friendly error message" being created
     *              by this method.
     * @returns     {SfdxError}
     * @description Builds a user-friendly error message that is appropriate to
     *              the CLI command that's being implemented by this class. The
     *              output of this method will always be used by the onError()
     *              method from the base class to communicate the end-of-command
     *              error state.
     * @protected
     */
    //───────────────────────────────────────────────────────────────────────────┘
    buildFinalError(cmdError) {
        // If not implementing anything special here, simply return cmdError.
        return cmdError;
    }
}
// Define the basic properties of this CLI command.
FalconApkCreate.description = commandMessages.getMessage('commandDescription');
FalconApkCreate.hidden = false;
FalconApkCreate.examples = [
    `$ sfdx falcon:apk:create`,
    `$ sfdx falcon:apk:create --outputdir ~/projects/sfdx-falcon-projects`
];
//───────────────────────────────────────────────────────────────────────────┐
// Define the flags used by this command.
// -d --OUTPUTDIR   Directory where SFDX-Falcon project will be created.
//                  Defaults to . (current directory) if not specified.
//───────────────────────────────────────────────────────────────────────────┘
FalconApkCreate.flagsConfig = Object.assign({ outputdir: command_1.flags.directory({
        char: 'd',
        required: false,
        description: commandMessages.getMessage('outputdir_FlagDescription'),
        default: '.',
        hidden: false
    }) }, sfdx_falcon_yeoman_command_1.SfdxFalconYeomanCommand.falconBaseflagsConfig);
// Identify the core SFDX arguments/features required by this command.
FalconApkCreate.requiresProject = false; // True if an SFDX Project workspace is REQUIRED.
FalconApkCreate.requiresUsername = false; // True if an org username is REQUIRED.
FalconApkCreate.requiresDevhubUsername = false; // True if a hub org username is REQUIRED.
FalconApkCreate.supportsUsername = false; // True if an org username is OPTIONAL.
FalconApkCreate.supportsDevhubUsername = false; // True if a hub org username is OPTIONAL.
exports.default = FalconApkCreate;
//# sourceMappingURL=create.js.map