import { SfdxFalconInterview } from '../modules/sfdx-falcon-interview';
import { SfdxFalconTableData } from '../modules/sfdx-falcon-util/ux';
import { GeneratorOptions } from '../modules/sfdx-falcon-yeoman-command';
import { SfdxFalconYeomanGenerator } from '../modules/sfdx-falcon-yeoman-generator';
import { YeomanChoice } from '../modules/sfdx-falcon-types';
/**
 * @interface   InterviewAnswers
 * @description Represents answers to questions asked in the interview of THIS Yeoman Generator.
 */
interface InterviewAnswers {
    targetDirectory: string;
    devHubAlias: string;
    envHubAlias: string;
    devHubUsername: string;
    envHubUsername: string;
    gitRemoteUri: string;
    gitCloneDirectory: string;
}
/**
 * @class       CloneAppxDemoProject
 * @extends     SfdxFalconYeomanGenerator
 * @summary     Yeoman generator class. Used to clone an SFDX-Falcon Appx Demo project from a
 *              remote Git repo.
 * @description Uses Yeoman to clone an SFDX-Falcon Appx Demo project built using the ADK template.
 *              This class defines the entire Yeoman interview process, git cloning process, and
 *              file modification operations to create config files on the user's local machine.
 * @public
 */
export default class CloneAppxDemoProject extends SfdxFalconYeomanGenerator<InterviewAnswers> {
    protected devHubAliasChoices: YeomanChoice[];
    protected envHubAliasChoices: YeomanChoice[];
    protected gitRemoteUri: string;
    protected gitCloneDirectory: string;
    protected localProjectPath: string;
    /**
     * @constructs  CloneAppxDemoProject
     * @param       {string|string[]} args Required. Not used (as far as I know).
     * @param       {GeneratorOptions}  opts Required. Sets generator options.
     * @description Constructs a CloneAppxDemoProject object.
     * @public
     */
    constructor(args: string | string[], opts: GeneratorOptions);
    /**
     * @method      _buildInterview
     * @returns     {SfdxFalconInterview<InterviewAnswers>} Returns a fully fleshed
     *              SfdxFalconInterview object with zero or more prompts that the
     *              user will answer in an interview once this is run.
     * @description Allows the developer to build a complex, multi-step interview
     *              that Yeoman will execute during the "prompting" phase.
     * @protected
     */
    protected _buildInterview(): SfdxFalconInterview<InterviewAnswers>;
    /**
     * @method      _buildInterviewAnswersTableData
     * @param       {InterviewAnswers}  userAnswers Required.
     * @returns     {Promise<SfdxFalconTableData>}
     * @description Builds an SfdxFalconTableData object based on the Interview
     *              Answer values provided by the caller. This function can be
     *              used by an SfdxFalconInterview to reflect input to the user
     *              at the end of an Interview.
     * @protected @async
     */
    protected _buildInterviewAnswersTableData(interviewAnswers: InterviewAnswers): Promise<SfdxFalconTableData>;
    /**
     * @method      initializing
     * @returns     {Promise<void>}
     * @description STEP ONE in the Yeoman run-loop.  Uses Yeoman's "initializing"
     *              run-loop priority.
     * @protected @async
     */
    protected initializing(): Promise<void>;
    /**
     * @method      prompting
     * @returns     {Promise<void>}
     * @description STEP TWO in the Yeoman run-loop. Interviews the User to get
     *              information needed by the "writing" and "installing" phases.
     * @protected @async
     */
    protected prompting(): Promise<void>;
    /**
     * @method      configuring
     * @returns     {Promise<void>}
     * @description STEP THREE in the Yeoman run-loop. Perform any pre-install
     *              configuration steps based on the answers provided by the User.
     * @protected @async
     */
    protected configuring(): Promise<void>;
    /**
     * @method      writing
     * @returns     {Promise<void>}
     * @description STEP FOUR in the Yeoman run-loop. Typically, this is where
     *              you perform filesystem writes, git clone operations, etc.
     * @protected @async
     */
    protected writing(): Promise<void>;
    /**
     * @method      install
     * @returns     {void}
     * @description STEP FIVE in the Yeoman run-loop. Typically, this is where
     *              you perform operations that must happen AFTER files are
     *              written to disk. For example, if the "writing" step downloaded
     *              an app to install, the "install" step would run the
     *              installation.
     * @protected
     */
    protected install(): void;
    /**
     * @method      end
     * @returns     {void}
     * @description STEP SIX in the Yeoman run-loop. This is the FINAL step that
     *              Yeoman runs and it gives us a chance to do any post-Yeoman
     *              updates and/or cleanup.
     * @protected
     */
    protected end(): void;
}
export {};
