import { SfdxFalconInterview } from '../modules/sfdx-falcon-interview';
import { SfdxFalconTableData } from '../modules/sfdx-falcon-util/ux';
import { GeneratorOptions } from '../modules/sfdx-falcon-yeoman-command';
import { SfdxFalconYeomanGenerator } from '../modules/sfdx-falcon-yeoman-generator';
import { YeomanChoice } from '../modules/sfdx-falcon-types';
/**
 * @interface   InterviewAnswers
 * @description Represents answers to the questions asked in the Yeoman interview.
 */
interface InterviewAnswers {
    projectFamily: 'APK' | 'ADK';
    projectType: 'single-demo' | 'multi-demo';
    projectVersion: string;
    targetDirectory: string;
    developerName: string;
    developerAlias: string;
    projectName: string;
    projectAlias: string;
    packageDirectory: string;
    defaultRecipe: string;
    devHubAlias: string;
    envHubAlias: string;
    devHubUsername: string;
    envHubUsername: string;
    scratchDefOrgName: string;
    scratchDefDescription: string;
    isInitializingGit: boolean;
    hasGitRemote: boolean;
    isGitRemoteReachable: boolean;
    ackGitRemoteUnreachable: boolean;
    gitRemoteUri: string;
    gitHubUrl: string;
    schemaVersion: string;
    pluginVersion: string;
    sfdcApiVersion: string;
}
/**
 * @class       CreateAppxDemoProject
 * @extends     SfdxFalconYeomanGenerator
 * @summary     Yeoman generator class. Creates and configures a local AppX Demo Kit (ADK) project.
 * @description Uses Yeoman to create a local ADK project using the AppExchange Demo Kit Template.
 *              This class defines the entire Yeoman interview process and the file template copy
 *              operations needed to create the project scaffolding on the user's local machine.
 * @public
 */
export default class CreateAppxDemoProject extends SfdxFalconYeomanGenerator<InterviewAnswers> {
    protected devHubAliasChoices: YeomanChoice[];
    protected envHubAliasChoices: YeomanChoice[];
    protected sourceDirectory: string;
    /**
     * @constructs  CreateAppxDemoProject
     * @param       {string|string[]} args Required. Not used (as far as I know).
     * @param       {GeneratorOptions}  opts Required. Sets generator options.
     * @description Constructs a CreateAppxDemoProject object.
     * @public
     */
    constructor(args: string | string[], opts: GeneratorOptions);
    /**
     * @method      _buildInterview
     * @returns     {SfdxFalconInterview<InterviewAnswers>} Returns a fully fleshed
     *              SfdxFalconInterview object with zero or more prompts that the
     *              user will answer in an interview once this is run.
     * @description Allows the developer to build a complex, multi-step interview
     *              that Yeoman will execute during the "prompting" phase.
     * @protected
     */
    protected _buildInterview(): SfdxFalconInterview<InterviewAnswers>;
    /**
     * @method      _buildInterviewAnswersTableData
     * @param       {InterviewAnswers}  userAnswers Required.
     * @returns     {Promise<SfdxFalconTableData>}
     * @description Builds an SfdxFalconTableData object based on the Interview
     *              Answer values provided by the caller. This function can be
     *              used by an SfdxFalconInterview to reflect input to the user
     *              at the end of an Interview.
     * @protected
     */
    protected _buildInterviewAnswersTableData(interviewAnswers: InterviewAnswers): Promise<SfdxFalconTableData>;
    /**
     * @method      initializing
     * @returns     {Promise<void>}
     * @description STEP ONE in the Yeoman run-loop.  Uses Yeoman's "initializing"
     *              run-loop priority.
     * @protected @async
     */
    protected initializing(): Promise<void>;
    /**
     * @method      prompting
     * @returns     {Promise<void>}
     * @description STEP TWO in the Yeoman run-loop. Interviews the User to get
     *              information needed by the "writing" and "installing" phases.
     * @protected @async
     */
    protected prompting(): Promise<void>;
    /**
     * @method      configuring
     * @returns     {void}
     * @description STEP THREE in the Yeoman run-loop. Perform any pre-install
     *              configuration steps based on the answers provided by the User.
     * @protected
     */
    protected configuring(): void;
    /**
     * @method      writing
     * @returns     {void}
     * @description STEP FOUR in the Yeoman run-loop. Typically, this is where
     *              you perform filesystem writes, git clone operations, etc.
     * @protected
     */
    protected writing(): void;
    /**
     * @method      install
     * @returns     {void}
     * @description STEP FIVE in the Yeoman run-loop. Typically, this is where
     *              you perform operations that must happen AFTER files are
     *              written to disk. For example, if the "writing" step downloaded
     *              an app to install, the "install" step would run the
     *              installation.
     * @protected @async
     */
    protected install(): Promise<void>;
    /**
     * @method      end
     * @returns     {void}
     * @description STEP SIX in the Yeoman run-loop. This is the FINAL step that
     *              Yeoman runs and it gives us a chance to do any post-Yeoman
     *              updates and/or cleanup.
     * @protected
     */
    protected end(): void;
}
export {};
