/**
 * @file          modules/sfdx-falcon-command/index.ts
 * @copyright     Vivek M. Chawla - 2018
 * @author        Vivek M. Chawla <@VivekMChawla>
 * @summary       Exports SfdxFalconCommand for use with creating custom Salesforce CLI commands.
 * @description   ???
 * @version       1.0.0
 * @license       MIT
 */
import { flags } from '@salesforce/command';
import { SfdxCommand } from '@salesforce/command';
import { SfdxError } from '@salesforce/core';
import { SfdxFalconError } from '../sfdx-falcon-error';
import { SfdxFalconResult } from '../sfdx-falcon-result';
import { SfdxFalconJsonResponse } from '../sfdx-falcon-types';
/**
 * @enum        SfdxFalconCommandType
 * @description Defines the possible types of SFDX-Falcon Commands.
 */
export declare enum SfdxFalconCommandType {
    APPX_PACKAGE = "APPX_PACKAGE",
    APPX_DEMO = "APPX_DEMO",
    APPX_EXTENSION = "APPX_EXTENSION",
    UNKNOWN = "UNKNOWN"
}
/**
 * @interface   SfdxFalconCommandResultDetail
 * @description Model of the Detail object that should be attached to an SFDX-Falcon COMMAND Result.
 */
export interface SfdxFalconCommandResultDetail {
    commandName: string;
    commandType: SfdxFalconCommandType;
    commandFlags: any;
    commandArgs: any;
    commandExitCode: number;
    enabledDebuggers: string[];
}
/**
 * @abstract
 * @class       SfdxFalconCommand
 * @extends     SfdxCommand
 * @summary     Abstract base class class for building Salesforce CLI commands that use Yeoman.
 * @description Classes that extend SfdxYeomanCommand will be able to run any Generator defined
 *              in the src/generators directory.  The file name in src/generators should match the
 *              generatorType string passed into runYeomanGenerator().  For example, if
 *              generatorType==="my-generator", then there MUST be a TS script file located at
 *              src/generators/my-generator.ts.
 * @version     1.0.0
 * @public
 */
export declare abstract class SfdxFalconCommand extends SfdxCommand {
    protected static falconBaseflagsConfig: {
        falcondebug: flags.Discriminated<flags.Array<string>>;
        falcondebugerror: flags.Discriminated<flags.Boolean<boolean>>;
        falcondebugsuccess: flags.Discriminated<flags.Boolean<boolean>>;
        falcondebugdepth: flags.Discriminated<flags.Number>;
    };
    protected falconCommandName: string;
    protected falconCommandResult: SfdxFalconResult;
    protected falconCommandResultDetail: SfdxFalconCommandResultDetail;
    protected falconJsonResponse: SfdxFalconJsonResponse;
    protected outputDirectory: string;
    protected projectDirectory: string;
    protected targetDirectory: string;
    protected recipeFile: string;
    protected configFile: string;
    protected extendedOptions: {
        any: any;
    };
    protected gitRemoteUri: string;
    protected gitCloneDirectory: string;
    protected falconDebugFlag: string[];
    protected falconDebugErrorFlag: boolean;
    protected falconDebugSuccessFlag: boolean;
    protected falconDebugDepthFlag: number;
    protected abstract buildFinalError(cmdError: SfdxFalconError): SfdxError;
    /**
     * @function    sfdxFalconCommandInit
     * @param       {string}  commandName Required. ???
     * @param       {SfdxFalconCommandType} commandType Required. ???
     * @returns     {void}
     * @description Initializes various SfdxFalconCommand structures.
     * @version     1.0.0
     * @protected
     */
    protected sfdxFalconCommandInit(commandName: string, commandType: SfdxFalconCommandType): void;
    /**
     * @function    onError
     * @param       {any}   rejectedPromise Required.
     * @param       {boolean} [showErrorDebug]  Optional. Determines if extended
     *              debugging output the Error Result can be shown.
     * @param       {boolean} [promptUser] Optional. Determines if the user will
     *              be prompted to display debug info. If FALSE, debug info will
     *              be shown without requiring additional user input.
     * @returns     {Promise<void>}
     * @description Recieves the results from a Rejected Promise and processes
     *              them to settle out the ultimate exit status of this
     *              COMMAND Result.
     * @version     1.0.0
     * @protected
     */
    protected onError(rejectedPromise: unknown, showErrorDebug?: boolean, promptUser?: boolean): Promise<void>;
    /**
     * @function    onSuccess
     * @param       {any}  resolvedPromise Required.
     * @returns     {Promise<void>}
     * @description Takes any resolved Promise which should be returned by some
     *              sort of Asynchronous call (implemented in a derived class)
     *              that does whatever "work" the CLI Command is meant to do and
     *              makes sure it's wrapped as an SFDX Falcon Result
     * @version     1.0.0
     * @protected
     */
    protected onSuccess(resolvedPromise: unknown): Promise<void>;
    /**
     * @method      terminateWithError
     * @param       {boolean} [showErrorDebug]  Optional. Determines if extended
     *              debugging output for the terminating Error can be shown.
     * @param       {boolean} [promptUser] Optional. Determines if the user will
     *              be prompted to display debug info. If FALSE, debug info will
     *              be shown without requiring additional user input.
     * @description Kills all ongoing async code (ie. Progress Notifications) and
     *              possibly renders an Error Debug before throwing an SfdxError
     *              so that the CLI can present user-friendly error info.
     * @private
     */
    private terminateWithError;
}
