/**
 * @file          modules/sfdx-falcon-error/index.ts
 * @copyright     Vivek M. Chawla - 2018
 * @author        Vivek M. Chawla <@VivekMChawla>
 * @summary       Provides specialized error structures for SFDX-Falcon modules.
 * @description   Provides specialized error structures for SFDX-Falcon modules.  Wraps SfdxError
 *                by adding additional SFDX-Falcon specific stack information as well as customized
 *                rendering capabilities to show formatted output via the console.
 * @version       1.0.0
 * @license       MIT
 */
import { SfdxError } from '@salesforce/core';
/**
 * @interface   CliErrorDetail
 * @description Data structure returned by Salesforce CLI calls made with --json flag set.
 */
export interface CliErrorDetail {
    command: string;
    status: number;
    result: unknown;
    name?: string;
    message?: string;
    actions?: string[];
    stack?: string;
    warnings?: unknown;
}
/**
 * @interface   SfdxFalconErrorRenderOptions
 * @description Options object used by the various Render functions to customize display output.
 */
export interface SfdxFalconErrorRenderOptions {
    headerColor: string;
    labelColor: string;
    errorLabelColor: string;
    valueColor: string;
    childInspectDepth: number;
    detailInspectDepth: number;
    errorInspectDepth: number;
}
/**
 * @interface   ShellErrorDetail
 * @description Represents information available after the failed execution of any shell command.
 */
export interface ShellErrorDetail {
    code: number;
    command: string;
    message: string;
    signal: string;
    stderr: string;
    stdout: string;
}
/**
 * @class       SfdxFalconError
 * @extends     SfdxError
 * @description Extends SfdxError to provide specialized error structures for SFDX-Falcon modules.
 * @public
 */
export declare class SfdxFalconError extends SfdxError {
    /**
     * @method      renderError
     * @param       {Error}   errorToRender  Required. Any object that is a child
     *              of Error.
     * @param       {number}  [childInspectDepth] Optional. Sets how deep the
     *              object inpsection goes when rendering "child" objects.
     * @param       {number}  [detailInspectDepth]  Optional. Sets how deep the
     *              object inpsection goes when rendering "detail" objects.
     * @param       {number}  [errorInspectDepth] Optional. Sets how deep the
     *              object inpsection goes when rendering "error" objects.
     * @returns     {string}
     * @description Generates a string of completely formatted output that's ready
     *              for display to the user via console.log() or debug(). Relies
     *              on the caller to decide how to actually display to the user.
     * @public @static
     */
    static renderError(errorToRender: Error, childInspectDepth?: number, detailInspectDepth?: number, errorInspectDepth?: number): string;
    /**
     * @method      wrap
     * @param       {Error|unknown} error  Required. The Error object to wrap. If
     *              something other than an Error object is provided, its data
     *              will be attached to a new SfdxFalconError object.
     * @param       {string}  source  Required. Identifies the source of the
     *              error, typically a reference to the debug address of the
     *              code where this is being called from.
     * @description Given an instance of Error, wraps it as SFDX-Falcon Error and
     *              returns the result.
     * @public @static
     */
    static wrap(error: Error | unknown, source?: string): SfdxFalconError;
    /**
     * @method      renderBaseDetail
     * @param       {SfdxFalconError} errorToRender  Required. Should be an
     *              SfdxFalconError, but could be any instanceof Error.
     * @param       {SfdxFalconErrorRenderOptions}  options  Required. Rendering
     *              options that determine colors and inspection depth.
     * @returns     {string}
     * @description Generates the baseline set of completely formatted output
     *              that is relevant to ALL Errors.
     * @private @static
     */
    private static renderBaseDetail;
    /**
     * @method      renderSfdxCliErrorDetail
     * @param       {SfdxCliError}  errorToRender  Required. Any object that is
     *              a child of Error.
     * @param       {SfdxFalconErrorRenderOptions}  options  Required. Rendering
     *              options that determine colors and inspection depth.
     * @returns     {string}
     * @description Generates an extended set of completely formatted output
     *              that is relevant only to SfdxCliError objects.
     * @private @static
     */
    private static renderSfdxCliErrorDetail;
    /**
     * @method      renderShellErrorDetail
     * @param       {ShellError}  errorToRender  Required. Any object that is
     *              a child of Error.
     * @param       {SfdxFalconErrorRenderOptions}  options  Required. Rendering
     *              options that determine colors and inspection depth.
     * @returns     {string}
     * @description Generates an extended set of completely formatted output
     *              that is relevant only to ShellError objects.
     * @private @static
     */
    private static renderShellErrorDetail;
    /**
     * @method      renderUnknownDetail
     * @param       {unknown} unknownObject  Required. Will be generally rendered.
     * @param       {SfdxFalconErrorRenderOptions}  options  Required. Rendering
     *              options that determine colors and inspection depth.
     * @returns     {string}
     * @description Given an unknown object, will render it in a general way.
     * @private @static
     */
    private static renderUnknownDetail;
    readonly source: string;
    private _resultStack;
    private _detail;
    private _userInfo;
    private _debugInfo;
    private _devInfo;
    /**
     * @constructs  SfdxFalconError
     * @param       {string}  message Required. Message for the error.
     * @param       {string}  [name]  Optional. Defaults to SfdxFalconError.
     * @param       {string}  [source]  Optional. Defaults to UNKNOWN.
     * @param       {Error}   [cause] Optional. Error object causing this error.
     * @param       {string[]}  [actions] Optional. Array of action messages.
     * @param       {number}  [exitCode]  Optional. Code passed to the CLI.
     * @description Extension of the SfdxError object. Adds special SFDX-Falcon
     *              specific stack and detail properties.
     * @public
     */
    constructor(message: string, name?: string, source?: string, cause?: Error, actions?: string[], exitCode?: number);
    /**
     * @method      addToStack
     * @param       {string}  stackItem Required. Custom message to add to stack.
     * @returns     {void}
     * @description Given a message string, adds that message to a custom "stack"
     *              that we can use to give a history of the internal actions that
     *              SFDX-Falcon code is taking.
     * @public
     */
    addToStack(stackItem: string): void;
    /**
     * @property    detail
     * @returns     {unknown}
     * @description Gets the current Detail object.
     * @public
     */
    readonly detail: unknown;
    /**
     * @property    debugInfo
     * @returns     {SfdxFalconErrorInfo}
     * @description Gets the current Debug Information object.
     * @public
     */
    readonly debugInfo: SfdxFalconErrorInfo;
    /**
     * @property    devInfo
     * @returns     {SfdxFalconErrorInfo}
     * @description Gets the current Developer Information object.
     * @public
     */
    readonly devInfo: SfdxFalconErrorInfo;
    /**
     * @property    resultStack
     * @returns     {string}
     * @description Gets the current Falcon Result Stack.
     * @public
     */
    readonly resultStack: string;
    /**
     * @property    rootCause
     * @returns     {SfdxFalconError}
     * @description Gets the "root" Error objec in the cause chain of this Error.
     * @public
     */
    readonly rootCause: SfdxFalconError;
    /**
     * @property    userInfo
     * @returns     {SfdxFalconErrorInfo}
     * @description Gets the current User Information object.
     * @public
     */
    readonly userInfo: SfdxFalconErrorInfo;
    /**
     * @method      setDetail
     * @param       {unknown} detail  Required. Detail to be set to this Error.
     * @returns     {this}  Returns this to allow for chaining.
     * @description Additional detail related to this object, provided in addition
     *              to what might be attached to the SfdxError.data property.
     * @public
     */
    setDetail(detail: unknown): this;
}
/**
 * @class       SfdxFalconErrorInfo
 * @description Stores detailed error information & messages for display to multiple user personas.
 * @public
 */
export declare class SfdxFalconErrorInfo {
    title: string;
    message: string;
    actions: string[];
    /**
     * @constructs  SfdxFalconErrorInfo
     * @description Stores detailed error information & messages for display to
     *              multiple user personas.
     * @public
     */
    constructor();
    /**
     * @method      addAction
     * @param       {string}  actionItem Required. Action that the user can take
     *              to recover from whatever caused this error.
     * @returns     {void}
     * @description Adds a new string to the "actions" array.
     * @public
     */
    addAction(actionItem?: string): void;
}
/**
 * @class       SfdxCliError
 * @extends     SfdxFalconError
 * @description Extends SfdxFalconError to provide specialized error handling of error results
 *              returned from CLI commands run via shell exec.
 * @public
 */
export declare class SfdxCliError extends SfdxFalconError {
    cliError: CliErrorDetail;
    /**
     * @constructs  SfdxCliError
     * @param       {string}  cliCommandString  Command string that was executed.
     * @param       {string}  cliResponseBuffer  Required. Response from a failed
     *              CLI command. Typically a string containing parseable JSON, but
     *              may be an unparseable string response.
     * @param       {string}  [message] Optional. Sets the SfdxFalconError message.
     * @param       {string}  [source]  Optional. Sets the SfdxFalconError source.
     * @description Given a string (typically the contents of a stderr buffer),
     *              returns an SfdxFalconError object with a specialized
     *              "cliError" object property.
     * @public
     */
    constructor(sfdxCommandString: string, cliResponseBuffer: string, message?: string, source?: string);
}
/**
 * @class       ShellError
 * @extends     SfdxFalconError
 * @description Extends SfdxFalconError to provide specialized error handling of error results
 *              returned by failed shell commands which may or may not provide a JSON structure as
 *              part of their error message.
 * @public
 */
export declare class ShellError extends SfdxFalconError {
    shellError: ShellErrorDetail;
    /**
     * @constructs  ShellError
     * @param       {string}  command Required. The shell command whose execution
     *              resulted in this ShellError.
     * @param       {number}  code  Required. Exit code provided by the Shell.
     *              If NULL, then signal must have a value.
     * @param       {string}  signal  Required. Signal which caused the Shell to
     *              terminate. If NULL, then code must have a value.
     * @param       {string}  stdErrBuffer  Required. Contents of stderr when the
     *              shell was terminated.
     * @param       {string}  [stdOutBuffer]  Optional. Contents of stdout when the
     *              shell was terminated.
     * @param       {string}  [source]  Optional. Sets the SfdxFalconError source.
     * @param       {string}  [message] Optional. Message that the caller would
     *              like the user to see. If not provided, will default to the
     *              contents of stderr.
     * @description Returns an SfdxFalconError object with a specialized set of
     *              information in the "shellError" object property.
     * @public
     */
    constructor(command: string, code: number, signal: string, stdErrBuffer: string, stdOutBuffer?: string, source?: string, message?: string);
}
