/**
 * @file          modules/sfdx-falcon-interview/index.ts
 * @copyright     Vivek M. Chawla - 2019
 * @author        Vivek M. Chawla <@VivekMChawla>
 * @summary       Exports SfdxFalconInterview which provides complex interactions via the console.
 * @description   Helps developers quickly build complex Inquirer based interviews as a collection
 *                of SfdxFalconPrompt objects. Can export interviews to external consumers (like
 *                Yeoman), or directly run Inquirer-based interactions.
 * @version       1.0.0
 * @license       MIT
 */
import { SfdxFalconPrompt } from '../sfdx-falcon-prompt';
import { AbortInterview } from '../sfdx-falcon-types';
import { InterviewGroupOptions } from '../sfdx-falcon-types';
import { InterviewOptions } from '../sfdx-falcon-types';
import { InterviewStatus } from '../sfdx-falcon-types';
import { ShowInterviewGroup } from '../sfdx-falcon-types';
/**
 * @class       InterviewGroup
 * @summary     Represents the combination of an SFDX-Falcon Prompt with an "abort check" function.
 * @description Objects created from this class are able to be run through an Interview, one at a
 *              time, and have their "abort check" function executed once the user indicates they
 *              are ready to proceed.  This way, if the user provided data that is invalid, the
 *              entire interview can be aborted.
 * @public
 */
export declare class InterviewGroup<T extends object> {
    readonly title: string;
    readonly abort: AbortInterview;
    when: ShowInterviewGroup;
    private readonly falconPrompt;
    /**
     * @constructs  InterviewGroup
     * @param       {SfdxFalconPrompt<T>} falconPrompt Required.
     * @param       {AbortInterview}  [abort] Optional.
     * @param       {ShowInterviewGroup}  [when] Optional.
     * @description Constructs an InterviewGroup object.
     * @public
     */
    constructor(falconPrompt: SfdxFalconPrompt<T>, abort?: AbortInterview, when?: ShowInterviewGroup, title?: string);
    /**
     * @method      prompt
     * @returns     {Promise<T>}  Returns the results of inquirer.prompt()
     * @description Executes the Inquirer prompt() function that lives inside of
     *              this SfdxFalconPrompt object.
     * @public @async
     */
    prompt(): Promise<T>;
}
/**
 * @class       SfdxFalconInterview
 * @summary     Provides a standard way of building a multi-group Interview to collect user input.
 * @description Uses Inquirer to prompt the user in the terminal using a flexible, multi-group set
 *              of prompts.  Each group can have it's own set of "abort" questions, and may be shown
 *              or hidden independently. The Interview ends with a final "proceed/restart/abort"
 *              question for the user.
 * @public
 */
export declare class SfdxFalconInterview<T extends object> {
    readonly context: object;
    readonly sharedData: object;
    readonly defaultAnswers: T;
    status: InterviewStatus;
    userAnswers: T;
    when: ShowInterviewGroup;
    private readonly _interviewGroups;
    private readonly _confirmation;
    private readonly _confirmationHeader;
    private readonly _display;
    private readonly _displayHeader;
    private readonly _invertConfirmation;
    readonly finalAnswers: T;
    /**
     * @constructs  SfdxFalconInterview
     * @param       {InterviewOptions<T>} opts Required. Options that define the
     *              default answers, Prompt Engine, and special "context" object
     *              for the entire inteview. These values will be carried on to
     *              each SfdxFalconPrompt group contained in this Interview.
     * @description Constructs an SfdxFalconInterview object.
     * @public
     */
    constructor(opts: InterviewOptions<T>);
    /**
     * @method      createGroup
     * @param       {InterviewGroupOptions} opts  Required.
     * @returns     {void}
     * @description Given valid Interview Group Options, creates an SFDX-Falcon
     *              Prompt and then combines it with an "abort" function (if
     *              provided) in order to create an Interview Group.  The Interview
     *              Group is then added to the Interview Groups collection.
     * @public
     */
    createGroup(opts: InterviewGroupOptions<T>): void;
    /**
     * @method      start
     * @returns     {Promise<T>}  Returns the answers provided by the user.
     * @description Starts the interview. Once started, the Interview will only stop
     *              once completed, aborted by the user, or an Error is thrown.
     * @public @async
     */
    start(): Promise<T>;
    /**
     * @method      abortInterview
     * @param       {string}
     * @returns     {T} Returns the Final Answers for this interview.
     * @description Marks the status of this Interview as "Aborted" and "Incomplete",
     *              then returns the interview's "Final Answers" to the caller.
     * @private
     */
    private abortInterview;
    /**
     * @method      displayAnswers
     * @returns     {Promise<void>}
     * @description Uses the Display function (if present) to display the User
     *              Answers from the prompt to the user. If the Display function
     *              returns void, it means that it rendered output to the user.
     *              If it returns an Array, it means that we need to manually
     *              render the returned data in a FalconTable.
     * @private @async
     */
    private displayAnswers;
    /**
     * @method      proceedRestartAbort
     * @returns     {Promise{T}}
     * @description Propmts the user with a special "final" prompt that needs to
     *              return a Confirmation Answers (proceed, restart) structure.
     *              If the user indicates a desire to "proceed", this function will
     *              return the Final Answers. If they want to "restart", this
     *              function will recursively call this.start(). If they don't want
     *              to proceed and they don't want to restart, the interview will
     *              be aborted.
     * @private @async
     */
    private proceedRestartAbort;
    /**
     * @method      skipInterviewGroup
     * @param       {InterviewGroup<T>} interviewGroup  Required.
     * @returns     {Promise<boolean>}
     * @description Given an Interview Group, determines if that group should be
     *              skipped based on the result of the "when" member of the group.
     *              If the "when" member is a function, it's executed with the
     *              expectation of getting back a boolean. If it's a simple boolean,
     *              that value is returned. If there is no "when" member, the value
     *              returned defaults to FALSE, ensuring the group isn't skipped.
     * @private @async
     */
    private skipInterviewGroup;
}
