import { JsonMap } from '@salesforce/ts-types';
import { SfdxFalconResult } from '../sfdx-falcon-result';
import { ProjectResolutionOptions } from '../sfdx-falcon-types';
import { SfdxFalconProjectConfig } from '../sfdx-falcon-types';
import { SfdxFalconLocalConfig } from '../sfdx-falcon-types';
import { SfdxFalconGlobalConfig } from '../sfdx-falcon-types';
/**
 * @class       SfdxFalconProject
 * @summary     Represents a resolved SFDX-Falcon project configuration.
 * @description Implements a number of tools and services for resolving and inspecting a variety of
 *              JSON configuration files that can be found in an SFDX-Falcon project.
 * @public
 */
export declare class SfdxFalconProject {
    /**
     * @method      resolve
     * @param       {string}  projectPath  Required. Path to a valid SFDX-Falcon
     *              project inside the user's local environment.
     * @returns     {Promise<SfdxFalconProject>} Resolves with an instance
     *              of an SfdxFalconProject object that represents the
     *              project found at the projectRoot using a combination of SFDX
     *              and SFDX-Falcon config files.
     * @description Given the path to an SFDX-Falcon project directory,
     *              initializes an SfdxFalconProject object and returns
     *              it to the caller.
     * @public @static @async
     */
    static resolve(projectPath: string, opts?: ProjectResolutionOptions): Promise<SfdxFalconProject>;
    /**
     * @method      resolveAggregatedSfdxConfig
     * @param       {string}  projectPath  Required. ???
     * @returns     {Promise<JsonMap>} ???
     * @description ???
     * @private @static @async
     */
    private static resolveAggregatedSfdxConfig;
    /**
     * @method      resolveGlobalSfdxFalconConfig
     * @returns     {Promise<SfdxFalconGlobalConfig>}  ???
     * @description ???
     * @private @static @async
     */
    private static resolveGlobalSfdxFalconConfig;
    /**
     * @method      resolveLocalSfdxFalconConfig
     * @param       {string}  projectPath ???
     * @returns     {Promise<SfdxFalconLocalConfig>}  ???
     * @description ???
     * @private @static @async
     */
    private static resolveSfdxFalconLocalConfig;
    /**
     * @method      initializeConfig
     * @param       {string}  projectPath Required. ???
     * @param       {SfdxFalconProject} sfdxFPO Required. ???
     * @param       {ProjectResolutionOptions}  opts  Required. ???
     * @returns     {Promise<void>} ???
     * @description Resolves various config files and saves copies of all relevant
     *              configuration JSON inside the given SFDX-Falcon Project Object.
     * @private @static @async
     */
    private static initializeConfig;
    /**
     * @method      initializePaths
     * @param       {string}  projectPath  Required. ???
     * @param       {SfdxFalconProject} sfdxFPO  Required. ???
     * @returns     {Promise<void>} ???
     * @description Initializes all of the path variables inside the given SFDX-
     *              Falcon Project Object.
     * @private @static @async
     */
    private static initializePaths;
    /**
     * @method      initializeSourceTracking
     * @param       {string}  projectPath  Required. ???
     * @param       {SfdxFalconProject} sfdxFPO  Required. ???
     * @returns     {Promise<void>} ???
     * @description Initializes all project variables related to source tracking.
     *              For now (ie. Aug 2018) this is more placeholder for future
     *              feature expansion than anything else.
     * @private @static @async
     */
    private static initializeSourceTracking;
    /**
     * @method      validateFalconGlobalConfig
     * @param       {SfdxFalconProject}  sfdxFPO Required.
     * @returns     {boolean|string[]}  Returns TRUE if the GLOBAL config for
     *              SFDX-Falcon is valid. If not valid, returns an array of
     *              strings listing each key that had an invalid value.
     * @description ???
     * @private @static
     */
    private static validateFalconGlobalConfig;
    /**
     * @method      validateFalconLocalConfig
     * @param       {SfdxFalconProject}  sfdxFPO Required.
     * @returns     {boolean|Array<string>}  Returns TRUE if the LOCAL config for
     *              SFDX-Falcon is valid. If not valid, returns an array of
     *              strings listing each key that had an invalid value.
     * @description ???
     * @private @static
     */
    private static validateFalconLocalConfig;
    /**
     * @method      validateFalconProjectConfig
     * @param       {SfdxFalconProject}  sfdxFPO Required.
     * @returns     {boolean|Array<string>}  Returns TRUE if the PROJECT config
     *              for SFDX-Falcon is valid. If not valid, returns an array of
     *              strings listing each key that had an invalid value.
     * @description ???
     * @private @static
     */
    private static validateFalconProjectConfig;
    /**
     * @method      validateOverallConfig
     * @param       {SfdxFalconProject}  sfdxFPO Required. ???
     * @param       {ProjectResolutionOptions}  opts  Required. ???
     * @returns     {Promise<void>} ???
     * @description ???
     * @private @static @async
     */
    private static validateOverallConfig;
    private _sfdxAggregateConfig;
    readonly sfdxAggregateConfig: JsonMap;
    private _falconProjectConfig;
    readonly falconProjectConfig: SfdxFalconProjectConfig;
    private _falconLocalConfig;
    readonly falconLocalConfig: SfdxFalconLocalConfig;
    private _falconGlobalConfig;
    readonly falconGlobalConfig: SfdxFalconGlobalConfig;
    private _projectPath;
    readonly projectPath: string;
    private _falconConfigPath;
    readonly falconConfigPath: string;
    private _templatesPath;
    readonly templatesPath: string;
    private _configPath;
    readonly configPath: string;
    private _mdapiSourcePath;
    readonly mdapiSourcePath: string;
    private _sfdxSourcePath;
    readonly sfdxSourcePath: string;
    private _dataPath;
    readonly dataPath: string;
    private _docsPath;
    readonly docsPath: string;
    private _toolsPath;
    readonly toolsPath: string;
    private _tempPath;
    readonly tempPath: string;
    private _trackedByGit;
    readonly trackedByGit: string;
    /**
     * @constructs  SfdxFalconProject
     * @description Private constructor. Should only be called by resolve().
     * @private
     */
    private constructor();
    /**
     * @method      runChosenRecipe
     * @param       {JsonMap} [compileOptions]  Optional. Object containing
     *              options/overrides that are relevant to the Engine that the
     *              Recipe is specified to use.
     * @param       {JsonMap} [executionOptions]  Optional. Object containing any
     *              options/overrides that are relevant when the Recipe's
     *              execute() method is called.
     * @returns     {Promise<SfdxFalconResult>}  Resolves with a fully
     *              populated SFDX-Falcon Recipe Result on success or bubbles
     *              up thrown errors that should be caught and handled by the
     *              caller.
     * @description Usses the console to present the user with choices of recipe
     *              to run based on either the "demo recipes" or "developer recipes"
     *              key in the respective "appxPackage" or "appxDemo" keys.
     * @public @async
     */
    runChosenRecipe(compileOptions?: JsonMap, executionOptions?: JsonMap): Promise<SfdxFalconResult>;
    /**
     * @method      runDefaultRecipe
     * @param       {JsonMap} [compileOptions]  Optional. Object containing
     *              options/overrides that are relevant to the Engine that the
     *              Recipe is specified to use.
     * @param       {JsonMap} [executionOptions]  Optional. Object containing any
     *              options/overrides that are relevant when the Recipe's
     *              execute() method is called.
     * @returns     {Promise<SfdxFalconResult>}  Resolves with a fully
     *              populated SFDX-Falcon Recipe Result on success or bubbles
     *              up thrown errors that should be caught and handled by the
     *              caller.
     * @description ???
     * @public @async
     */
    runDefaultRecipe(compileOptions?: JsonMap, executionOptions?: JsonMap): Promise<SfdxFalconResult>;
    /**
     * @method      runSpecifiedRecipe
     * @param       {string}  recipeName  Required. Name of the .json Recipe file
     *              as found within the Config Path of this project.
     * @param       {JsonMap} [compileOptions]  Optional. Object containing
     *              options/overrides that are relevant to the Engine that the
     *              Recipe is specified to use.
     * @param       {JsonMap} [executionOptions]  Optional. Object containing any
     *              options/overrides that are relevant when the Recipe's
     *              execute() method is called.
     * @returns     {Promise<SfdxFalconRecipeResponse>}  Resolves with a fully
     *              populated SFDX-Falcon Recipe Result on success or bubbles
     *              up thrown errors that should be caught and handled by the
     *              caller.
     * @description ???
     * @public @async
     */
    runSpecifiedRecipe(recipeName: string, compileOptions?: JsonMap, executionOptions?: JsonMap): Promise<SfdxFalconResult>;
    /**
     * @method      compileAndRunRecipe
     * @param       {string}  recipeName  Required. Name of the .json Recipe file
     *              as found within the Config Path of this project.
     * @param       {JsonMap} [compileOptions]  Optional. Object containing
     *              options/overrides that are relevant to the Engine that the
     *              Recipe is specified to use.
     * @param       {JsonMap} [executionOptions]  Optional. Object containing any
     *              options/overrides that are relevant when the Recipe's
     *              execute() method is called.
     * @returns     {Promise<SfdxFalconResult>}  Resolves with a fully populated
     *              SFDX-Falcon RECIPE Result on success or bubbles up thrown
     *              errors that should be caught and handled by the caller.
     * @description Compiles and runs the recipe
     * @private @async
     */
    private compileAndRunRecipe;
}
