/**
 * @file          modules/sfdx-falcon-prompt/index.ts
 * @copyright     Vivek M. Chawla - 2019
 * @author        Vivek M. Chawla <@VivekMChawla>
 * @summary       Exports SfdxFalconPrompt which provides user interaction via the console.
 * @description   Helps developers quickly build Inquirer based interviews. Prompts created this
 *                way can be executed alone or as part of an SfdxFalconInterview group.
 * @version       1.0.0
 * @license       MIT
 */
import { ConfirmationAnswers } from '../sfdx-falcon-types';
import { PromptOptions } from '../sfdx-falcon-types';
import { Questions } from '../sfdx-falcon-types';
/**
 * @class       SfdxFalconPrompt
 * @summary     Collection of one or more Inquirer Questions that can be used to gather user input.
 * @description Allows easy creation of Inquirer prompts that have a "confirmation" question that
 *              can be used to restart collection of the information.
 * @public
 */
export declare class SfdxFalconPrompt<T extends object> {
    readonly defaultAnswers: T;
    userAnswers: T;
    confirmationAnswers: ConfirmationAnswers;
    context: object;
    private readonly _questions;
    private readonly _confirmation;
    private readonly _display;
    private readonly _invertConfirmation;
    readonly confirmation: Questions;
    readonly sharedData: object;
    readonly finalAnswers: T;
    readonly questions: Questions;
    /**
     * @constructs  SfdxFalconPrompt
     * @param       {PromptOptions<T>} opts Required. Options that define the
     *              questions, default answers, confirmation questions, and
     *              Prompt Engine that should be used by this SfdxFalconPrompt
     *              object.
     * @description Constructs an SfdxFalconPrompt object.
     * @public
     */
    constructor(opts: PromptOptions<T>);
    /**
     * @method      prompt
     * @returns     {Promise<T>}  Returns the answers provided by the user.
     * @description Uses an Inquirer prompt() function to show the questions defined
     *              in this SfdxFalconPrompt to the user and returns their input.
     * @public @async
     */
    prompt(): Promise<T>;
    /**
     * @method      confirmRestart
     * @returns     {Promise<boolean>}  Returns true if the user wants to restart,
     *              false if otherwise.
     * @description If this.confirmation contains Questions, prompts the user with
     *              those questions and then decides to continue or restart based
     *              on the answers.
     * @private @async
     */
    private confirmRestart;
    /**
     * @method      displayAnswers
     * @returns     {Promise<void>}
     * @description Uses the Display function (if present) to display the User
     *              Answers from the prompt to the user. If the Display function
     *              returns void, it means that it rendered output to the user.
     *              If it returns an Array, it means that we need to manually
     *              render the returned data in a FalconTable.
     * @private @async
     */
    private displayAnswers;
}
