"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// Executor Imports
const hybrid_1 = require("../../../executors/hybrid"); // Function. Hybrid executor.
const actions_1 = require("../../appx/actions"); // Abstract class. Extend this to build a custom Action for the Appx Recipe Engine.
const types_1 = require("../../../types/"); // Enum. Represents types of SfdxFalconActions.
// Import Utility Functions
const sfdx_falcon_util_1 = require("../../../../sfdx-falcon-util"); // Function. Reads a JSON config file from disk and returns as JS Object.
const sfdx_1 = require("../../../../sfdx-falcon-util/sfdx"); // Function. SFDX Executor for getting the username associated with an Org Alias.
// Set the File Local Debug Namespace
const dbgNs = 'ACTION:configure-admin-user:';
//─────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @class       ConfigureAdminUserAction
 * @extends     AppxEngineAction
 * @description Implements the action "configure-admin-user".
 * @public
 */
//─────────────────────────────────────────────────────────────────────────────────────────────────┘
class ConfigureAdminUserAction extends actions_1.AppxEngineAction {
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      initializeAction
     * @returns     {void}
     * @description Sets member variables based on the specifics of this action.
     * @protected
     */
    //───────────────────────────────────────────────────────────────────────────┘
    initializeAction() {
        // Set values for all the base member vars to better define THIS AppxEngineAction.
        this.actionType = types_1.SfdxFalconActionType.SFDC_API;
        this.actionName = 'configure-admin-user';
        this.description = 'Configure Admin User';
        this.successDelay = 2;
        this.errorDelay = 2;
        this.progressDelay = 1000;
    }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      validateActionOptions
     * @param       {ActionOptions} actionOptions Required. The options that should
     *              be validated because they are required by this specific action.
     * @returns     {void}
     * @description Given an object containing Action Options, make sure that
     *              everything expected by this Action in order to properly
     *              execute has been provided.
     * @protected
     */
    //───────────────────────────────────────────────────────────────────────────┘
    validateActionOptions(actionOptions) {
        if (typeof actionOptions.definitionFile === 'undefined') {
            throw new Error(`ERROR_MISSING_OPTION: 'definitionFile'`);
        }
    }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      executeAction
     * @param       {ActionOptions} actionOptions Optional. Any options that the
     *              command execution logic will require in order to properly
     *              do its job.
     * @returns     {Promise<SfdxFalconResult>} Resolves with an SfdxFalconResult
     *              of type ACTION that has one or more EXECUTOR Results as
     *              children.
     * @description Performs the custom logic that's wrapped by the execute method
     *              of the base class.
     * @protected @async
     */
    //───────────────────────────────────────────────────────────────────────────┘
    async executeAction(actionContext, actionOptions = {}) {
        // Get an SFDX-Falcon Result that's customized for this Action.
        const actionResult = this.createActionResult(actionContext, actionOptions, { startNow: true,
            bubbleError: true,
            bubbleFailure: true,
            failureIsError: true });
        // Merge core DETAIL added by createActionResult() with additional DETAIL for this specific Result.
        actionResult.detail = Object.assign({}, actionResult.detail, {
            executorMessages: null,
            userDefinition: null,
            adminUsername: null
        });
        actionResult.debugResult(`Initialized`, `${dbgNs}executeAction:`);
        // Create a typed variable to represent this function's ACTION Result Detail.
        const actionResultDetail = actionResult.detail;
        // Find and read the user definition file.
        const userDefinition = await sfdx_falcon_util_1.readConfigFile(actionContext.projectContext.configPath, actionOptions.definitionFile)
            .catch(rejectedPromise => { actionResult.addRejectedChild(rejectedPromise, "UTILITY" /* UTILITY */, `general:readConfigFile`); });
        actionResultDetail.userDefinition = userDefinition;
        actionResult.debugResult(`User Definition File Read`, `${dbgNs}executeAction:`);
        // Get the username associated with the Target Org Alias (this should be the Admin User)
        const adminUsername = await sfdx_1.getUsernameFromAlias(actionContext.targetOrg.alias)
            .catch(error => { actionResult.throw(error); });
        actionResultDetail.adminUsername = adminUsername;
        actionResult.debugResult(`Determined Admin Username from Alias`, `${dbgNs}executeAction:`);
        // Define the messages that are relevant to this Action
        const executorMessages = {
            progressMsg: `Configuring user '${adminUsername}' in ${actionContext.targetOrg.alias}`,
            errorMsg: `Failed to configure user '${adminUsername}' in ${actionContext.targetOrg.alias}`,
            successMsg: `User '${adminUsername}' configured successfully`
        };
        actionResultDetail.executorMessages = executorMessages;
        actionResult.debugResult(`Executor Messages Set`, `${dbgNs}executeAction:`);
        // Run the executor then return or throw the result.
        // OPTIONAL: If you want to override success/error handling, do it here.
        return await hybrid_1.configureUser(adminUsername, userDefinition, actionContext.targetOrg, executorMessages, actionContext.listrExecOptions.observer)
            .catch(rejectedPromise => actionResult.addRejectedChild(rejectedPromise, "EXECUTOR" /* EXECUTOR */, `hybrid:configureUser`))
            .then(resolvedPromise => {
            if (resolvedPromise === actionResult) {
                // If "resolvedPromise" points to the same location in memory as "actionResult", it means that
                // executeSfdxCommand() returned an ERROR which was suppressed. If you don't want to suppress EXECUTOR
                // errors, the ACTION Result used by this class must be instantiated with "bubbleError" set to FALSE.
                return actionResult;
            }
            return actionResult.addResolvedChild(resolvedPromise, "EXECUTOR" /* EXECUTOR */, `hybrid:configureUser`);
        });
    }
}
exports.ConfigureAdminUserAction = ConfigureAdminUserAction;
//# sourceMappingURL=configure-admin-user.js.map