"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// Executor Imports
const hybrid_1 = require("../../../executors/hybrid"); // Function. Hybrid executor
const actions_1 = require("../../appx/actions"); // Abstract class. Extend this to build a custom Action for the Appx Recipe Engine.
const types_1 = require("../../../types/"); // Enum. Represents types of SfdxFalconActions.
// Import Utility Functions
const sfdx_falcon_util_1 = require("../../../../sfdx-falcon-util"); // Function. Adds a UUID to a username to create something unique.
const sfdx_falcon_util_2 = require("../../../../sfdx-falcon-util"); // Function. Reads a JSON config file from disk and returns as JS Object.
// Set the File Local Debug Namespace
const dbgNs = 'ACTION:create-user:';
//─────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @class       CreateUserAction
 * @extends     AppxEngineAction
 * @description Implements the action "create-user".
 * @public
 */
//─────────────────────────────────────────────────────────────────────────────────────────────────┘
class CreateUserAction extends actions_1.AppxEngineAction {
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      initializeAction
     * @returns     {void}
     * @description Sets member variables based on the specifics of this action.
     * @protected
     */
    //───────────────────────────────────────────────────────────────────────────┘
    initializeAction() {
        // Set values for all the base member vars to better define THIS AppxEngineAction.
        this.actionType = types_1.SfdxFalconActionType.SFDC_API;
        this.actionName = 'create-user';
        this.description = 'Create User';
        this.successDelay = 2;
        this.errorDelay = 2;
        this.progressDelay = 1000;
    }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      validateActionOptions
     * @param       {ActionOptions} actionOptions Required. The options that
     *              should be validated because they are required by this specific
     *              action.
     * @returns     {void}
     * @description Given an object containing Action Options, make sure that
     *              everything expected by this Action in order to properly
     *              execute has been provided.
     * @protected
     */
    //───────────────────────────────────────────────────────────────────────────┘
    validateActionOptions(actionOptions) {
        if (typeof actionOptions.definitionFile === 'undefined')
            throw new Error(`ERROR_MISSING_OPTION: 'definitionFile'`);
        if (typeof actionOptions.sfdxUserAlias === 'undefined')
            throw new Error(`ERROR_MISSING_OPTION: 'sfdxUserAlias'`);
    }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      executeAction
     * @param       {ActionOptions} actionOptions Optional. Any options that the
     *              command execution logic will require in order to properly do
     *              its job.
     * @returns     {Promise<SfdxFalconResult>} Resolves with an SfdxFalconResult
     *              of type ACTION that has one or more EXECUTOR Results as
     *              children.
     * @description Performs the custom logic that's wrapped by the execute method
     *              of the base class.
     * @protected @async
     */
    //───────────────────────────────────────────────────────────────────────────┘
    async executeAction(actionContext, actionOptions = {}) {
        // Get an SFDX-Falcon Result that's customized for this Action.
        const actionResult = this.createActionResult(actionContext, actionOptions, { startNow: true,
            bubbleError: true,
            bubbleFailure: true,
            failureIsError: true });
        // Merge core DETAIL added by createActionResult() with additional DETAIL for this specific Result.
        actionResult.detail = Object.assign({}, actionResult.detail, {
            executorMessages: null,
            userDefinition: null,
            uniqueUsername: null,
            defaultPassword: null
        });
        actionResult.debugResult(`Initialized`, `${dbgNs}executeAction:`);
        // Create a typed variable to represent this function's ACTION Result Detail.
        const actionResultDetail = actionResult.detail;
        // Find and read the user definition file.
        const userDefinition = await sfdx_falcon_util_2.readConfigFile(actionContext.projectContext.configPath, actionOptions.definitionFile)
            .catch(error => { actionResult.throw(error); });
        actionResultDetail.userDefinition = userDefinition;
        actionResult.debugResult(`User Definition File Read`, `${dbgNs}executeAction:`);
        // Create a unique username based on what's in the definition file.
        const uniqueUsername = sfdx_falcon_util_1.createUniqueUsername(userDefinition['Username']);
        actionResultDetail.uniqueUsername = uniqueUsername;
        actionResult.debugResult(`Unique Username Generated`, `${dbgNs}executeAction:`);
        // Determine what the appropriate default password should be.
        const defaultPassword = determineDefaultPassword(userDefinition['password']);
        actionResultDetail.defaultPassword = defaultPassword;
        actionResult.debugResult(`Default Password Determined`, `${dbgNs}executeAction:`);
        // Define the messages for this command.
        const executorMessages = {
            progressMsg: `Creating User '${uniqueUsername}' in ${actionContext.targetOrg.alias}`,
            errorMsg: `Failed to create User '${uniqueUsername}' in ${actionContext.targetOrg.alias}`,
            successMsg: `User '${uniqueUsername}' created successfully`
        };
        actionResultDetail.executorMessages = executorMessages;
        actionResult.debugResult(`Executor Messages Set`, `${dbgNs}executeAction:`);
        // Run the executor then return or throw the result.
        // OPTIONAL: If you want to override success/error handling, do it here.
        return await hybrid_1.createUser(uniqueUsername, defaultPassword, userDefinition, actionContext.targetOrg, executorMessages, actionContext.listrExecOptions.observer)
            .catch(rejectedPromise => actionResult.addRejectedChild(rejectedPromise, "EXECUTOR" /* EXECUTOR */, `hybrid:createUser`))
            .then(resolvedPromise => {
            if (resolvedPromise === actionResult) {
                // If "resolvedPromise" points to the same location in memory as "actionResult", it means that
                // executeSfdxCommand() returned an ERROR which was suppressed. If you don't want to suppress EXECUTOR
                // errors, the ACTION Result used by this class must be instantiated with "bubbleError" set to FALSE.
                return actionResult;
            }
            return actionResult.addResolvedChild(resolvedPromise, "EXECUTOR" /* EXECUTOR */, `hybrid:createUser`);
        });
    }
}
exports.CreateUserAction = CreateUserAction;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    determineDefaultPassword
 * @param       {string}  suggestedPassword Optional. This will usually be returned as-is if it's
 *              provided. Otherwise this will be determined by some "default processing" logic.
 * @returns     {string}  Either a reflectoin of the suggested password, or a better one.
 * @description Given a "suggested password", this function will determine if it's OK to use that
 *              password or supply a different one as a replacement.
 * @version     1.0.0
 * @private
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function determineDefaultPassword(suggestedPassword) {
    if (!suggestedPassword) {
        return '1HappyCloud';
    }
    else {
        return suggestedPassword;
    }
}
//# sourceMappingURL=create-user.js.map