import { SfdxFalconResult } from '../../../../sfdx-falcon-result';
import { SfdxFalconResultOptions } from '../../../../sfdx-falcon-result';
import { SfdxCommandDefinition } from '../../../executors/sfdx';
import { AppxEngineActionContext } from '../../../engines/appx';
import { ActionOptions } from '../../../types/';
import { ExecutorMessages } from '../../../types/';
import { SfdxFalconActionType } from '../../../types/';
/**
 * @interface   CoreActionResultDetail
 * @description Represents the core set of "detail" information that every ACTION result should have.
 */
export interface CoreActionResultDetail {
    actionType: SfdxFalconActionType;
    actionName: string;
    description: string;
    executorMessages: ExecutorMessages;
    actionContext: AppxEngineActionContext;
    actionOptions: ActionOptions;
}
/**
 * @interface   SfdxCliActionResultDetail
 * @description Represents the "detail" information that every SFDX-CLI ACTION result should have.
 */
export interface SfdxCliActionResultDetail extends CoreActionResultDetail {
    sfdxCommandDef: SfdxCommandDefinition;
}
/**
 * @class       AppxEngineAction
 * @description Base class for all "actions" that are supported by the "Appx" Falcon Recipe Engine.
 * @public @abstract
 */
export declare abstract class AppxEngineAction {
    protected actionName: string;
    protected actionType: SfdxFalconActionType;
    protected description: string;
    protected successDelay: number;
    protected errorDelay: number;
    protected progressDelay: number;
    /**
     * @constructs  AppxEngineAction
     * @description Constructor for the AppxEngineAction class.
     * @public
     */
    constructor();
    /**
     * @method      execute
     * @param       {AppxEngineActionContext} actionContext Required. The context
     *              of the AppxEngine that is executing this action.
     * @param       {ActionOptions} [actionOptions] Optional. Any options that the
     *              command execution logic will require in order to properly do
     *              its job.
     * @returns     {Promise<SfdxFalconActionResponse>} Resolves with an SFDX-
     *              Falcon Action Response object, populated with an array of
     *              the Executors that were run when the Action was executed.
     * @description Wraps the calling of a specific SFDX-Falcon Action, as defined
     *              by the child class that extends this.
     * @public @async
     */
    execute(actionContext: AppxEngineActionContext, actionOptions?: ActionOptions): Promise<SfdxFalconResult>;
    protected abstract executeAction(actionContext: AppxEngineActionContext, actionOptions: ActionOptions): Promise<SfdxFalconResult>;
    protected abstract initializeAction(): void;
    protected abstract validateActionOptions(actionOptions: ActionOptions): void;
    /**
     * @method      createActionResult
     * @param       {AppxEngineActionContext} actionContext Required. The context
     *              of the AppxEngine that is executing the action.
     * @param       {ActionOptions} [actionOptions] Optional. Any options that the
     *              command execution logic will require in order to properly do
     *              its job.
     * @param       {SfdxFalconResultOptions} [sfdxFalconResultOptions] Optional.
     *              Specifies options that will be passed to the SfdxFalconResult
     *              contstructor.
     * @returns     {SfdxFalconResult} A new SFDX-Falcon Result customized with
     *              the information specific to this AppxEngineAction instance.
     * @description Takes an AppxEngineActionContext and Action Options along
     *              with options for the SfdxFalconResult constructor and creates
     *              an SFDX-Falcon Result object, then adds details about this
     *              Action to it before returning it to the caller.
     * @protected
     */
    protected createActionResult(actionContext: AppxEngineActionContext, actionOptions?: ActionOptions, sfdxFalconResultOptions?: SfdxFalconResultOptions): SfdxFalconResult;
    /**
     * @method      onError
     * @param       {unknown} rejectedPromise Required. The result of a rejected promise.
     * @returns     {void}
     * @description Handles rejected calls returning from this.executeAction()
     *              calls that have been made to the child class.
     * @version     1.0.0
     * @private
     */
    private onError;
    /**
     * @method      onSuccess
     * @param       {SfdxFalconExecutorResponse}  execSuccessResponse Required.
     * @returns     {SfdxFalconResult}
     * @description ???
     * @private
     */
    private onSuccess;
    /**
     * @method      validateActionContext
     * @param       {AppxEngineActionContext} actionContext Required. ???
     * @description ???
     * @private
     */
    private validateActionContext;
}
