"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
//─────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @file          modules/sfdx-falcon-recipe/engines/appx/actions/index.ts
 * @copyright     Vivek M. Chawla - 2018
 * @author        Vivek M. Chawla <@VivekMChawla>
 * @summary       Abstract base class that provides the core logic for SFDX-Falcon Actions.
 * @description   Abstract base class that provides the core logic for SFDX-Falcon Actions.
 * @version       1.0.0
 * @license       MIT
 */
//─────────────────────────────────────────────────────────────────────────────────────────────────┘
// Import local modules
const sfdx_falcon_debug_1 = require("../../../../sfdx-falcon-debug"); // Why?
const sfdx_falcon_error_1 = require("../../../../sfdx-falcon-error"); // Why?
const sfdx_falcon_result_1 = require("../../../../sfdx-falcon-result"); // Class. Provides framework for bubbling "results" up from nested calls.
const async_1 = require("../../../../sfdx-falcon-util/async"); // Why?
const types_1 = require("../../../types/"); // Enum. Represents types of SfdxFalconActions.
// Set the File Local Debug Namespace
const dbgNs = 'ACTION:appx-engine-action:';
//─────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @class       AppxEngineAction
 * @description Base class for all "actions" that are supported by the "Appx" Falcon Recipe Engine.
 * @public @abstract
 */
//─────────────────────────────────────────────────────────────────────────────────────────────────┘
class AppxEngineAction {
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @constructs  AppxEngineAction
     * @description Constructor for the AppxEngineAction class.
     * @public
     */
    //───────────────────────────────────────────────────────────────────────────┘
    constructor() {
        // Set default values for base properties.
        this.actionName = 'unspecified-action';
        this.actionType = types_1.SfdxFalconActionType.UNSPECIFIED;
        this.description = 'Unspecified Action';
        this.successDelay = 2;
        this.errorDelay = 2;
        this.progressDelay = 1000;
        // Give the child class the chance to initialize over the defaults
        this.initializeAction();
    }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      execute
     * @param       {AppxEngineActionContext} actionContext Required. The context
     *              of the AppxEngine that is executing this action.
     * @param       {ActionOptions} [actionOptions] Optional. Any options that the
     *              command execution logic will require in order to properly do
     *              its job.
     * @returns     {Promise<SfdxFalconActionResponse>} Resolves with an SFDX-
     *              Falcon Action Response object, populated with an array of
     *              the Executors that were run when the Action was executed.
     * @description Wraps the calling of a specific SFDX-Falcon Action, as defined
     *              by the child class that extends this.
     * @public @async
     */
    //───────────────────────────────────────────────────────────────────────────┘
    async execute(actionContext, actionOptions = {}) {
        // Validate the Action Context (implemented here by parent class)
        this.validateActionContext(actionContext);
        // Validate the Action Options (implemented by child class)
        this.validateActionOptions(actionOptions);
        // Call the executeAction method and handle success/errors
        const falconActionResult = await this.executeAction(actionContext, actionOptions)
            .then(successResult => this.onSuccess(successResult))
            .catch(failureResult => this.onError(failureResult));
        // Wait some number of seconds to give the user a chance to see the final status message.
        await async_1.waitASecond(this.successDelay);
        // Return whatever was put together by the onSuccess() method.
        return falconActionResult;
    }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      createActionResult
     * @param       {AppxEngineActionContext} actionContext Required. The context
     *              of the AppxEngine that is executing the action.
     * @param       {ActionOptions} [actionOptions] Optional. Any options that the
     *              command execution logic will require in order to properly do
     *              its job.
     * @param       {SfdxFalconResultOptions} [sfdxFalconResultOptions] Optional.
     *              Specifies options that will be passed to the SfdxFalconResult
     *              contstructor.
     * @returns     {SfdxFalconResult} A new SFDX-Falcon Result customized with
     *              the information specific to this AppxEngineAction instance.
     * @description Takes an AppxEngineActionContext and Action Options along
     *              with options for the SfdxFalconResult constructor and creates
     *              an SFDX-Falcon Result object, then adds details about this
     *              Action to it before returning it to the caller.
     * @protected
     */
    //───────────────────────────────────────────────────────────────────────────┘
    createActionResult(actionContext, actionOptions = {}, sfdxFalconResultOptions = {}) {
        // Initialize an SFDX-Falcon ACTION Result.
        const falconActionResult = new sfdx_falcon_result_1.SfdxFalconResult(`${this.actionName}:executeAction`, "ACTION" /* ACTION */, sfdxFalconResultOptions);
        // Set the detail for the ACTION Result.
        falconActionResult.setDetail({
            actionType: this.actionType,
            actionName: this.actionName,
            description: this.description,
            actionContext: actionContext,
            actionOptions: actionOptions
        });
        // Done. Send our new friend back to the caller.
        return falconActionResult;
    }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      onError
     * @param       {unknown} rejectedPromise Required. The result of a rejected promise.
     * @returns     {void}
     * @description Handles rejected calls returning from this.executeAction()
     *              calls that have been made to the child class.
     * @version     1.0.0
     * @private
     */
    //───────────────────────────────────────────────────────────────────────────┘
    onError(rejectedPromise) {
        // Debug the contents of the Rejected Promise BEFORE we do anything to it.
        sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}onError:rejectedPromise:`, { rejectedPromise: rejectedPromise }, `rejectedPromise: `);
        // Make sure any rejected promises are wrapped as an SFDX-Falcon Result.
        const thisActionResult = sfdx_falcon_result_1.SfdxFalconResult.wrap(rejectedPromise, "UNKNOWN" /* UNKNOWN */, 'ActionResult (REJECTED)');
        // If the ACTION Result still WAITING, then the Child class did not complete the Result.  Do that now.
        if (thisActionResult.status === "WAITING" /* WAITING */) {
            // NOTE: By passing its own errObj, the error() method will handle a missing Error Object.
            thisActionResult.error(thisActionResult.errObj);
        }
        // Since thie was a FAILUIRE, throw the Action Response to inform the caller who started the Engine.
        throw thisActionResult;
    }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      onSuccess
     * @param       {SfdxFalconExecutorResponse}  execSuccessResponse Required.
     * @returns     {SfdxFalconResult}
     * @description ???
     * @private
     */
    //───────────────────────────────────────────────────────────────────────────┘
    onSuccess(falconActionResult) {
        // Make sure that we were provided with an SFDX-Falcon Result
        if ((falconActionResult instanceof sfdx_falcon_result_1.SfdxFalconResult) !== true) {
            const typeError = new sfdx_falcon_error_1.SfdxFalconError(`AppxEngineAction.onSuccess() expects an argument that's an instance  `
                + `of 'SfdxFalconResult', not '${falconActionResult.constructor.name}'`, `TypeError`, `${dbgNs}onSuccess`);
            typeError.setData({ rawResult: falconActionResult });
            throw typeError;
        }
        // Debug the contents of the Action Result.
        sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}onSuccess:falconActionResult:`, falconActionResult, `falconActionResult: `);
        // If the ACTION Result still WAITING, then the Child class did not complete the Result.  Do that now.
        if (falconActionResult.status === "WAITING" /* WAITING */) {
            falconActionResult.success();
        }
        // Done.
        return falconActionResult;
    }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      validateActionContext
     * @param       {AppxEngineActionContext} actionContext Required. ???
     * @description ???
     * @private
     */
    //───────────────────────────────────────────────────────────────────────────┘
    validateActionContext(actionContext) {
        // VALIDATION: actionContext must be provided.
        if (typeof actionContext === 'undefined') {
            throw new sfdx_falcon_error_1.SfdxFalconError(`Missing actionContxt object when attempting to `
                + `execute an AppxEngineAction`, `InvalidActionContext`, `${dbgNs}validateActionContext`);
        }
        // VALIDATION: If the target is a Scratch Org, then a DevHub must be provided.
        if (actionContext.targetOrg.isScratchOrg === true && (!actionContext.devHubAlias)) {
            const actionContextError = new sfdx_falcon_error_1.SfdxFalconError(`Target Org is identified as a scratch org, but `
                + `a DevHub Alias was not provided`, `InvalidActionContext`, `${dbgNs}validateActionContext`);
            actionContextError.setData(actionContext);
            throw actionContextError;
        }
    }
}
exports.AppxEngineAction = AppxEngineAction;
//# sourceMappingURL=index.js.map