/**
 * @file          modules/sfdx-falcon-recipe/engines/appx/index.ts
 * @copyright     Vivek M. Chawla - 2018
 * @author        Vivek M. Chawla <@VivekMChawla>
 * @version       1.0.0
 * @license       MIT
 * @summary       ???
 * @description   ???
 */
import { AnyJson } from '@salesforce/ts-types';
import { JsonMap } from '@salesforce/ts-types';
import { SfdxFalconResult } from '../../../../modules/sfdx-falcon-result';
import { ListrContext } from '../../../../modules/sfdx-falcon-types';
import { ListrExecutionOptions } from '../../../../modules/sfdx-falcon-types';
import { ListrObservable } from '../../../../modules/sfdx-falcon-types';
import { ListrObject } from '../../../../modules/sfdx-falcon-types';
import { SfdxCliLogLevel } from '../../../../modules/sfdx-falcon-types';
import { ActionOptions } from '../../types';
import { CompileOptions } from '../../types';
import { ExecutionOptions } from '../../types';
import { TargetOrg } from '../../types';
import { SfdxFalconProject } from '../../../../modules/sfdx-falcon-project';
import { SfdxFalconRecipe } from '../../../../modules/sfdx-falcon-recipe';
import { SfdxFalconRecipeJson } from '../../../../modules/sfdx-falcon-recipe';
import { AppxEngineAction } from '../appx/actions';
export interface AppxEngineContext {
    compileOptions: CompileOptions;
    recipeObserver: ListrObservable;
    executing: boolean;
    initialized: boolean;
    haltOnError: boolean;
    skipGroups: string[];
    skipActions: string[];
    devHubAlias: string;
    projectContext: SfdxFalconProject;
    logLevel: SfdxCliLogLevel;
    targetOrg: TargetOrg;
}
export interface AppxEngineActionContext extends AppxEngineContext {
    listrExecOptions: ListrExecutionOptions;
}
export declare type AppxEngineActionFunction = (actionContext: AppxEngineActionContext, actionOptions: ActionOptions) => Promise<SfdxFalconResult>;
export interface AppxEngineHandler {
    handlerName: string;
}
export interface AppxEngineStepGroup {
    stepGroupName: string;
    alias: string;
    description: string;
    recipeSteps: AppxEngineStep[];
}
export interface AppxEngineStep {
    stepName: string;
    description: string;
    action: string;
    options: JsonMap;
    onSuccess?: string;
    onError?: string;
}
export interface AppxEngineStepResult {
    status: AppxEngineStepResultStatus;
    message: string;
    data: AnyJson;
}
export declare enum AppxEngineStepResultStatus {
    SUCCESS = "SUCCESS",
    WARNING = "WARNING",
    ERROR = "ERROR"
}
export interface FalconEngineResultDetail {
    recipeName: string;
    engineName: string;
    engineContext: AppxEngineContext;
    projectContext: SfdxFalconProject;
    supportedActions: IterableIterator<string>;
}
/**
 * @class       AppxRecipeEngine
 * @summary     Abstract class for Appx Recipe Engines.
 * @description Abstract class for creating custom SFDX-Falcon Recipe Engines for Appx flavored
 *              projects.
 * @public @abstract
 */
export declare abstract class AppxRecipeEngine {
    /**
     * @method      validateOuterRecipe
     * @param       {SfdxFalconRecipeJson} recipe Required.
     * @returns     {void}
     * @description ???
     * @protected @static
     */
    protected static validateOuterRecipe(recipe: SfdxFalconRecipeJson): void;
    /**
     * @method      validateHandler
     * @param       {AppxEngineHandler} handler Required.
     * @returns     {void}
     * @description ???
     * @private @static
     */
    private static validateHandler;
    /**
     * @method      validateRecipeStep
     * @param       {AppxEngineStep} recipeStep Required.
     * @returns     {void}
     * @description ???
     * @private @static
     */
    private static validateRecipeStep;
    /**
     * @method      validateRecipeStepGroup
     * @param       {AppxRecipeStepGroup} stepGroup Required.
     * @returns     {void}
     * @description ???
     * @private @static
     */
    private static validateRecipeStepGroup;
    /**
     * @method      validateTargetOrg
     * @param       {TargetOrg} targetOrg Required.
     * @returns     {void}
     * @description ???
     * @private
     */
    private static validateTargetOrg;
    protected actionExecutorMap: Map<string, AppxEngineAction>;
    protected listrTasks: ListrObject;
    protected recipe: SfdxFalconRecipe;
    protected falconEngineResult: SfdxFalconResult;
    protected falconEngineResultDetail: FalconEngineResultDetail;
    protected preBuildStepGroups: AppxEngineStepGroup[];
    protected postBuildStepGroups: AppxEngineStepGroup[];
    protected engineContext: AppxEngineContext;
    /**
     * @constructs  AppxRecipeEngine
     * @param       {string}  engineName  Required. Name of the specific Engine
     *              that is extending this class.
     * @param       {string}  recipeName  Required. Name of the Recipe being
     *              compiled into the Engine.
     * @description Creates an instance ONLY for use of the compileRecipe() and
     *              compile() functions.  That is how instances of this object
     *              are passed back to the caller.
     */
    constructor(engineName: string, recipeName: string);
    /**
     * @method      execute
     * @param       {ExecutionOptions}  [executionOptions]  Optional.
     * @returns     {Promise<SfdxFalconResult>} Resolves with the SFDX-Falcon
     *              Result for this ENGINE.
     * @description Starts the execution of a compiled recipe.  Execution starutp
     *              and cleanup are handled here in the base class. Relies on the
     *              extended class to actually execute the Listr Tasks, though.
     *              This gives the extended class the ability to inject any final
     *              pre or post task execution logic.
     * @public @async
     */
    execute(executionOptions?: ExecutionOptions): Promise<SfdxFalconResult>;
    protected abstract executeEngine(executionOptions: ExecutionOptions): Promise<ListrContext>;
    protected abstract initializeActionMap(): Promise<void>;
    protected abstract initializePostBuildStepGroups(): Promise<void>;
    protected abstract initializePreBuildStepGroups(): Promise<void>;
    protected abstract initializeRecipeEngineContext(): Promise<void>;
    protected abstract initializeSkipActions(): Promise<void>;
    protected abstract initializeSkipGroups(): Promise<void>;
    protected abstract initializeTargetOrg(): Promise<void>;
    /**
     * @method      compile
     * @param       {SfdxFalconRecipe}  recipe  Required.
     * @param       {CompileOptions}  compileOptions  Required.
     * @returns     {Promise<void>} ???
     * @description ???
     * @protected @async
     */
    protected compile(recipe: SfdxFalconRecipe, compileOptions: CompileOptions): Promise<void>;
    /**
     * @method      compileAllTasks
     * @returns     {void}
     * @description Compiles all of the Listr tasks required by the recipe and
     *              adds them to the listrTasks member var.
     * @private
     */
    private compileAllTasks;
    /**
     * @method      compileParentTasks
     * @param       {AppxEngineStepGroup[]}  recipeStepGroups Required.
     * @returns     {ListrObject} Returns an instantiated Listr object fully
     *              populated with SubTasks.
     * @description ???
     * @private
     */
    private compileParentTasks;
    /**
     * @method      compileSubTasks
     * @param       {AppxRecipeStepGroup} recipeStepGroup Required.
     * @returns     {ListrObject} Returns an instantiated Listr object fully
     *              populated by all active Sub Tasks based on the Recipe Step
     *              Group.
     * @description ???
     * @private
     */
    private compileSubTasks;
    /**
     * @method      endExecution
     * @returns     {void}
     * @description Lets the Engine know that execution has stopped without error.
     * @private
     */
    private endExecution;
    /**
     * @method      executeStep
     * @param       {AppxEngineStep}  recipeStep  Required. The step to execute.
     * @param       {ListrExecutionOptions} executionOptions  Required. Holds a
     *              number of execution options (context, task, and observer).
     * @returns     {Promise<SfdxFalconResult>}  Resolves AND rejects with
     *              an SFDX-Falcon Result object.  If any other type
     *              of object bubbles up, it should be an Error.
     * @description Given a valid Falcon Recipe Step object, tries to
     *              route the requested Step Action to the appropriate Executor.
     * @private @async
     */
    private executeStep;
    /**
     * @method      initializeEngineResultDetail
     * @returns     {void}
     * @description ???
     * @private
     */
    private initializeEngineResultDetail;
    /**
     * @method      onError
     * @param       {unknown} resultOrError  Required. This SHOULD always be an
     *              ENGINE Result (this.falconEngineResult, to be exact)...BUT
     *              there is a small chance this may be some other kind of error.
     * @returns     {void}
     * @description Handles rejected calls returning from this.executeEngine().
     * @private
     */
    private onError;
    /**
     * @method      onSuccess
     * @param       {ListrContext}  listrContext  Required.
     * @returns     {void}
     * @description Called upon successful return from this.executeEngine().
     * @private
     */
    private onSuccess;
    /**
     * @method      skipAction
     * @param       {string}  actionToCheck Required. The Action to check.
     * @returns     {boolean} Returns true if the action should be skipped.
     * @description Checks if an action should be skipped during compile.
     * @private
     */
    private skipAction;
    /**
     * @method      skipGroup
     * @param       {string}  groupToCheck Required. The Step Group Name to check.
     * @returns     {boolean} Returns true if the group should be skipped.
     * @description Checks if a group of steps should be skipped during compile.
     * @private
     */
    private skipGroup;
    /**
     * @method      startExecution
     * @returns     {void}
     * @description Executes pre-engine-run logic right before running Listr tasks.
     * @private
     */
    private startExecution;
    /**
     * @method      stepGroupHasActiveTasks
     * @param       {AppxEngineStepGroup}  stepGroupToCheck Required. The Step
     *              Group object that will be inspected by this method.
     * @returns     {boolean} Returns true if the group has at least one active
     *              step. An "active step" is one whose Action is not on the
     *              skipActions list.
     * @description Checks if a Step Group has at least one active step.
     * @version     1.0.0
     * @private
     */
    private stepGroupHasActiveTasks;
    /**
     * @method      validateEngine
     * @returns     {void}
     * @description Validates the overall state of the AppxRecipeEngine and
     *              determines if it is ready for compilation.
     * @private
     */
    private validateEngine;
}
