/**
 * @file          modules/sfdx-falcon-recipe/executors/hybrid.ts
 * @copyright     Vivek M. Chawla - 2018
 * @author        Vivek M. Chawla <@VivekMChawla>
 * @summary       Executor logic that leverages SFDX and JSForce.
 * @description   Executor logic that leverages SFDX and JSForce.
 * @version       1.0.0
 * @license       MIT
 */
import { Connection } from '@salesforce/core';
import { JsonMap } from '@salesforce/ts-types';
import { SfdxFalconResult } from '../../sfdx-falcon-result';
import { TargetOrg } from '../types';
import { ExecutorMessages } from '../types';
import { Subscriber } from '../../sfdx-falcon-types';
/**
 * @function    assignPermsets
 * @param       {string|Connection} aliasOrConnection  Required. Either a string containing the
 *              Alias of the org being queried or an authenticated JSForce Connection object.
 * @param       {string}  userId    Required. Id of the user getting the permsets.
 * @param       {string[]}  permsets  Required. Arrary of permset names to assign.
 * @returns     {Promise<SfdxFalconResult>}  ???
 * @description Assigns permsets to the specified user in the specified org.
 * @public @async
 */
export declare function assignPermsets(aliasOrConnection: string | Connection, userId: string, permsets: string[]): Promise<SfdxFalconResult>;
/**
 * @function    configureUser
 * @param       {string}  username  Required. Username of the user that is going to be configured.
 * @param       {JsonMap} userDefinition  Required. JSON representing a definition of the user, based
 *              on the standard user-definition.json format used by the Salesforce CLI.
 * @param       {TargetOrg} targetOrg Required. The org where the user will be configured.
 * @param       {ExecutorMessages}  executorMessages  Required. Determines what the user sees for
 *              progress, error, and success events.
 * @param       {Subscriber}  [observer]  Optional. An Observable object that allows this function
 *              to communicate status to an outside listener (typically used by Listr).
 * @returns     {Promise<SfdxFalconResult>}  Resolves with an EXECUTOR Result.
 * @description Given a username, a user definition object, and a target org, attempts to modify
 *              the metadata of the specified user based on the contents of the user definition.
 * @public @async
 */
export declare function configureUser(username: string, userDefinition: JsonMap, targetOrg: TargetOrg, executorMessages: ExecutorMessages, observer?: Subscriber): Promise<SfdxFalconResult>;
/**
 * @function    createUser
 * @param       {string}  uniqueUsername  Required. Username for the user that will be created. As
 *              with all Salesforce usernames, this must be globally unique.
 * @param       {string}  password  Required.  Pass NULL if no password should be set.
 * @param       {JsonMap} userDefinition  Required. JSON representing a definition of the user, based
 *              on the standard user-definition.json format used by the Salesforce CLI.
 * @param       {TargetOrg} targetOrg Required. The org where the user will be created.
 * @param       {ExecutorMessages}  executorMessages  Required. Determines what the user sees for
 *              progress, error, and success events.
 * @param       {Subscriber}  [observer]  Optional. An Observable object that allows this function
 *              to communicate status to an outside listener (typically used by Listr).
 * @returns     {Promise<SfdxFalconResult>}  Resolves with an EXECUTOR Result.
 * @description Given a unique username, a user definition object, and a target org, attempts to
 *              create a new user in the Target Org based on the contents of the user definition.
 * @public @async
 */
export declare function createUser(uniqueUsername: string, password: string, userDefinition: JsonMap, targetOrg: TargetOrg, executorMessages: ExecutorMessages, observer?: Subscriber): Promise<SfdxFalconResult>;
