import { JsonMap } from '@salesforce/ts-types';
import { SfdxFalconProject } from '../../modules/sfdx-falcon-project';
import { SfdxFalconResult } from '../../modules/sfdx-falcon-result';
import { AppxRecipeEngine } from './engines/appx';
import { RecipeType } from './types';
import { CompileOptions } from './types';
import { ExecutionOptions } from './types';
/**
 * @interface   SfdxFalconRecipeJson
 * @description Defines the expected JSON schema of an SFDX-Falcon Recipe file.
 */
export interface SfdxFalconRecipeJson {
    recipeName: string;
    description: string;
    recipeType: string;
    recipeVersion: string;
    schemaVersion: string;
    options: JsonMap;
    recipeStepGroups: JsonMap[];
    handlers: JsonMap[];
}
/**
 * @class       SfdxFalconRecipe
 * @summary     Represents an SFDX-Falcon "recipe"
 * @description Use the SfdxFalconRecipe.read() static method to locate an SFDX-Falcon recipe in
 *              the local environment and then provide compile/execute cabability on that recipe.
 * @version     1.0.0
 * @public
 */
export declare class SfdxFalconRecipe {
    /**
     * @method      read
     * @param       {SfdxFalconProject} sfdxFPO Required. SFDX-Falcon Project
     *              Object associated with the project context that this Recipe
     *              is being read into.
     * @param       {string}  recipeFile  Required. Name of a Recipe.json file
     *              that will be expected to reside inside the /config directory
     *              within the SFDX-Falcon Project.
     * @description ???
     * @public @static @async
     */
    static read(sfdxFPO: SfdxFalconProject, recipeFile: string): Promise<SfdxFalconRecipe>;
    /**
     * @method      resolveSfdxFalconRecipeFile
     * @param       {string}  recipePath ???
     * @param       {string}  recipeFile ???
     * @returns     {Promise<object>}  ???
     * @description ???
     * @private @static @async
     */
    private static resolveSfdxFalconRecipe;
    /**
     * @method      validate
     * @param       {SfdxFalconRecipeJson}  sfdxFalconRecipeJson Required. Parsed
     *              JSON object that is a direct representation of an SFDX-Falcon
     *              Recipe file as read from the local filesystem.
     * @returns     {void}  Throws an error if the Recipe is found to be invalid.
     * @description Given a Recipe already parsed into a JS Object, takes multiple
     *              actions to ensure that the recipe is valid. Throws an error if
     *              the Recipe is invalid.
     * @version     1.0.0
     * @private @static
     */
    private static validate;
    /**
     * @method      validateTopLevelProperties
     * @param       {SfdxFalconRecipeJson}  sfdxFalconRecipe Required. JS Object
     *              that is a direct representation of an SFDX-Falcon Recipe file
     *              as read from the local filesystem.
     * @returns     {void}  Throws Error listing invalid/missing top-level keys.
     * @description Given a Recipe already parsed into a JS Object, throws an
     *              error with a list of top-level keys that are missing or
     *              invalid. Nothing happens validation is successful.
     * @private @static
     */
    private static validateTopLevelProperties;
    private _recipeName;
    readonly recipeName: string;
    private _description;
    readonly description: string;
    private _recipeType;
    readonly recipeType: RecipeType;
    private _recipeVersion;
    readonly recipeVersion: string;
    private _schemaVersion;
    readonly schemaVersion: string;
    private _options;
    readonly options: JsonMap;
    private _recipeStepGroups;
    readonly recipeStepGroups: JsonMap[];
    private _handlers;
    readonly handlers: JsonMap[];
    private _compiled;
    readonly compiled: boolean;
    private _projectContext;
    readonly projectContext: SfdxFalconProject;
    private _recipeEngine;
    readonly recipeEngine: AppxRecipeEngine;
    private _falconRecipeResult;
    readonly falconRecipeResult: SfdxFalconResult;
    private _falconRecipeResultDetail;
    readonly falconRecipeResultDetail: any;
    private _validated;
    readonly validated: boolean;
    /**
     * @constructs  SfdxFalconRecipe
     * @description EMPTY CONSTRUCTOR. SfdxFalconRecipe.read() should be used to
     *              instantiate SFDX Falcon Recipes.
     * @private
     */
    private constructor();
    /**
     * @method      compile
     * @param       {CompileOptions}  compileOptions Optional. ???
     * @description ???
     * @public @async
     */
    compile(compileOptions?: CompileOptions): Promise<void>;
    /**
     * @method      execute
     * @param       {any}  executionOptions Optional. ???
     * @returns     {Promise<SfdxFalconResult>}  ???
     * @description Executes the the SFDX-Falcon Recipe that was compiled during
     *              the SfdxFalconRecipe.read() process.
     * @public @async
     */
    execute(executionOptions?: ExecutionOptions): Promise<SfdxFalconResult>;
    /**
     * @method      onError
     * @param       {SfdxFalconResult}  engineError Required.
     * @returns     {void}
     * @description Handles rejected calls returning from this.executeEngine().
     * @private
     */
    private onError;
    /**
     * @method      onSuccess
     * @param       {SfdxFalconResult}  falconEngineResult Required.
     * @returns     {void}
     * @description Called upon successful return from this.execute().
     * @private
     */
    private onSuccess;
    /**
     * @method      renderSuccessMessage
     * @returns     {void}
     * @description Renders summary output to the Console providing information
     *              about the Recipe that was just executed.
     * @private
     */
    private renderSuccessMessage;
}
