"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
//─────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @file          modules/sfdx-falcon-recipe/index.ts
 * @copyright     Vivek M. Chawla - 2018
 * @author        Vivek M. Chawla <@VivekMChawla>
 * @summary       ???
 * @description   ???
 * @version       1.0.0
 * @license       MIT
 */
//─────────────────────────────────────────────────────────────────────────────────────────────────┘
// Import External Modules
const core = require("@salesforce/core"); // Allows us to use SFDX core functionality.
const path = require("path"); // Why?
// Import Local Modules
const sfdx_falcon_debug_1 = require("../../modules/sfdx-falcon-debug"); // Why?
const sfdx_falcon_error_1 = require("../../modules/sfdx-falcon-error"); // Why?
const sfdx_falcon_result_1 = require("../../modules/sfdx-falcon-result"); // Why?
const demo_config_1 = require("./engines/appx/demo-config"); // Why?
// Import Recipe-Specific Types
const types_1 = require("./types"); // Why?
// Set the File Local Debug Namespace
const dbgNs = 'RECIPE:sfdx-falcon-recipe:';
//─────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @class       SfdxFalconRecipe
 * @summary     Represents an SFDX-Falcon "recipe"
 * @description Use the SfdxFalconRecipe.read() static method to locate an SFDX-Falcon recipe in
 *              the local environment and then provide compile/execute cabability on that recipe.
 * @version     1.0.0
 * @public
 */
//─────────────────────────────────────────────────────────────────────────────────────────────────┘
class SfdxFalconRecipe {
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      read
     * @param       {SfdxFalconProject} sfdxFPO Required. SFDX-Falcon Project
     *              Object associated with the project context that this Recipe
     *              is being read into.
     * @param       {string}  recipeFile  Required. Name of a Recipe.json file
     *              that will be expected to reside inside the /config directory
     *              within the SFDX-Falcon Project.
     * @description ???
     * @public @static @async
     */
    //───────────────────────────────────────────────────────────────────────────┘
    static async read(sfdxFPO, recipeFile) {
        // The recipe path should be the config directory, as specified in the SFDX-Falcon Project.
        const recipePath = sfdxFPO.configPath;
        // Read the SFDX-Falcon Recipe file specified by the caller.
        const recipe = await SfdxFalconRecipe.resolveSfdxFalconRecipe(recipePath, recipeFile);
        // Validate the overall Recipe before continuing.
        try {
            SfdxFalconRecipe.validate(recipe);
        }
        catch (error) {
            const combinedPath = path.join(recipePath, recipeFile);
            throw new sfdx_falcon_error_1.SfdxFalconError(`Recipe file '${combinedPath}' is invalid.  ${error.message}`, `InvalidRecipe`, `${dbgNs}read`, error);
        }
        // Instantiate an SfdxFalconRecipe object so we can populate and return it.
        const sfdxFR = new SfdxFalconRecipe();
        // Initialize members that get thier values from what's in the Recipe File
        sfdxFR._recipeName = recipe.recipeName;
        sfdxFR._description = recipe.description;
        sfdxFR._recipeType = recipe.recipeType;
        sfdxFR._recipeVersion = recipe.recipeVersion;
        sfdxFR._schemaVersion = recipe.schemaVersion;
        sfdxFR._options = recipe.options;
        sfdxFR._recipeStepGroups = recipe.recipeStepGroups;
        sfdxFR._handlers = recipe.handlers;
        // Initialize members that DO NOT get info from the Recipe File
        sfdxFR._validated = true;
        sfdxFR._projectContext = sfdxFPO;
        // Setup the RECIPE Result.
        sfdxFR._falconRecipeResult =
            new sfdx_falcon_result_1.SfdxFalconResult(sfdxFR.recipeName, "RECIPE" /* RECIPE */, { startNow: false,
                bubbleError: true,
                bubbleFailure: true });
        // Setup the shell of the DETAIL for the RECIPE Result.
        sfdxFR._falconRecipeResultDetail = { projectContext: sfdxFR._projectContext,
            recipeEngine: null,
            recipeName: sfdxFR.recipeName,
            recipeDescription: sfdxFR._description,
            recipeType: sfdxFR._recipeType,
            recipeVersion: sfdxFR._recipeVersion,
            recipeSchemaVersion: sfdxFR._schemaVersion,
            recipeOptions: sfdxFR._options,
            recipeStepGroups: sfdxFR._recipeStepGroups,
            recipeHandlers: sfdxFR._handlers };
        // Done with the initial read.  Return the SfdxFalconRecipe we just created.
        return sfdxFR;
    }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      resolveSfdxFalconRecipeFile
     * @param       {string}  recipePath ???
     * @param       {string}  recipeFile ???
     * @returns     {Promise<object>}  ???
     * @description ???
     * @private @static @async
     */
    //───────────────────────────────────────────────────────────────────────────┘
    static async resolveSfdxFalconRecipe(recipePath, recipeFile) {
        // Build a ConfigOptions object that points to the specified Falcon Recipe.
        const configOptions = {
            rootFolder: recipePath,
            filename: recipeFile,
            isGlobal: false,
            isState: false
        };
        const combinedPath = path.join(configOptions.rootFolder, configOptions.filename);
        sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}resolveSfdxFalconRecipe:configOptions:`, configOptions, `configOptions: `);
        // Using the options set above, retrieve the SFDX-Falcon Recipe file.
        const sfdxFalconRecipeFile = await core.ConfigFile.create(configOptions)
            .catch(error => {
            throw new sfdx_falcon_error_1.SfdxFalconError(`Recipe file '${combinedPath}' is invalid.  Reason: ${error.message}`, `InvalidRecipe`, `${dbgNs}resolveSfdxFalconRecipe`, error);
        });
        // Make sure that the file actually exists on disk (as opposed to being created for us).
        if (await sfdxFalconRecipeFile.exists() === false) {
            throw new sfdx_falcon_error_1.SfdxFalconError(`Recipe file '${combinedPath}' does not exist.`, `RecipeNotFound`, `${dbgNs}resolveSfdxFalconRecipe`);
        }
        sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}resolveSfdxFalconRecipe:sfdxFalconRecipeFile:`, sfdxFalconRecipeFile, `sfdxFalconRecipeFile: `);
        // Convert the SFDX-Falcon Recipe file to an object
        const sfdxFalconRecipe = sfdxFalconRecipeFile.toObject();
        sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}resolveSfdxFalconRecipe:sfdxFalconRecipe:`, sfdxFalconRecipe, `sfdxFalconRecipe: `);
        // Done. Return the resolved Recipe object to the caller.
        return sfdxFalconRecipe;
    }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      validate
     * @param       {SfdxFalconRecipeJson}  sfdxFalconRecipeJson Required. Parsed
     *              JSON object that is a direct representation of an SFDX-Falcon
     *              Recipe file as read from the local filesystem.
     * @returns     {void}  Throws an error if the Recipe is found to be invalid.
     * @description Given a Recipe already parsed into a JS Object, takes multiple
     *              actions to ensure that the recipe is valid. Throws an error if
     *              the Recipe is invalid.
     * @version     1.0.0
     * @private @static
     */
    //───────────────────────────────────────────────────────────────────────────┘
    static validate(sfdxFalconRecipeJson) {
        // Validate top-level Falcon Recipe config.
        SfdxFalconRecipe.validateTopLevelProperties(sfdxFalconRecipeJson);
        // Let the specific Recipe Engine take a shot at validation.
        switch (sfdxFalconRecipeJson.recipeType) {
            case types_1.RecipeType.APPX_DEMO:
                demo_config_1.AppxDemoConfigEngine.validateRecipe(sfdxFalconRecipeJson);
                break;
            case types_1.RecipeType.APPX_PACKAGE:
                //AppxPackageConfigEngine.validateRecipe(sfdxFalconRecipeJson);
                break;
            default:
                throw new sfdx_falcon_error_1.SfdxFalconError(`Recipe type '${sfdxFalconRecipeJson.recipeType}' does not exist.`, `InvalidRecipe`, `${dbgNs}validate`);
        }
    }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      validateTopLevelProperties
     * @param       {SfdxFalconRecipeJson}  sfdxFalconRecipe Required. JS Object
     *              that is a direct representation of an SFDX-Falcon Recipe file
     *              as read from the local filesystem.
     * @returns     {void}  Throws Error listing invalid/missing top-level keys.
     * @description Given a Recipe already parsed into a JS Object, throws an
     *              error with a list of top-level keys that are missing or
     *              invalid. Nothing happens validation is successful.
     * @private @static
     */
    //───────────────────────────────────────────────────────────────────────────┘
    static validateTopLevelProperties(sfdxFalconRecipe) {
        // Declare a local variable to hold invalid top-level keys.
        const invalidConfigKeys = new Array();
        // Validate the high-level recipe config
        if (!sfdxFalconRecipe.recipeName)
            invalidConfigKeys.push('recipeName');
        if (!sfdxFalconRecipe.description)
            invalidConfigKeys.push('description');
        if (!sfdxFalconRecipe.recipeType)
            invalidConfigKeys.push('recipeType');
        if (!sfdxFalconRecipe.recipeVersion)
            invalidConfigKeys.push('recipeVersion');
        if (!sfdxFalconRecipe.schemaVersion)
            invalidConfigKeys.push('schemaVersion');
        if (!sfdxFalconRecipe.options)
            invalidConfigKeys.push('options');
        if (!sfdxFalconRecipe.recipeStepGroups)
            invalidConfigKeys.push('recipeStepGroups');
        if (!sfdxFalconRecipe.handlers)
            invalidConfigKeys.push('handlers');
        // Check if any invalid keys were found.
        if (invalidConfigKeys.length > 0) {
            throw new sfdx_falcon_error_1.SfdxFalconError(`Recipe has missing/invalid settings (${invalidConfigKeys}).`, `InvalidRecipe`, `${dbgNs}validateTopLevelProperties`);
        }
    }
    get recipeName() { return this._recipeName; }
    get description() { return this._description; }
    get recipeType() { return this._recipeType; }
    get recipeVersion() { return this._recipeVersion; }
    get schemaVersion() { return this._schemaVersion; }
    get options() { return this._options; }
    get recipeStepGroups() { return this._recipeStepGroups; }
    get handlers() { return this._handlers; }
    get compiled() { return this._compiled; }
    get projectContext() { return this._projectContext; }
    get recipeEngine() { return this._recipeEngine; }
    get falconRecipeResult() { return this._falconRecipeResult; }
    get falconRecipeResultDetail() { return this._falconRecipeResultDetail; } // tslint:disable-line: no-any
    get validated() { return this._validated; }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @constructs  SfdxFalconRecipe
     * @description EMPTY CONSTRUCTOR. SfdxFalconRecipe.read() should be used to
     *              instantiate SFDX Falcon Recipes.
     * @private
     */
    //───────────────────────────────────────────────────────────────────────────┘
    constructor() {
        // Intentionally Empty. Instantiate SfdxFalconRecipe objects with read();
    }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      compile
     * @param       {CompileOptions}  compileOptions Optional. ???
     * @description ???
     * @public @async
     */
    //───────────────────────────────────────────────────────────────────────────┘
    async compile(compileOptions = {}) {
        sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}compile:compileOptions:`, compileOptions, `compileOptions: `);
        // Figure out which Engine to use
        switch (this._recipeType) {
            case types_1.RecipeType.APPX_DEMO:
                this._recipeEngine = await demo_config_1.AppxDemoConfigEngine.compileRecipe(this, compileOptions);
                break;
            case types_1.RecipeType.APPX_PACKAGE:
                //this._recipeEngine = new AppxPackageRecipeEngine(this, compileOptions);
                break;
            default:
                throw new sfdx_falcon_error_1.SfdxFalconError(`Recipe type '${this._recipeType}' is invalid.`, `InvalidRecipe`, `${dbgNs}compile`);
        }
        // If we get here, it means the recipe compiled successfully.
        this._compiled = true;
        // Add the compiled Recipe to the DETAIL that will go with the RECIPE Result.
        this._falconRecipeResultDetail.recipeEngine = this._recipeEngine;
        // Done with compile
        return;
    }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      execute
     * @param       {any}  executionOptions Optional. ???
     * @returns     {Promise<SfdxFalconResult>}  ???
     * @description Executes the the SFDX-Falcon Recipe that was compiled during
     *              the SfdxFalconRecipe.read() process.
     * @public @async
     */
    //───────────────────────────────────────────────────────────────────────────┘
    async execute(executionOptions = {}) {
        // Make sure that the Recipe has been compiled before allowing execution.
        if (this._compiled !== true) {
            throw new Error(`ERROR_RECIPE_NOT_COMPILED: The compile() method must be called before you can  `
                + `execute this recipe`);
        }
        // Ask the Recipe Engine (determined during compile) to execute the recipe.
        await this._recipeEngine.execute(executionOptions)
            .then(engineSuccessResponse => {
            this.onSuccess(engineSuccessResponse);
        })
            .catch(engineFailureResponse => {
            this.onError(engineFailureResponse);
        });
        // Use stardard Debug to show the entire Recipe Response results.
        sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}execute:_falconRecipeResult:`, this._falconRecipeResult, `this._falconRecipeResult: `);
        // Render a "success message" to the user via the Console.
        this.renderSuccessMessage();
        // Return the SFDX-Falcon Recipe Response to the caller.
        return this._falconRecipeResult;
    }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      onError
     * @param       {SfdxFalconResult}  engineError Required.
     * @returns     {void}
     * @description Handles rejected calls returning from this.executeEngine().
     * @private
     */
    //───────────────────────────────────────────────────────────────────────────┘
    onError(engineError) {
        // Make sure any rejected promises are wrapped as an ERROR Result.
        const falconEngineResult = sfdx_falcon_result_1.SfdxFalconResult.wrapRejectedPromise(engineError, "ENGINE" /* ENGINE */, `EngineResult (REJECTED)`);
        // Debug the contents of the RECIPE Result in it's final state.
        sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}onError:falconEngineResult:`, falconEngineResult, `falconEngineResult: `);
        // If the ACTION Result's "bubbleError" is TRUE, addChild() will throw an Error.
        this._falconRecipeResult.addChild(falconEngineResult);
        // Debug the contents of the RECIPE Result in it's final state.
        sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}onError:falconRecipeResult:`, this.falconRecipeResult, `this.falconRecipeResult: `);
        // Throw the ENGINE Result so the caller (likely a COMMAND) knows what happened.
        throw this._falconRecipeResult;
    }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      onSuccess
     * @param       {SfdxFalconResult}  falconEngineResult Required.
     * @returns     {void}
     * @description Called upon successful return from this.execute().
     * @private
     */
    //───────────────────────────────────────────────────────────────────────────┘
    onSuccess(falconEngineResult) {
        // Debug the contents of the Engine Success Response.
        sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}onSuccess:falconEngineResult:`, falconEngineResult, `falconEngineResult: `);
        // Add the successful engine to the Recipe's array of engine success.
        this._falconRecipeResult.addChild(falconEngineResult);
        // Right now, we're only running one Recipe at a time, so mark the Recipe as complete, too.
        this._falconRecipeResult.success();
        // Done.
        return;
    }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      renderSuccessMessage
     * @returns     {void}
     * @description Renders summary output to the Console providing information
     *              about the Recipe that was just executed.
     * @private
     */
    //───────────────────────────────────────────────────────────────────────────┘
    renderSuccessMessage() {
        console.log(`Recipe Executed Successfully`);
        // TODO: Implement any sort of user messaging here (eg. "All done! Command took 135 seconds")
    }
} // End of SfdxFalconRecipe class.
exports.SfdxFalconRecipe = SfdxFalconRecipe;
//# sourceMappingURL=index.js.map