import { SfdxFalconError } from '../sfdx-falcon-error';
/**
 * @interface   SfdxFalconResultDisplayOptions
 * @description Options object used by the displayResult() function.
 */
export interface SfdxFalconResultDisplayOptions {
    displayResult?: boolean;
    contextLabel: string;
    detailInspectDepth: number;
    childInspectDepth: number;
    errorInspectDepth: number;
}
/**
 * @interface   SfdxFalconResultRenderOptions
 * @extends     SfdxFalconResultDisplayOptions
 * @description Options object used by the various Render functions to customize display output.
 */
export interface SfdxFalconResultRenderOptions extends SfdxFalconResultDisplayOptions {
    headerColor: string;
    labelColor: string;
    errorLabelColor: string;
    valueColor: string;
}
/**
 * @interface   SfdxFalconResultOptions
 * @description Represents the options that can be set when an SfdxFalconResult object is constructed.
 */
export interface SfdxFalconResultOptions {
    startNow?: boolean;
    bubbleError?: boolean;
    bubbleFailure?: boolean;
    failureIsError?: boolean;
}
/**
 * @enum        SfdxFalconResultStatus
 * @description Represents the different types of sources where Results might come from.
 */
export declare const enum SfdxFalconResultStatus {
    INITIALIZED = "INITIALIZED",
    WAITING = "WAITING",
    SUCCESS = "SUCCESS",
    FAILURE = "FAILURE",
    WARNING = "WARNING",
    ERROR = "ERROR",
    UNKNOWN = "UNKNOWN"
}
/**
 * @enum        SfdxFalconResultType
 * @description Represents the different types of sources where Results might come from.
 */
export declare const enum SfdxFalconResultType {
    ACTION = "ACTION",
    COMMAND = "COMMAND",
    ENGINE = "ENGINE",
    EXECUTOR = "EXECUTOR",
    FUNCTION = "FUNCTION",
    GENERATOR = "GENERATOR",
    INQUIRER = "INQUIRER",
    LISTR = "LISTR",
    RECIPE = "RECIPE",
    UNKNOWN = "UNKNOWN",
    UTILITY = "UTILITY"
}
/**
 * @class       SfdxFalconResult
 * @description Implements a framework for creating results-driven, informational objects with a
 *              concept of heredity (child results) and the ability to "bubble up" both Errors
 *              (thrown exceptions) and application-defined "failures".
 * @public
 */
export declare class SfdxFalconResult {
    /**
     * @method      isValid
     * @param       {object}  objectToCheck  Required. The object that will be
     *              checked to see if it's a valid SFDX-Falcon Result.
     * @returns     {void}
     * @description Given any object, determines if that object is an SFDX-Falcon
     *              Result and throws an SfdxFalconError if not.
     * @public @static
     */
    static isValid(objectToCheck: object): void;
    /**
     * @method      validate
     * @param       {unknown}  objectToCheck  Required. The object that will be
     *              checked to see if it's a valid SFDX-Falcon Result.
     * @param       {SfdxFalconResultType}  expectedResultType  Required. Type of
     *              SFDX-Falcon Result you expect to find.
     * @param       {SfdxFalconResultStatus} [status] Optional. The status that
     *              you expect the SFDX-Falcon Result to have.
     * @returns     {boolean}  Returns TRUE if the Object to Check matches all
     *              specified criteria, FALSE if otherwise.
     * @description Given an Object to check, an SFDX-Falcon Result Type, and
     *              (optionally) an SFDX-Falcon Result Status, determines if the
     *              Object matches the specified criteria and returns TRUE if
     *              it does.
     * @public @static
     */
    static validate(objectToCheck: unknown, expectedType?: SfdxFalconResultType, expectedStatus?: SfdxFalconResultStatus): boolean;
    /**
     * @method      wrap
     * @param       {unknown}  thingToWrap  Required.  The thing (string, object,
     *              error, whatever) that is being wrapped.
     * @param       {SfdxFalconResultType}  resultType  Required. Type of the new Result.
     * @param       {string}  resultName  Required. The name of the new Result.
     * @param       {string}  [resultSource]  Optional. Defaults to concatenation of
     *              "resultType:resultName" if not provided.
     * @returns     {SfdxFalconResult}  Returns a new SFDX-Falcon Result
     * @description Instantiates an SFDX-Falcon Result object and tries to
     *              populate it as best as possible.
     * @public @static
     */
    static wrap(thingToWrap: unknown, resultType: SfdxFalconResultType, resultName: string, resultSource?: string): SfdxFalconResult;
    /**
     * @method      wrapRejectedPromise
     * @param       {unknown}  rejectedPromiseData  Required.  The data passed
     *              into a catch() function that's hung off of a rejected Promise.
     * @param       {SfdxFalconResultType}  resultType  Required. Type of the new Result.
     * @param       {string}  resultName  Required. Name of the Result.
     * @param       {string}  [resultSource]  Required. Source of the Result.
     * @returns     {SfdxFalconResult}  Returns a new SFDX-Falcon Result
     * @description Tries to figure out what's in the Rejected Promise Data and
     *              then instantiates an SFDX-Falcon Result object and tries to
     *              populate it as best as possible.
     * @public @static
     */
    static wrapRejectedPromise(rejectedPromiseData: unknown, resultType: SfdxFalconResultType, resultName: string, resultSource?: string): SfdxFalconResult;
    /**
     * @method      renderActionDetail
     * @param       {SfdxFalconResult}  result  Required. The Result to render.
     * @param       {SfdxFalconResultRenderOptions}  options  Required. Options
     *              object that determines how the Result is rendered (colors,
     *              obect inspection depth, etc).
     * @returns     {void}
     * @description Given an SFDX-Falcon Result, renders to the console info
     *              that's specifid to SFDX-Falcon Results of type ACTION.
     * @private @static
     */
    private static renderActionDetail;
    /**
     * @method      renderAnyDetail
     * @param       {SfdxFalconResult}  result  Required. The Result to render.
     * @param       {SfdxFalconResultRenderOptions}  options  Required. Options
     *              object that determines how the Result is rendered (colors,
     *              obect inspection depth, etc).
     * @returns     {string}
     * @description Given an SFDX-Falcon Result, renders to the console info
     *              in a way acceptable for any type of SFDX-Falcon Result.
     * @private @static
     */
    private static renderAnyDetail;
    /**
     * @method      renderBaseDetail
     * @param       {SfdxFalconResult}  result  Required. The Result to render.
     * @param       {SfdxFalconResultRenderOptions}  options  Required. Options
     *              object that determines how the Result is rendered (colors,
     *              obect inspection depth, etc).
     * @returns     {string}
     * @description Given an SFDX-Falcon Result, renders to the console all of the
     *              basic information that's common to all SFDX-Falcon Results.
     * @private @static
     */
    private static renderBaseDetail;
    /**
     * @method      renderErrorDetail
     * @param       {SfdxFalconResult}  result  Required. The Result to render.
     * @param       {SfdxFalconResultRenderOptions}  options  Required. Options
     *              object that determines how the Result is rendered (colors,
     *              obect inspection depth, etc).
     * @returns     {string}
     * @description Given an SFDX-Falcon Result, unpacks any errors and renders
     *              detailed error info to the console, going back through any
     *              chained errors that are found to be children of the error
     *              held by the Result passed to this method.
     * @private @static
     */
    private static renderErrorDetail;
    name: string;
    type: SfdxFalconResultType;
    children: SfdxFalconResult[];
    private _detail;
    private _errObj;
    private _bubbleError;
    private _bubbleFailure;
    private _failureIsError;
    private _status;
    private _startTime;
    private _endTime;
    detail: object;
    readonly duration: number;
    readonly durationSecs: number;
    readonly durationString: string;
    readonly errObj: SfdxFalconError;
    readonly resultTypeErrorName: string;
    readonly lastChild: SfdxFalconResult;
    readonly status: SfdxFalconResultStatus;
    /**
     * @constructs  SfdxFalconResult
     * @param       {string}  name  Required. The name of this Result.
     * @param       {SfdxFalconResultType}  type  Required. Type of this Result.
     * @param       {SfdxFalconResultOptions} [options] Optional. Available
     *              Options are "startNow", "bubbleError", "bubbleFailure", and
     *              "failureAsError".
     * @description Constructs an SfdxFalconResult object.
     * @public
     */
    constructor(name: string, type: SfdxFalconResultType, options?: SfdxFalconResultOptions);
    /**
     * @method      setDetail
     * @param       {unknown} [incomingDetail]  Optional. Typically this will be
     *              a string, an unknown type of object, or an Error object.
     * @returns     {void}
     * @description Parses the given result and saves it to this object's "detail"
     *              property.  Will attempt to coerce any non-object data into
     *              an object.
     * @public
     */
    setDetail(incomingDetail?: unknown): void;
    /**
     * @method      start
     * @returns     {this}  Returns "this" instance to support method chaining.
     * @description Sets the start time to NOW if it hasn't been set before.
     * @public
     */
    start(): this;
    /**
     * @method      success
     * @returns     {this}  Returns "this" instance to support method chaining.
     * @description Sets the stop time to NOW and marks the status of this Result
     *              as SUCCESS.
     * @public
     */
    success(): this;
    /**
     * @method      throw
     * @param       {unknown} anyResult  Required. Typically an Error object, but
     *              could be anything. Whatever is provided will end up bound to
     *              an SfdxFalconError via this.error().
     * @returns     {void}  Always throws this instance of SfdxFalconResult.
     * @description Forwards the contents of anyResult to this.error() which sets
     *              the stop time to NOW, marks status as ERROR, and stores the
     *              provided result as an Error object then throws this.
     * @public
     */
    throw(anyResult: unknown): void;
    /**
     * @method      warning
     * @returns     {this}  Returns "this" instance to support method chaining.
     * @description Sets the stop time to NOW and marks status as WARNING.
     * @public
     */
    warning(): this;
    /**
     * @method      unknown
     * @returns     {this}  Returns "this" instance to support method chaining.
     * @description Sets the stop time to NOW and marks status as UNKNOWN.
     * @public
     */
    unknown(): this;
    /**
     * @method      addChild
     * @param       {SfdxFalconResult}  childResult  Required.
     * @param       {boolean} [requireValidChildResult] Optional. If set to TRUE,
     *              calls to this method will throw an Error if the value passed
     *              to childResult is not a valid SfdxFalconResult object.
     * @returns     {this}  Returns "this" instance to support method chaining.
     * @description Given an SFDX-Falcon Result, inspect the result then add it
     *              to the children array. If the child result is a FAILURE or
     *              ERROR, bubble this up by marking this as FAILURE or ERROR if
     *              bubbleFailure or bubbleError are TRUE.
     * @public
     */
    addChild(childResult: SfdxFalconResult, requireValidChildResult?: boolean): this;
    /**
     * @method      addRejectedChild
     * @param       {unknown} rejectedPromise  Required.  The data passed into
     *              a catch() function that's hung off of a rejected Promise.
     * @param       {SfdxFalconResultType}  resultType  Required. Type of the new
     *              Child Result. Only used if the contents of rejectedPromise
     *              are not already an SfdxFalconResult.
     * @param       {string}  resultName  Required. The name of the new Result.
     *              Only used if the contents of rejectedPromise are not
     *              already an SfdxFalconResult.
     * @param       {string}  [resultSource]  Optional. The source of the
     *              Rejected Child.  Defaults to a concatenation of resultType
     *              and resultName if not provided.
     * @returns     {this}  Returns the current instance of SfdxFalconResult.
     * @description Wraps the data from a Rejected Promise as an SfdxFalconResult
     *              and adds the new Result as a Child of this Result.
     * @public
     */
    addRejectedChild(rejectedPromise: unknown, resultType: SfdxFalconResultType, resultName: string, resultSource?: string): this;
    /**
     * @method      addResolvedChild
     * @param       {unknown} resolvedPromise  Required.  The data passed into
     *              a then() function that's hung off of a resolved Promise.
     * @param       {SfdxFalconResultType}  resultType  Required. Type of the new
     *              Child Result. Only used if the contents of resolvedPromise
     *              are not already an SfdxFalconResult.
     * @param       {string}  resultName  Required. The name of the new Result.
     *              Only used if the contents of resolvedPromiseData are not
     *              already an SfdxFalconResult.
     * @param       {string}  [resultSource]  Optional. The source of the
     *              Resolved Child.  Defaults to a concatenation of resultType
     *              and resultName if not provided.
     * @returns     {this}  Returns the current instance of SfdxFalconResult.
     * @description Wraps the data from a Rejected Promise as an SfdxFalconResult
     *              and adds the new Result as a Child of this Result.
     * @public
     */
    addResolvedChild(resolvedPromise: unknown, resultType: SfdxFalconResultType, resultName: string, resultSource?: string): this;
    /**
     * @method      debugResult
     * @param       {string}  [contextLabel]  Optional. Adds a string inside
     *              parenthesis to the immediate right of the Result Label.
     * @param       {string}  [debugNamespace]  Optional. Defines the Debug
     *              Namespace that should be used for debugging this call.
     * @param       {number}  [childInspectDepth]   Optional. Overrides the
     *              default setting for inspection depth of Child Results.
     * @param       {number}  [detailInspectDepth]  Optional. Overrides the
     *              default setting for inspection depth of Detail Results.
     * @param       {number}  [errorInspectDepth]   Optional. Overrides the
     *              default setting for inspection depth of stored Errors.
     * @returns     {void}
     * @description Calls this.renderResult() to generate a complete, formatted
     *              set of information about this Result and displays that result
     *              to the user via SfdxFalconDebug (guranteed output).
     * @public
     */
    debugResult(contextLabel?: string, debugNamespace?: string, childInspectDepth?: number, detailInspectDepth?: number, errorInspectDepth?: number): void;
    /**
     * @method      displayErrorDebugInfo
     * @param       {boolean} showErrorDebug  Required. Determines if extended
     *              debugging output for the terminating Error can be shown.
     * @param       {boolean} promptUser  Required. Determines if the user will
     *              be prompted to display debug info. If FALSE, debug info will
     *              be shown without requiring additional user input.
     * @returns     {Promise<void>}
     * @description Makes the final determination of what the Debug Display
     *              options are. May prompt the user interactively to make this
     *              determination.
     * @public @async
     */
    displayErrorDebugInfo(showErrorDebug: boolean, promptUser: boolean): Promise<void>;
    /**
     * @method      displayResult
     * @param       {SfdxFalconResultDisplayOptions}  [displayOptions]  Optional.
     *              Sets all the display options for the displayResult() function.
     * @returns     {void}
     * @description Calls this.renderResult() to generate a complete, formatted
     *              set of information about this Result and displays that result
     *              to the user via console.log().
     * @public
     */
    displayResult(displayOptions?: SfdxFalconResultDisplayOptions): void;
    /**
     * @method      error
     * @param       {Error|unknown} errorObject  Required.
     * @returns     {this}  Returns "this" instance to support method chaining.
     * @description Sets the stop time to NOW, marks status as ERROR, sets this
     *              Result's detail to an empty object, and stores the provided
     *              Error object as a Falcon Error as this Result's errObj.
     * @public
     */
    error(errorObject: Error | unknown): this;
    /**
     * @method      failure
     * @param       {Error|unknown} errorObject Required.
     * @returns     {this}  Returns "this" instance to support method chaining.
     * @description Sets the stop time to NOW, marks status as FAILURE, and stores
     *              the provided "failure detail" as the detail for this result.
     * @public
     */
    failure(errorObject: Error | unknown): this;
    /**
     * @method      finish
     * @param       {SfdxFalconResultStatus}  finalStatus Required.
     * @returns     {SfdxFalconResultStatus}  This Result's actual final status.
     * @description Sets the stop time to NOW if it hasn't been set before and
     *              sets the final status to whatever is passed in, as long as
     *              the incoming status is of higher priority than the current
     *              status.  Priorities are:
     *                6 - ERROR
     *                5 - FAILURE
     *                4 - WARNING
     *                3 - SUCCESS
     *                2 - UNKNOWN
     *                1 - WAITING
     *                0 - INITIALIZED
     * @private
     */
    private finish;
    /**
     * @method      renderResult
     * @param       {string}  [contextLabel]  Optional. Adds a string inside
     *              parenthesis to the immediate right of the Result Label.
     * @param       {number}  [childInspectDepth]   Optional. Overrides the
     *              default setting for inspection depth of Child Results.
     * @param       {number}  [detailInspectDepth]  Optional. Overrides the
     *              default setting for inspection depth of Detail Results.
     * @param       {number}  [errorInspectDepth]   Optional. Overrides the
     *              default setting for inspection depth of stored Errors.
     * @returns     {string}  Returns string to be rendered by console.log() or
     *              debug().
     * @description Generates a string of completely formatted output that's ready
     *              for display to the user via console.log() or debug(). Relies
     *              on the caller to decide how to actually display to the user.
     * @private
     */
    private renderResult;
}
