import { ShellString } from 'shelljs';
import { ShellExecResult } from '../../modules/sfdx-falcon-types';
/**
 * @function    gitClone
 * @param       {string}  gitRemoteUri  Required. URI of the Git Remote to clone.
 * @param       {string}  targetDirectory Required. Local path into which the repo will be cloned.
 * @param       {string}  repoDirectory Required. Name of the local directory the repo is cloned
 *              into. If not provided, this defaults to the name of the Repo (part of the URI).
 * @returns     {void}  No return value. Will throw Error if any problems.
 * @description Clones a Git repository located at gitRemoteUri to the local machine inside of the
 *              directory specified by targetDirectory.
 * @public
 */
export declare function gitClone(gitRemoteUri: string, targetDirectory?: string, repoDirectory?: string): Promise<ShellExecResult>;
/**
 * @function    gitInit
 * @param       {string}  targetDirectory Location where the git command will be run
 * @returns     {ShellString} Returns a ShellString object, containing code, stdout, and stderr.
 *              Will throw an Error if there are any problems prior to executing "git init".
 * @description Initializes Git at the location specified by targetDirectory.  Note that there are
 *              no adverse effects if gitInit is called on the same location more than once.
 * @public
 */
export declare function gitInit(targetDirectory: string): ShellString;
/**
 * @function    gitAdd
 * @param       {string}  targetDirectory Required. Location where the git command will be run
 * @returns     {ShellString}
 * @description Executes "git add -A" inside of the target directory.
 * @public
 */
export declare function gitAdd(targetDirectory: string): ShellString;
/**
 * @function    gitAddAndCommit
 * @param       {string}  targetDirectory Required. Location where the git command will be run
 * @param       {string}  commitMessage   Required. String to be used as the commit message
 * @returns     {void}  No return value. Will throw Error if any problems.
 * @description Executes "git add -A" and "git commit" inside of the target directory.  For the
 *              commit, it adds the message passed in via commitMessage.
 * @public
 */
export declare function gitAddAndCommit(targetDirectory: string, commitMessage: string): void;
/**
 * @function    gitAddAndCommit
 * @param       {string}  targetDirectory Required. Location where the git command will be run
 * @param       {string}  commitMessage   Required. String to be used as the commit message
 * @returns     {ShellString}
 * @description Executes "git commit" inside of the target directory.  For the commit, it adds the
 *              message passed in via commitMessage.
 * @public
 */
export declare function gitCommit(targetDirectory: string, commitMessage: string): ShellString;
/**
 * @function    gitRemoteAddOrigin
 * @param       {string}  targetDirectory Required. Location where the git command will be run
 * @param       {string}  gitRemoteUri  Required. URI of the Git Remote to be added as origin.
 * @returns     {ShellString} Result of a call to shell.exec().
 * @description Executes "git remote add origin" inside of the target directory, connecting the
 *              repo to the Remote specified by gitRemoteUri.
 * @public
 */
export declare function gitRemoteAddOrigin(targetDirectory: string, gitRemoteUri: string): ShellString;
/**
 * @function    isGitInstalled
 * @returns     {boolean} TRUE if the Git executable is installed and available
 * @description Determines if Git has been installed on the user's local machine and if the
 *              executable has been added to the path.
 * @public
 */
export declare function isGitInstalled(): boolean;
/**
 * @function    getRepoNameFromRemoteUri
 * @param       {string}  gitRemoteUri  Required. URI of the Git Remote Repository to be checked.
 * @returns     {string}  Repository name as parsed from the Git Remote URI.
 * @description Given a Git Remote URI, parses that string and extracts the name of its associated
 *              Git repo. For example, "https://github.com/sfdx-isv/testbed-falcon-apk.git" would
 *              return the string "testbed-falcon-apk".
 * @public
 */
export declare function getRepoNameFromUri(gitRemoteUri: string): string;
/**
 * @function    isGitRemoteEmpty
 * @param       {string}  gitRemoteUri  Required. URI of the Git Remote Repository to be checked.
 * @returns     {boolean} FALSE if gitRemoteUri points to a remote repo that is reachable and
 *              readable by the current user AND that has at least one commit.
 * @description Determines if the URI provided points to a remote repo that is reachable and
 *              readable by the currently configured Git user AND that it has at least one commit.
 * @public
 */
export declare function isGitRemoteEmpty(gitRemoteUri: string): boolean;
/**
 * @function    checkGitRemoteStatus
 * @param       {string}  gitRemoteUri  Required. URI of the Git Remote Repository to be checked.
 * @param       {number}  [waitSecs=0]  Optional. Number of seconds of delay to add before the Git
 *              shell command is executed.
 * @returns     {Promise<ShellExecResult>}  Returns an object comprised of code, stdout, stderr,
 *              and a custom message with both resolve() and reject() paths.
 * @description Async version of a function that determines if the URI provided points to a remote
 *              repo that is reachable and readable by the currently configured Git user AND that
 *              it has at least one commit.
 * @public @async
 */
export declare function checkGitRemoteStatus(gitRemoteUri: string, waitSeconds?: number): Promise<ShellExecResult>;
/**
 * @function    isGitRemoteReadable
 * @param       {string}  gitRemoteUri
 * @returns     {boolean} True if gitRemoteUri is a valid Git Remote URI.
 * @description Determines if the URI provided points to a remote repo that is reachable and
 *              readable by the currently configured Git user.
 * @public
 */
export declare function isGitRemoteReadable(gitRemoteUri: string): boolean;
/**
 * @function    isGitUriValid
 * @param       {string}  gitRemoteUri  Required. Git Remote URI to validate
 * @param       {RegExp}  [acceptedProtocols] Optional. RegExp that matches only certain protocols.
 * @returns     {boolean}     TRUE if gitRemoteUri is a syntactically valid Git Remote URI.
 * @description Determines if the URI provided is a syntactically valid Git Remote URI. The accepted
 *              protocols are ssh:, git:, http:, and https:.
 * @public
 */
export declare function isGitUriValid(gitRemoteUri: string, acceptedProtocols?: RegExp): boolean;
