"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
//─────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @file          modules/sfdx-falcon-util/index.ts
 * @copyright     Vivek M. Chawla - 2018
 * @author        Vivek M. Chawla <@VivekMChawla>
 * @summary       SFDX-Falcon Utility Module
 * @description   Exports functions that provide common, helpful utility logic.
 * @version       1.0.0
 * @license       MIT
 */
//─────────────────────────────────────────────────────────────────────────────────────────────────┘
// Import External Modules
const core_1 = require("@salesforce/core"); // Module. SFDX Core library.
const path = require("path"); // Module. Node's path library.
// Import Local Modules
const sfdx_falcon_debug_1 = require("../sfdx-falcon-debug"); // Class. Internal Debug module
const sfdx_falcon_error_1 = require("../sfdx-falcon-error"); // Class. Provides custom Error structures for SFDX-Falcon.
// Require Modules
const uuid = require('uuid/v1'); // Generates a timestamp-based UUID
// Set the File Local Debug Namespace
const dbgNs = 'UTILITY:general:';
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    createUniqueUsername
 * @param       {string}  baseUsername  The starting point for the username.  It should already be
 *                                      in the form of an email, eg 'name@domain.org'.
 * @returns     {string}  Returns the baseUsername with a pseudo-uuid appended to the end.
 * @description Given a base username to start with (eg. 'name@domain.org'), returns what should be
 *              a globally unique username with a pseudo-uuid appended the end of the username base.
 * @version     1.0.0
 * @public
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function createUniqueUsername(baseUsername) {
    const usernameMaxLength = 35;
    if (typeof baseUsername === 'undefined')
        throw new sfdx_falcon_error_1.SfdxFalconError(`Function createUniqueUsername() expects a value for baseUsername but got undefined`, `InvalidArgument`);
    if (baseUsername.length > usernameMaxLength)
        throw new sfdx_falcon_error_1.SfdxFalconError(`Username can not be longer than ${usernameMaxLength} chars to keep room for appending a UUID`, `InvalidUsername`);
    return baseUsername + uuid();
}
exports.createUniqueUsername = createUniqueUsername;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    readConfigFile
 * @param       {string}  rootFolder  Required. The root folder where the config file is stored.
 * @param       {string}  filename    Required. The name of the config file.
 * @returns     {Promise<any>}    Resolves by returning the config file contents as a JS Object.
 * @description Given a path and filename, attempts to load the contents of the config file and
 *              convert them into a JavaScript Object.
 * @version     1.0.0
 * @public @async
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
async function readConfigFile(rootFolder, filename) {
    // Combine rootFolder and filename to get a complete path
    const filePath = path.join(rootFolder, filename);
    // Get the DemoConfigJson file that (should be) referenced in project config.
    const configFileOptions = {
        rootFolder: rootFolder,
        filename: filename,
        isGlobal: false,
        isState: false
    };
    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}readConfigFile:configFileOptions:`, configFileOptions, `configFileOptions: `);
    // Retrieve the config file specified by the Config File Options.
    const configFile = await core_1.ConfigFile.create(configFileOptions);
    // Verify that the file exists before trying to parse it.
    if (await configFile.exists() === false) {
        throw new sfdx_falcon_error_1.SfdxFalconError(`Config file does not exist - ${filePath}`, `FileNotFound`);
    }
    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}readConfigFile:configFile:`, configFile, `configFile: `);
    // Parse the Demo Build Config File to get a Demo Build Sequence object.
    return configFile.toObject();
}
exports.readConfigFile = readConfigFile;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    safeParse
 * @param       {any} contentToParse  Required. The content to be parsed.
 * @returns     {object}  A JavaScript object based on the content to parse.
 * @description Given any content to parse, returns a JavaScript object based on that content. If
 *              the content is not parseable, it is returned as an object with one key: unparsed.
 * @public
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function safeParse(contentToParse) {
    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}safeParse:contentToParse:`, { contentToParse: contentToParse }, `contentToParse: `);
    try {
        return JSON.parse(contentToParse);
    }
    catch (e) {
        return { unparsed: `${contentToParse}` };
    }
}
exports.safeParse = safeParse;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    findJson
 * @param       {string} contentToSearch  Required. A string buffer that may contain JSON.
 * @returns     {JsonMap}  A parsed JavaScript object found in the string buffer, or NULL.
 * @description Given any string buffer, search that buffer to find a single JSON object. If
 *              a parseable object is found, it is returned as an object. Otherwise returns NULL.
 * @public
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function findJson(contentToSearch) {
    sfdx_falcon_debug_1.SfdxFalconDebug.str(`${dbgNs}findJson:contentToSearch:`, contentToSearch, `contentToSearch: `);
    const possibleJson = contentToSearch.substring(contentToSearch.indexOf('{'), contentToSearch.lastIndexOf('}') + 1);
    let foundJson = safeParse(possibleJson);
    if (foundJson.hasOwnProperty('unparsed')) {
        foundJson = null;
    }
    return foundJson;
}
exports.findJson = findJson;
//# sourceMappingURL=index.js.map