/**
 * @file          modules/sfdx-falcon-util/inquirer-questions.ts
 * @copyright     Vivek M. Chawla - 2019
 * @author        Vivek M. Chawla <@VivekMChawla>
 * @summary       Exports several functions that create commonly used Inquirer Questions.
 * @description   Helps developers building Inquirer based interviews (including Yeoman) by exporting
 *                several functions that create a suite of commonly used Inquirer Question objects.
 *                there are also aggregator functions that expose pre-built collections of certain
 *                Inquirer Questions.
 * @version       1.0.0
 * @license       MIT
 */
import { Question } from '../sfdx-falcon-types';
import { Questions } from '../sfdx-falcon-types';
import { YeomanChoice } from '../sfdx-falcon-types';
/**
 * @function    chooseDevHub
 * @param       {YeomanChoice[]}  [devHubChoices]  Optional. Array of Dev Hub choices.
 * @returns     {Question}  A single Inquirer Question.
 * @description Prompts the user to select a Dev Hub from the list provided by devHubChoices.
 * @public
 */
export declare function chooseDevHub(devHubChoices?: YeomanChoice[]): Question;
/**
 * @function    chooseEnvHub
 * @param       {YeomanChoice[]}  [envHubChoices]  Optional. Array of Environment Hub choices.
 * @returns     {Question}  A single Inquirer Question.
 * @description Prompts the user to select an Environment Hub from the list in envHubChoices.
 * @public
 */
export declare function chooseEnvHub(envHubChoices?: YeomanChoice[]): Question;
/**
 * @function    choosePkgOrg
 * @param       {YeomanChoice[]}  [pkgOrgChoices]  Optional. Array of Packaging Org choices.
 * @returns     {Question}  A single Inquirer Question.
 * @description Prompts the user to select a Packaging Org from the list in pkgOrgChoices.
 * @public
 */
export declare function choosePkgOrg(pkgOrgChoices?: YeomanChoice[]): Question;
/**
 * @function    choosePkgProjectType
 * @param       {YeomanChoice[]}  [pkgProjectTypeChoices]  Optional. Array of Project Type choices.
 * @returns     {Questions}  A group of Inquirer Questions.
 * @description Prompts the user to select a Packaging Project Type from the list provided.
 * @public
 */
export declare function choosePkgProjectType(pkgProjectTypeChoices?: YeomanChoice[]): Questions;
/**
 * @function    confirmNoDevHub
 * @returns     {Questions}  An array of Inquirer Question objects.
 * @description Warns the user that a Developer Hub must be selected if they want to continue.
 * @public
 */
export declare function confirmNoDevHub(): Questions;
/**
 * @function    confirmNoEnvHub
 * @returns     {Questions}  An array of Inquirer Question objects.
 * @description Suggest to the user that an Environment Hub might be useful.
 * @public
 */
export declare function confirmNoEnvHub(): Questions;
/**
 * @function    confirmNoGitHubRepo
 * @returns     {Questions}  An array of Inquirer Question objects.
 * @description Warns the user that specifying a Git Remote is strongly recommended.
 * @public
 */
export declare function confirmNoGitHubRepo(): Questions;
/**
 * @function    confirmNoPkgOrg
 * @returns     {Questions} A group of Inquirer Questions.
 * @description Warns the user that a Packaging Org must exist if they want to continue.
 * @public
 */
export declare function confirmNoPkgOrg(): Questions;
/**
 * @function    confirmNoPkgOrgConnection
 * @returns     {Questions} A group of Inquirer Questions.
 * @description Warns the user that they must choose a Packaging Org that is connected to their CLI.
 * @public
 */
export declare function confirmNoPkgOrgConnection(): Questions;
/**
 * @function    confirmProceedRestart
 * @returns     {Questions} A group of Inquirer Questions.
 * @description Asks the user to confirm that they want to proceed with an operation based on the
 *              values that they have previously provided during an Interview.  If they say "no",
 *              they will be asked if they want to restart.  If they choose not to restart, they
 *              are effectively aborting the operation.
 * @public
 */
export declare function confirmProceedRestart(): Questions;
/**
 * @function    provideDeveloperInfo
 * @returns     {Questions}  An array of Inquirer Question objects.
 * @description Asks the user to provide information about the company or individual developer who
 *              owns the project.
 * @public
 */
export declare function provideDeveloperInfo(): Questions;
/**
 * @function    provideGitRemote
 * @returns     {Questions}  An array of Inquirer Question objects.
 * @description Asks the user if they have a Git remote and to provide the URI if they do.
 * @public
 */
export declare function provideGitRemote(): Questions;
/**
 * @function    provideProjectInfo
 * @returns     {Questions}  An array of Inquirer Question objects.
 * @description Asks the user to provide information about the project being created.
 * @public
 */
export declare function provideProjectInfo(): Questions;
/**
 * @function    provideTargetDirectory
 * @returns     {Questions}  An array of Inquirer Question objects.
 * @description Asks the user to provide a target directory for a project being cloned or created.
 * @public
 */
export declare function provideTargetDirectory(): Questions;
