/**
 * @file          modules/sfdx-falcon-util/jsforce.ts
 * @copyright     Vivek M. Chawla - 2018
 * @author        Vivek M. Chawla <@VivekMChawla>
 * @summary       Utility Module - JSForce
 * @description   Utility functions related to JSForce. Allows developer to work directly against
 *                any Salesforce Org that is connected to the local CLI environment.
 * @version       1.0.0
 * @license       MIT
 */
import { Connection } from '@salesforce/core';
import { AnyJson } from '@salesforce/ts-types';
import { AliasOrConnection } from '../sfdx-falcon-types';
import { MetadataPackage } from '../sfdx-falcon-types';
import { ObjectDescribe } from '../sfdx-falcon-types';
import { QueryResult } from '../sfdx-falcon-types';
import { RestApiRequestDefinition } from '../sfdx-falcon-types';
/**
 * @function    changePassword
 * @param       {AliasOrConnection} aliasOrConnection  Required. Either a string containing the
 *              Alias of the org being queried or an authenticated JSForce Connection object.
 * @param       {string}  userId Required. Id of the user whose password is being changed.
 * @param       {string}  newPassword Required. The new password.
 * @returns     {Promise<void>}  Resolves void if successful, throws error if failure.
 * @description Given a User Id, changes a password in the target/connected org.
 * @version     1.0.0
 * @public @async
 */
export declare function changePassword(aliasOrConnection: AliasOrConnection, userId: string, newPassword: string): Promise<void>;
/**
 * @function    createSfdxOrgConfig
 * @param       {AliasOrConnection} aliasOrConnection  Required. Either a string containing an Alias
 *              to or an authenticated JSForce Connection object of the org the user is a part of.
 * @param       {string}  username  Required. Username we are creating an SFDX connection for.
 * @param       {string}  password  Required. Password of the user.
 * @param       {string}  orgAlias  Required. Alias to be set for this user.
 * @returns     {Promise<void>} Resolves void if successful. TOOO: This will likely be updated.
 * @description TODO: This function needs to be updated once we figure out how to persist the authinfo.
 *              Borrowed from https://github.com/wadewegner/sfdx-waw-plugin/blob/master/commands/auth_username_login.js
 * @version     1.0.0
 * @public @async
 */
export declare function createSfdxOrgConfig(aliasOrConnection: AliasOrConnection, username: string, password: string, orgAlias: string): Promise<void>;
/**
 * @function    describeSignupRequest
 * @param       {AliasOrConnection} aliasOrConnection  Required. Either a string containing the
 *              Alias of the org being queried or an authenticated JSForce Connection object.
 * @returns     {Promise<ObjectDescribe>}  Resolves with an Object Describe REST result.
 * @description Given an Org Alias or a JSForce Connection, tries to get an "Object Describe" back
 *              for the SignupRequest SObject. This object should only be present and createable if
 *              the connected org has the Signup Request API enabled.
 * @public @async
 */
export declare function describeSignupRequest(aliasOrConnection: AliasOrConnection): Promise<ObjectDescribe>;
/**
 * @function    getAssignedPermsets
 * @param       {string|Connection} aliasOrConnection  Required. Either a string containing the
 *              Alias of the org being queried or an authenticated JSForce Connection object.
 * @param       {string}  userId  Required. Id of the user whose permsets we are getting.
 * @returns     {Promise<Array<string>>}  Resolves with an Array of Permset Ids assigned to the user.
 * @description Given a User ID, return an Array of Permset Ids that are assigned to that user.
 * @public @async
 */
export declare function getAssignedPermsets(aliasOrConnection: string | Connection, userId: string): Promise<string[]>;
/**
 * @function    getPackages
 * @param       {AliasOrConnection} aliasOrConnection  Required. Either a string containing the
 *              Alias of the org being queried or an authenticated JSForce Connection object.
 * @returns     {Promise<QueryResult<MetadataPackage>>}  Resolves with a Query Result containing
 *              Metadata Package records for each package developed in the target org, Include the
 *              related Metadata Package Version child records for each package.
 * @description Given an Org Alias or a JSForce Connection, queries the related org and returns a
 *              QueryResult containing the MetadataPackage objects and their child objects.  This has
 *              the effect of detailing all of the packages (managed and unmanaged) that are
 *              developed in that org.
 * @public @async
 */
export declare function getPackages(aliasOrConnection: AliasOrConnection): Promise<QueryResult<MetadataPackage>>;
/**
 * @function    getProfileId
 * @param       {AliasOrConnection} aliasOrConnection Required. Either a string containing the Alias
 *              of the org being queried or an authenticated JSForce Connection object.
 * @param       {string}  profileName Required. Name of the profile being searched for.
 * @returns     {Promise<string>}  Resolves with a string containing the xx-character record ID of
 *              the named profile. Rejects if no matching profile can be found.
 * @description Given a Profile Name, returns the xx-character record ID of the named profile.
 * @public @async
 */
export declare function getProfileId(aliasOrConnection: AliasOrConnection, profileName: string): Promise<string>;
/**
 * @function    getUserId
 * @param       {AliasOrConnection} aliasOrConnection Required. Either a string containing the Alias
 *              of the org being queried or an authenticated JSForce Connection object.
 * @param       {string}  username    Required. Name of the user being searched for.
 * @returns     {Promise<string>}  Resolves with a string containing the xx-character record ID of
 *              the named user. Rejects if no matching user can be found.
 * @description Given a Username, returns the xx-character record ID of the named user.
 * @public @async
 */
export declare function getUserId(aliasOrConnection: AliasOrConnection, username: string): Promise<string>;
/**
 * @function    restApiRequest
 * @param       {RestApiRequestDefinition}  restRequestDef  Required. ???
 * @returns     {Promise<any>}  Result of a REST API request to Salesforce.
 * @description Given a REST API Request Definition, makes a REST call using JSForce.
 * @version     1.0.0
 * @public @async
 */
export declare function restApiRequest(restRequestDef: RestApiRequestDefinition): Promise<AnyJson>;
/**
 * @function    toolingApiQuery
 * @param       {AliasOrConnection} aliasOrConnection  Required. Either a string containing the
 *              Alias of the org being queried or an authenticated JSForce Connection object.
 * @param       {string}  query  Required. Tooling API query to be executed.
 * @returns     {Promise<any>}  Result of a Tooling API SObject query to Salesforce.
 * @description Given an Org Alias or JSForce Connection, makes a REST call to the target org's
 *              tooling API to query about a particular Tooling object.
 * @public @async
 */
export declare function toolingApiQuery<T>(aliasOrConnection: AliasOrConnection, query: string): Promise<QueryResult<T>>;
