"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
//─────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @file          modules/sfdx-falcon-util/listr-tasks.ts
 * @copyright     Vivek M. Chawla - 2019
 * @author        Vivek M. Chawla <@VivekMChawla>
 * @summary       Exports several functions that create general-purpose Listr Task objects.
 * @description   Helps developers building setup or initialization tasks with Listr by exporting
 *                several functions that create a suite of commonly used Listr Task objects.  There
 *                are also aggregator functions that expose pre-built collections of certain Listr
 *                tasks.
 * @version       1.0.0
 * @license       MIT
 */
//─────────────────────────────────────────────────────────────────────────────────────────────────┘
// Import External Libraries & Modules
const lodash_1 = require("lodash"); // Useful function for detecting empty objects.
const path = require("path"); // Helps resolve local paths at runtime.
const rxjs_1 = require("rxjs"); // Class. Used to communicate status with Listr.
// Import Internal Libraries
const sfdxHelper = require("../sfdx-falcon-util/sfdx"); // Library of SFDX Helper functions specific to SFDX-Falcon.
const yoHelper = require("../sfdx-falcon-util/yeoman"); // Library of Yeoman Helper functions specific to SFDX-Falcon.
const gitHelper = require("./git"); // Library of Git Helper functions specific to SFDX-Falcon.
const zipHelper = require("./zip"); // Library of Zip Helper functions.
// Import Internal Classes & Functions
const sfdx_falcon_debug_1 = require("../sfdx-falcon-debug"); // Class. Specialized debug provider for SFDX-Falcon code.
const sfdx_falcon_error_1 = require("../sfdx-falcon-error"); // Class. Extends SfdxError to provide specialized error structures for SFDX-Falcon modules.
const sfdx_falcon_notifications_1 = require("../sfdx-falcon-notifications"); // Class. Manages progress notifications inside Falcon.
const sfdx_falcon_result_1 = require("../sfdx-falcon-result"); // Class. Implements a framework for creating results-driven, informational objects with a concept of heredity (child results) and the ability to "bubble up" both Errors (thrown exceptions) and application-defined "failures".
const async_1 = require("../sfdx-falcon-util/async"); // Function. Simple helper that introduces a delay when called inside async functions using "await".
// Requires
const falconUpdateRenderer = require('falcon-listr-update-renderer'); // Custom renderer for Listr
const listr = require('listr'); // Provides asynchronous list with status of task completion.
// Set the File Local Debug Namespace
const dbgNs = 'UTILITY:listr-tasks:';
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    addGitRemote
 * @param       {string}  targetDir Required. Location where the git command will be run
 * @param       {string}  gitRemoteUri  Required. URI of the Git Remote to be added as origin.
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that adds the provided Git Remote as the
 *              origin remote.
 * @public
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function addGitRemote(targetDir, gitRemoteUri) {
    // Make sure the calling scope has access to Shared Data.
    validateSharedData.call(this);
    // Build and return a Listr Task.
    return {
        title: `Adding the Git Remote...`,
        enabled: () => (typeof targetDir === 'string' && targetDir !== '' && typeof gitRemoteUri === 'string' && gitRemoteUri !== ''),
        skip: (listrContext) => {
            if (listrContext.gitInstalled !== true) {
                return true;
            }
            if (listrContext.gitRemoteIsValid !== true) {
                return 'Git Remote is Invalid';
            }
        },
        task: (listrContext, thisTask) => {
            return new rxjs_1.Observable(observer => {
                // Initialize an OTR (Observable Task Result).
                const otr = initObservableTaskResult(`${dbgNs}addGitRemote`, listrContext, thisTask, observer, this.sharedData, this.generatorResult, `Adding the Git Remote ${gitRemoteUri} to the local repository`);
                // Define the Task Logic to be executed.
                const asyncTask = async () => {
                    const shellString = gitHelper.gitRemoteAddOrigin(targetDir, gitRemoteUri);
                    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}addGitRemote:shellString:`, shellString, `shellString: `);
                    return shellString;
                };
                // Execute the Task Logic.
                asyncTask()
                    .then(async (result) => {
                    await async_1.waitASecond(3);
                    thisTask.title += 'Done!';
                    listrContext.gitRemoteAdded = true;
                    finalizeObservableTaskResult(otr);
                })
                    .catch(async (error) => {
                    await async_1.waitASecond(3);
                    thisTask.title += 'Failed';
                    listrContext.gitRemoteAdded = false;
                    finalizeObservableTaskResult(otr, error);
                });
            });
        }
    };
}
exports.addGitRemote = addGitRemote;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    buildDevHubAliasList
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that takes a list of identified Dev Hubs
 *              and uses it to create an Inquirer-compatible "choice list". This function must be
 *              executed using the call() method because it relies on the caller's "this" context
 *              to properly function.
 * @public
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function buildDevHubAliasList() {
    // Make sure the calling scope has a valid context variable.
    validateSharedData.call(this);
    // Build and return a Listr Task.
    return {
        title: 'Building DevHub Alias List...',
        enabled: () => Array.isArray(this.sharedData.devHubAliasChoices),
        task: (listrContext, thisTask) => {
            // DEBUG
            sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}buildDevHubAliasList:listrContext.devHubOrgInfos:`, listrContext.devHubOrgInfos, `listrContext.devHubOrgInfos: `);
            // Build a list of Choices based on the DevHub org infos.
            this.sharedData.devHubAliasChoices = yoHelper.buildOrgAliasChoices(listrContext.devHubOrgInfos);
            // Add a separator and a "not specified" option
            this.sharedData.devHubAliasChoices.push(new yoHelper.YeomanSeparator());
            this.sharedData.devHubAliasChoices.push({ name: 'My DevHub Is Not Listed Above', value: 'NOT_SPECIFIED', short: 'Not Specified' });
            thisTask.title += 'Done!';
            return;
        }
    };
}
exports.buildDevHubAliasList = buildDevHubAliasList;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    buildEnvHubAliasList
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that takes a list of identified Environment
 *              Hubs and uses it to create an Inquirer-compatible "choice list". This function must
 *              be executed using the call() method because it relies on the caller's "this" context
 *              to properly function.
 * @public
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function buildEnvHubAliasList() {
    // Make sure the calling scope has a valid context variable.
    validateSharedData.call(this);
    // Build and return a Listr Task.
    return {
        title: 'Building EnvHub Alias List...',
        enabled: () => Array.isArray(this.sharedData.envHubAliasChoices),
        task: (listrContext, thisTask) => {
            // DEBUG
            sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}buildEnvHubAliasList:listrContext.envHubOrgInfos:`, listrContext.envHubOrgInfos, `listrContext.envHubOrgInfos: `);
            // Build a list of Choices based on the Env Hub org infos.
            this.sharedData.envHubAliasChoices = yoHelper.buildOrgAliasChoices(listrContext.envHubOrgInfos);
            // Add a separator and a "not specified" option
            this.sharedData.envHubAliasChoices.push(new yoHelper.YeomanSeparator());
            this.sharedData.envHubAliasChoices.push({ name: 'My Environment Hub Is Not Listed', value: 'NOT_SPECIFIED', short: 'Not Specified' });
            thisTask.title += 'Done!';
            return;
        }
    };
}
exports.buildEnvHubAliasList = buildEnvHubAliasList;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    buildPkgOrgAliasList
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that takes a list of identified Packaging
 *              Orgs and uses it to create an Inquirer-compatible "choice list". This function must
 *              be executed using the call() method because it relies on the caller's "this" context
 *              to properly function.
 * @public
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function buildPkgOrgAliasList() {
    // Make sure the calling scope has a valid context variable.
    validateSharedData.call(this);
    // Build and return a Listr Task.
    return {
        title: 'Building PkgOrg Alias List...',
        enabled: () => Array.isArray(this.sharedData.pkgOrgAliasChoices),
        task: (listrContext, thisTask) => {
            // DEBUG
            sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}buildPkgOrgAliasList:listrContext.pkgOrgInfos:`, listrContext.pkgOrgInfos, `listrContext.pkgOrgInfos: `);
            // Build Choices based on ALL Packaging Org infos, followed by a separator and a "not specified" option.
            this.sharedData.pkgOrgAliasChoices = yoHelper.buildOrgAliasChoices(listrContext.pkgOrgInfos);
            this.sharedData.pkgOrgAliasChoices.push(new yoHelper.YeomanSeparator());
            this.sharedData.pkgOrgAliasChoices.push({ name: 'My Packaging Org Is Not Listed', value: 'NOT_SPECIFIED', short: 'Not Specified' });
            // Build Choices based on MANAGED Packaging Org infos, followed by a separator and a "not specified" option.
            const managedPkgOrgInfos = new Array();
            for (const orgInfo of listrContext.pkgOrgInfos) {
                if (orgInfo.nsPrefix) {
                    managedPkgOrgInfos.push(orgInfo);
                }
            }
            this.sharedData.managedPkgOrgAliasChoices = yoHelper.buildOrgAliasChoices(managedPkgOrgInfos);
            this.sharedData.managedPkgOrgAliasChoices.push(new yoHelper.YeomanSeparator());
            this.sharedData.managedPkgOrgAliasChoices.push({ name: 'My Packaging Org Is Not Listed', value: 'NOT_SPECIFIED', short: 'Not Specified' });
            // Build Choices based on UNMANAGED Packaging Org infos, followed by a separator and a "not specified" option.
            const unmanagedPkgOrgInfos = new Array();
            for (const orgInfo of listrContext.pkgOrgInfos) {
                if (!orgInfo.nsPrefix) {
                    unmanagedPkgOrgInfos.push(orgInfo);
                }
            }
            this.sharedData.unmanagedPkgOrgAliasChoices = yoHelper.buildOrgAliasChoices(unmanagedPkgOrgInfos);
            this.sharedData.unmanagedPkgOrgAliasChoices.push(new yoHelper.YeomanSeparator());
            this.sharedData.unmanagedPkgOrgAliasChoices.push({ name: 'My Packaging Org Is Not Listed', value: 'NOT_SPECIFIED', short: 'Not Specified' });
            // All done!
            thisTask.title += 'Done!';
            return;
        }
    };
}
exports.buildPkgOrgAliasList = buildPkgOrgAliasList;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    cloneGitRemote
 * @param       {string}  gitRemoteUri Required. ???
 * @param       {string}  targetDirectory Required. ???
 * @param       {string}  [gitCloneDirectory='']  Required. ???
 * @returns     {ListrObject}  A "runnable" Listr Object
 * @description Returns a "runnable" Listr Object that attempts to clone the Git Repository referred
 *              to by the provided Git Remote URI.
 * @public
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function cloneGitRemote(gitRemoteUri, targetDirectory, gitCloneDirectory = '') {
    // Make sure the calling scope has access to Shared Data.
    validateSharedData.call(this);
    // Validate incoming arguments.
    validateGitCloneArguments.apply(null, arguments);
    // Build and return a Listr Task Object.
    return new listr(
    // TASK GROUP: Git Clone Tasks
    [{
            title: `Cloning ${gitRemoteUri}...`,
            enabled: () => (gitRemoteUri && targetDirectory),
            task: (listrContext, thisTask) => {
                return new rxjs_1.Observable(observer => {
                    // Initialize an OTR (Observable Task Result).
                    const otr = initObservableTaskResult(`${dbgNs}cloneGitRemote`, listrContext, thisTask, observer, this.sharedData, this.generatorResult, `Cloning repository to ${path.join(targetDirectory, gitCloneDirectory)}`);
                    // Define the Task Logic to be executed.
                    const asyncTask = async () => {
                        await async_1.waitASecond(5);
                        sfdx_falcon_debug_1.SfdxFalconDebug.str(`${dbgNs}cloneGitRemote:gitRemoteUri:`, gitRemoteUri, `gitRemoteUri: `);
                        sfdx_falcon_debug_1.SfdxFalconDebug.str(`${dbgNs}cloneGitRemote:targetDirectory:`, targetDirectory, `targetDirectory: `);
                        sfdx_falcon_debug_1.SfdxFalconDebug.str(`${dbgNs}cloneGitRemote:gitCloneDirectory:`, gitCloneDirectory, `gitCloneDirectory: `);
                        return gitHelper.gitClone(gitRemoteUri, targetDirectory, gitCloneDirectory);
                        //return;
                    };
                    // Execute the Task Logic.
                    asyncTask()
                        .then(async (shellExecResult) => {
                        await async_1.waitASecond(3);
                        thisTask.title += 'Done!';
                        listrContext.gitRemoteCloned = true;
                        finalizeObservableTaskResult(otr);
                    })
                        .catch(async (shellExecError) => {
                        await async_1.waitASecond(3);
                        thisTask.title += 'Failed';
                        listrContext.gitRemoteCloned = false;
                        finalizeObservableTaskResult(otr, new sfdx_falcon_error_1.SfdxFalconError(`Could not clone repository: ${shellExecError.message}`, `GitCloneFailure`, `${dbgNs}cloneGitRemote`, sfdx_falcon_error_1.SfdxFalconError.wrap(shellExecError)));
                    });
                });
            }
        }], 
    // TASK GROUP OPTIONS: Git Clone Tasks
    {
        concurrent: false,
        collapse: false,
        renderer: falconUpdateRenderer
    });
}
exports.cloneGitRemote = cloneGitRemote;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    commitProjectFiles
 * @param       {string}  targetDir Required.
 * @param       {string}  commitMessage Required.
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that commits whatever is in the Target
 *              Directory, using the commitMessage parameter for the Commit Message.
 * @public
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function commitProjectFiles(targetDir, commitMessage) {
    // Make sure the calling scope has access to Shared Data.
    validateSharedData.call(this);
    // Build and return a Listr Task.
    return {
        title: `Committing Files...`,
        enabled: () => (typeof targetDir === 'string' && targetDir !== ''),
        skip: (listrContext) => {
            if (listrContext.gitInstalled !== true) {
                return true;
            }
        },
        task: (listrContext, thisTask) => {
            return new rxjs_1.Observable(observer => {
                // Initialize an OTR (Observable Task Result).
                const otr = initObservableTaskResult(`${dbgNs}commitProjectFiles`, listrContext, thisTask, observer, this.sharedData, this.generatorResult, `Committing files with this message: '${commitMessage}'`);
                // Define the Task Logic to be executed.
                const asyncTask = async () => {
                    const shellString = gitHelper.gitCommit(targetDir, commitMessage);
                    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}commitProjectFiles:shellString:`, shellString, `shellString: `);
                    return shellString;
                };
                // Execute the Task Logic.
                asyncTask()
                    .then(async (result) => {
                    await async_1.waitASecond(3);
                    thisTask.title += 'Done!';
                    listrContext.projectFilesCommitted = true;
                    finalizeObservableTaskResult(otr);
                })
                    .catch(async (error) => {
                    await async_1.waitASecond(3);
                    listrContext.projectFilesCommitted = false;
                    thisTask.skip('Nothing to Commit');
                    // NOTE: We are finalizing *without* passing the Error to force the observer to
                    // end with complete() instead of error().
                    finalizeObservableTaskResult(otr);
                });
            });
        }
    };
}
exports.commitProjectFiles = commitProjectFiles;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    convertMetadataSource
 * @param       {string}  mdapiSourceRootDir Required. Root directory that contains metadata that
 *              was retrieved using Metadata API.
 * @param       {string}  sfdxSourceOutputDir  Required. Directory to store the source files in
 *              after they’ve been converted to the source format. Must be an abosulte path.
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that attempts to convert source that was
 *              retrieved via the Metadata API (MDAPI Source) into SFDX Source.
 * @private
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function convertMetadataSource(mdapiSourceRootDir, sfdxSourceOutputDir) {
    // Make sure the calling scope has access to Shared Data.
    validateSharedData.call(this);
    // Build and return an OTR (Observable Listr Task).
    return {
        title: 'Converting MDAPI Source...',
        task: (listrContext, thisTask) => {
            return new rxjs_1.Observable(observer => {
                // Initialize an OTR (Observable Task Result).
                const otr = initObservableTaskResult(`${dbgNs}convertMetadataSource`, listrContext, thisTask, observer, this.sharedData, this.generatorResult, `Converting MDAPI source from ${mdapiSourceRootDir}`);
                // Execute the Task Logic
                sfdxHelper.mdapiConvert(mdapiSourceRootDir, sfdxSourceOutputDir)
                    .then((successResult) => {
                    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}convertMetadataSource:successResult:`, successResult, `successResult: `);
                    thisTask.title += 'Done!';
                    finalizeObservableTaskResult(otr);
                })
                    .catch((failureResult) => {
                    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}convertMetadataSource:failureResult:`, failureResult, `failureResult: `);
                    thisTask.title += 'Failed';
                    finalizeObservableTaskResult(otr, failureResult);
                });
            });
        }
    };
}
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    extractMdapiSource
 * @param       {string}  zipFile Required. Path to a zip file produced by an MDAPI retrieve
 *              operation, like "sfdx force:mdapi:retrieve".
 * @param       {string}  zipExtractTarget  Required. Path of the directory where the MDAPI source
 *              in the Zip File will be extracted to.
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that attempts to extract the MDAPI source
 *              from inside of a Zip File produced by an MDAPI retrieve operation.
 * @private
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function extractMdapiSource(zipFile, zipExtractTarget) {
    // Make sure the calling scope has access to Shared Data.
    validateSharedData.call(this);
    // Build and return an OTR (Observable Listr Task).
    return {
        title: 'Extracting MDAPI Source...',
        task: (listrContext, thisTask) => {
            return new rxjs_1.Observable(observer => {
                // Initialize an OTR (Observable Task Result).
                const otr = initObservableTaskResult(`${dbgNs}extractMdapiSource`, listrContext, thisTask, observer, this.sharedData, this.generatorResult, `Extracting source into ${zipExtractTarget}`);
                // Execute the Task Logic
                zipHelper.extract(zipFile, zipExtractTarget)
                    .then(() => {
                    listrContext.sourceExtracted = true;
                    thisTask.title += 'Done!';
                    finalizeObservableTaskResult(otr);
                })
                    .catch(extractionError => {
                    listrContext.sourceExtracted = false;
                    thisTask.title += 'Failed';
                    finalizeObservableTaskResult(otr, new sfdx_falcon_error_1.SfdxFalconError(`MDAPI source from ${zipFile} could not be extracted to ${zipExtractTarget}`, `SourceExtractionError`, `${dbgNs}extractZipFile`, extractionError));
                });
            });
        }
    };
}
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    fetchAndConvertManagedPackage
 * @param       {string}  aliasOrUsername Required. The alias or username associated with a
 *              packaging org that the Salesforce CLI is currently connected to.
 * @param       {string}  packageName Required. The name of the desired managed package.
 * @param       {string}  projectDir  Required. The root of the Project Directory
 * @param       {string}  packageDir  Required. Name of the default package directory, located
 *              inside of "projectDir/sfdx-source/"
 * @returns     {ListrObject}  A "runnable" Listr Object
 * @description Returns a "runnable" Listr Object that attempts to retrieve, extract, and convert
 *              the metadata for the specified package from the specified org. The converted
 *              metadata source will be saved to the Package Directory specified by the caller.
 * @public
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function fetchAndConvertManagedPackage(aliasOrUsername, packageName, projectDir, packageDir) {
    // Validate incoming arguments.
    validatePkgConversionArguments.apply(null, arguments);
    // Determine various directory locations.
    const retrieveTargetDir = path.join(projectDir, 'temp');
    const zipFile = path.join(projectDir, 'temp', 'unpackaged.zip');
    const zipExtractTarget = path.join(projectDir, 'mdapi-source', 'original');
    const mdapiSourceRootDir = path.join(projectDir, 'mdapi-source', 'original');
    const sfdxSourceOutputDir = path.join(projectDir, 'sfdx-source', packageDir);
    // Build and return a Listr Task Object.
    return new listr(
    // TASK GROUP: SFDX Config Tasks
    [
        packagedMetadataFetch.call(this, aliasOrUsername, [packageName], retrieveTargetDir),
        extractMdapiSource.call(this, zipFile, zipExtractTarget),
        convertMetadataSource.call(this, mdapiSourceRootDir, sfdxSourceOutputDir)
    ], 
    // TASK GROUP OPTIONS: SFDX Config Tasks
    {
        concurrent: false,
        collapse: false,
        renderer: falconUpdateRenderer
    });
}
exports.fetchAndConvertManagedPackage = fetchAndConvertManagedPackage;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    finalizeGit
 * @param       {string}  targetDirectory Required. Directory that will be initialized with Git.
 * @param       {string}  [gitRemoteUri]  Optional. URI of the remote that should be associated
 *              with the repository that we're going to initialize.
 * @returns     {ListrObject}  A "runnable" Listr Object
 * @description Returns a "runnable" Listr Object that initializes Git in the Target Directory, then
 *              connects that repo to the remote specified by the Git Remote URI.
 * @public
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function finalizeGit(targetDir, gitRemoteUri = '', gitCommitMsg = 'Initial Commit') {
    // Debug incoming arguments
    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}finalizeGit:arguments:`, arguments, `arguments: `);
    // Make sure the calling scope has a valid context variable.
    validateSharedData.call(this);
    // Make sure the caller provided a Target Directory.
    if (typeof targetDir !== 'string' || targetDir === '') {
        throw new sfdx_falcon_error_1.SfdxFalconError(`Expected targetDir to be a non-empty string but got type '${typeof targetDir}' instead.`, `TypeError`, `${dbgNs}finalizeGit`);
    }
    // Build and return a Listr Object.
    return new listr([
        // TASKS: Git Finalization Tasks
        gitRuntimeCheck.call(this),
        initializeGit.call(this, targetDir),
        stageProjectFiles.call(this, targetDir),
        commitProjectFiles.call(this, targetDir, gitCommitMsg),
        reValidateGitRemote.call(this, gitRemoteUri),
        addGitRemote.call(this, targetDir, gitRemoteUri)
    ], {
        // TASK OPTIONS: Git Finalization Tasks
        concurrent: false,
        collapse: false,
        exitOnError: false,
        renderer: falconUpdateRenderer
    });
}
exports.finalizeGit = finalizeGit;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    finalizeObservableTaskResult
 * @param       {SfdxFalconResult}  otr Required. An "Observable Task Result".
 * @param       {ErrorOrResult} [errorOrResult] Optional. Might be an Error or an SfdxFalconResult.
 * @returns     {void}
 * @description Given a LISTR Result, attempts to "finalize" it by performing a number of tasks.
 *              These include finishing any progress notifications, marking the result as succeeded
 *              (or errror if an SfdxFalconError is provided), attaching the result to its parent
 *              result, and then completing (or error-ing) the observer.
 * @private
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function finalizeObservableTaskResult(otr, errorOrResult) {
    // Define a special "observer throw".
    const observerThrow = (error) => {
        try {
            otr.detail['observer'].error(sfdx_falcon_error_1.SfdxFalconError.wrap(error));
        }
        catch (noObserverError) {
            sfdx_falcon_debug_1.SfdxFalconDebug.debugObject('WARNING! OBSERVER MISSING:', error);
            throw sfdx_falcon_error_1.SfdxFalconError.wrap(error);
        }
    };
    // Validate incoming arguments.
    if (typeof otr === 'undefined') {
        return observerThrow(new sfdx_falcon_error_1.SfdxFalconError(`Expected otr to be an SfdxFalconResult but got 'undefined' instead.`, `TypeError`, `${dbgNs}finalizeObservableTaskResult`));
    }
    if ((otr instanceof sfdx_falcon_result_1.SfdxFalconResult) !== true) {
        return observerThrow(new sfdx_falcon_error_1.SfdxFalconError(`Expected otr to be an SfdxFalconResult but got '${(otr.constructor) ? otr.constructor.name : 'unknown'}' instead.`, `TypeError`, `${dbgNs}finalizeObservableTaskResult`));
    }
    if (typeof otr.detail !== 'object') {
        return observerThrow(new sfdx_falcon_error_1.SfdxFalconError(`Expected otr.detail to be an object but got '${typeof otr.detail}' instead.`, `TypeError`, `${dbgNs}finalizeObservableTaskResult`));
    }
    if (typeof errorOrResult !== 'undefined' && ((errorOrResult instanceof Error) !== true) && ((errorOrResult instanceof sfdx_falcon_result_1.SfdxFalconResult) !== true)) {
        return observerThrow(new sfdx_falcon_error_1.SfdxFalconError(`Expected errorOrResult to be an instance of Error or SfdxFalconResult if provided but got '${(errorOrResult.constructor) ? errorOrResult.constructor.name : 'unknown'}' instead.`, `TypeError`, `${dbgNs}finalizeObservableTaskResult`));
    }
    // Extract key objects from the OTR Detail.
    const otrObserver = otr.detail['observer'] || {};
    const parentResult = otr.detail['parentResult'] || {};
    const progressNotification = otr.detail['progressNotification'];
    // Finish any Progress Notifications attached to this OTR (Observable Task Result).
    sfdx_falcon_notifications_1.FalconProgressNotifications.finish(progressNotification);
    // Set the final state of the OTR (success or error).
    if (typeof errorOrResult === 'undefined') {
        // Succeeded
        otr.success();
    }
    else {
        // Failed
        if (errorOrResult instanceof Error) {
            otr.error(errorOrResult);
        }
        if (errorOrResult instanceof sfdx_falcon_result_1.SfdxFalconResult) {
            try {
                otr.addChild(errorOrResult);
            }
            catch (_a) {
                // No need to do anything here. We are just suppressing any
                // bubbled errors from the previous addChild() call.
            }
        }
    }
    // Add the OTR as a child of it's attached Parent Result (if present).
    if (parentResult instanceof sfdx_falcon_result_1.SfdxFalconResult) {
        try {
            parentResult.addChild(otr);
        }
        catch (bubbledError) {
            // If we get here, it means the parent was set to Bubble Errors.
            // That means that bubbledError should be an SfdxFalconResult
            return observerThrow(sfdx_falcon_error_1.SfdxFalconError.wrap(bubbledError.errObj));
        }
    }
    // Tell the Observable to "complete" or "error", depending on whether we got an Error object or not.
    if (typeof errorOrResult === 'undefined') {
        if (typeof otrObserver.complete === 'function') {
            return otrObserver.complete();
        }
    }
    else {
        // Is it an Error object?
        if (errorOrResult instanceof Error) {
            return observerThrow(sfdx_falcon_error_1.SfdxFalconError.wrap(errorOrResult));
        }
        // Is it an SfdxFalconResult Object?
        if (errorOrResult instanceof sfdx_falcon_result_1.SfdxFalconResult) {
            if (lodash_1.isEmpty(errorOrResult.errObj) !== true) {
                return observerThrow(sfdx_falcon_error_1.SfdxFalconError.wrap(errorOrResult.errObj));
            }
            else {
                return observerThrow(new sfdx_falcon_error_1.SfdxFalconError(`finalizeObservableTaskResult() received an Error Result that did not contain an Error Object.`, `MissingErrObj`, `${dbgNs}finalizeObservableTaskResult`, sfdx_falcon_error_1.SfdxFalconError.wrap(errorOrResult)));
            }
        }
        // It's not an Error OR an SfdxFalconResult. It's something invalid.
        return observerThrow(new sfdx_falcon_error_1.SfdxFalconError(`finalizeObservableTaskResult() received an invalid value for its errorOrResult parameter.`, `InvalidErrorOrResult`, `${dbgNs}finalizeObservableTaskResult`, sfdx_falcon_error_1.SfdxFalconError.wrap(errorOrResult)));
    }
}
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    gitInitTasks
 * @returns     {ListrObject}  A "runnable" Listr Object
 * @description Returns a Listr-compatible Task Object that verifies the presence of the Git
 *              executable in the local environment.
 * @public
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function gitInitTasks() {
    // Make sure the calling scope has a valid context variable.
    validateSharedData.call(this);
    // Grab the Command Name and Git Remote URI out of Shared Data.
    const cliCommandName = this.sharedData.cliCommandName;
    const gitRemoteUri = this.sharedData.gitRemoteUri;
    // Check for the presence of key variables in the calling scope.
    if (typeof cliCommandName !== 'string' || cliCommandName === '') {
        throw new sfdx_falcon_error_1.SfdxFalconError(`Expected cliCommandName to be a non-empty string but got type '${typeof cliCommandName}' instead.`, `TypeError`, `${dbgNs}gitInitTasks`);
    }
    // Build and return a Listr Object.
    return new listr([
        {
            // PARENT_TASK: "Initialize" the Falcon command.
            title: `Inspecting Environment`,
            task: listrContext => {
                return new listr([
                    // SUBTASKS: Check for Git executable and for valid Git Remote URI.
                    gitRuntimeCheck.call(this),
                    validateGitRemote(gitRemoteUri)
                ], {
                    // SUBTASK OPTIONS: (Git Init Tasks)
                    concurrent: false,
                    renderer: falconUpdateRenderer
                });
            }
        }
    ], {
        // PARENT_TASK OPTIONS: (Git Validation/Initialization)
        concurrent: false,
        collapse: false,
        renderer: falconUpdateRenderer
    });
}
exports.gitInitTasks = gitInitTasks;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    gitRuntimeCheck
 * @param       {string}  dbgNs Required. Debug namespace. Ensures proper debug output.
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that verifies the presence of the Git
 *              executable in the local environment.
 * @public
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function gitRuntimeCheck() {
    return {
        title: 'Looking for Git...',
        task: (listrContext, thisTask) => {
            if (gitHelper.isGitInstalled() === true) {
                thisTask.title += 'Found!';
                listrContext.gitInstalled = true;
            }
            else {
                listrContext.gitInstalled = false;
                thisTask.title += 'Not Found!';
                throw new sfdx_falcon_error_1.SfdxFalconError('Git must be installed in your local environment.', 'GitNotFound', `${dbgNs}gitRuntimeCheck`);
            }
        }
    };
}
exports.gitRuntimeCheck = gitRuntimeCheck;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    identifyDevHubs
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that takes a list of raw SFDX Org Infos
 *              and searches them to identify any Dev Hubs.  This function must be invoked using
 *              the call() method because it relies on the caller's "this" context to function.
 * @public
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function identifyDevHubs() {
    // Make sure the calling scope has access to Shared Data.
    validateSharedData.call(this);
    // Build and return a Listr Task.
    return {
        title: 'Identifying DevHub Orgs...',
        enabled: () => Array.isArray(this.sharedData.devHubAliasChoices),
        task: (listrContext, thisTask) => {
            // DEBUG
            sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}identifyDevHubs:`, listrContext.rawSfdxOrgList, `listrContext.rawSfdxOrgList: `);
            // Search the SFDX Org Infos list for any DevHub orgs.
            const devHubOrgInfos = sfdxHelper.identifyDevHubOrgs(this.sharedData.sfdxOrgInfoMap);
            // DEBUG
            sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}identifyDevHubs:`, devHubOrgInfos, `devHubOrgInfos: `);
            // Make sure there is at least one active Dev Hub.
            if (devHubOrgInfos.length < 1) {
                thisTask.title += 'No Dev Hubs Found';
                throw new sfdx_falcon_error_1.SfdxFalconError(`No Dev Hubs found. You must have at least one active Dev Hub to continue. `
                    + `Please run force:auth:web:login to connect to your Dev Hub.`, `NoDevHubs`, `${dbgNs}identifyDevHubs`);
            }
            // Give the Listr Context variable access to this.devHubOrgInfos
            listrContext.devHubOrgInfos = devHubOrgInfos;
            // Update the Task Title
            thisTask.title += 'Done!';
        }
    };
}
exports.identifyDevHubs = identifyDevHubs;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    identifyEnvHubs
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that takes a list of raw SFDX Org Infos
 *              and searches them to identify any Environment Hubs.  This function must be invoked
 *              using the call() method because it relies on the caller's "this" context to function.
 * @public
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function identifyEnvHubs() {
    // Make sure the calling scope has access to Shared Data.
    validateSharedData.call(this);
    // Build and return a Listr Task.
    return {
        title: 'Identifying EnvHub Orgs...',
        enabled: () => Array.isArray(this.sharedData.envHubAliasChoices),
        task: (listrContext, thisTask) => {
            // DEBUG
            sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}identifyEnvHubs:listrContext.rawSfdxOrgList:`, listrContext.rawSfdxOrgList, `listrContext.rawSfdxOrgList: `);
            // Search the SFDX Org Infos list for any Environment Hub orgs.
            return sfdxHelper.identifyEnvHubOrgs(this.sharedData.sfdxOrgInfoMap)
                .then(envHubOrgInfos => {
                // DEBUG
                sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}identifyEnvHubs:envHubOrgInfos:`, envHubOrgInfos, `envHubOrgInfos: `);
                // Give the Listr Context variable access to the Packaging Org Infos just returned.
                listrContext.envHubOrgInfos = envHubOrgInfos;
                // Update the task title based on the number of EnvHub Org Infos
                if (envHubOrgInfos.length < 1) {
                    thisTask.title += 'No Environment Hubs Found';
                }
                else {
                    thisTask.title += 'Done!';
                }
            })
                .catch(error => {
                // We normally should NOT get here.
                sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}identifyEnvHubs:error:`, error, `error: `);
                thisTask.title += 'Unexpected error while identifying Environment Hub Orgs';
                throw error;
            });
        }
    };
}
exports.identifyEnvHubs = identifyEnvHubs;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    identifyPkgOrgs
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that takes a list of raw SFDX Org Infos
 *              and searches them to identify any Packaging Orgs.  This function must be invoked
 *              using the call() method because it relies on the caller's "this" context to function.
 * @public
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function identifyPkgOrgs() {
    // Make sure the calling scope has access to Shared Data.
    validateSharedData.call(this);
    // Build and return a Listr Task.
    return {
        title: 'Identifying Packaging Orgs...',
        enabled: () => Array.isArray(this.sharedData.pkgOrgAliasChoices),
        task: (listrContext, thisTask) => {
            // DEBUG
            sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}identifyPkgOrgs:listrContext.rawSfdxOrgList:`, listrContext.rawSfdxOrgList, `listrContext.rawSfdxOrgList: (BEFORE ASYNC CALL)`);
            // Search the SFDX Org Infos list for any Packaging orgs.
            return sfdxHelper.identifyPkgOrgs(this.sharedData.sfdxOrgInfoMap)
                .then(pkgOrgInfos => {
                // DEBUG
                sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}identifyPkgOrgs:pkgOrgInfos:`, pkgOrgInfos, `pkgOrgInfos: `);
                // Give the Listr Context variable access to the Packaging Org Infos just returned.
                listrContext.pkgOrgInfos = pkgOrgInfos;
                // Update the task title based on the number of EnvHub Org Infos
                if (pkgOrgInfos.length < 1) {
                    thisTask.title += 'No Packaging Orgs Found';
                }
                else {
                    thisTask.title += 'Done!';
                }
            })
                .catch(error => {
                // We normally should NOT get here.
                sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}identifyPkgOrgs:error:`, error, `error: `);
                thisTask.title += 'Unexpected error while identifying Packaging Orgs';
                throw error;
            });
        }
    };
}
exports.identifyPkgOrgs = identifyPkgOrgs;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    initializeGit
 * @param       {string}  targetDir
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that initializes Git in the target directory.
 * @public
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function initializeGit(targetDir) {
    // Make sure the calling scope has access to Shared Data.
    validateSharedData.call(this);
    // Build and return a Listr Task.
    return {
        title: `Initializing Git in Target Directory...`,
        enabled: () => (typeof targetDir === 'string' && targetDir !== ''),
        skip: (listrContext) => {
            if (listrContext.gitInstalled !== true) {
                return true;
            }
        },
        task: (listrContext, thisTask) => {
            return new rxjs_1.Observable(observer => {
                // Initialize an OTR (Observable Task Result).
                const otr = initObservableTaskResult(`${dbgNs}:initializeGit`, listrContext, thisTask, observer, this.sharedData, this.generatorResult, `Running git init in ${targetDir}`);
                // Define the Task Logic to be executed.
                const asyncTask = async () => {
                    const shellString = gitHelper.gitInit(targetDir);
                    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}initializeGit:shellString:`, shellString, `shellString: `);
                    return shellString;
                };
                // Execute the Task Logic.
                asyncTask()
                    .then(async (result) => {
                    await async_1.waitASecond(3);
                    thisTask.title += 'Done!';
                    listrContext.gitInitialized = true;
                    finalizeObservableTaskResult(otr);
                })
                    .catch(async (error) => {
                    await async_1.waitASecond(3);
                    thisTask.title += 'Failed';
                    listrContext.gitInitialized = false;
                    finalizeObservableTaskResult(otr, error);
                });
            });
        }
    };
}
exports.initializeGit = initializeGit;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    initObservableTaskResult
 * @param       {string}  extDbgNs  Required.
 * @param       {object}  listrContext Required.
 * @param       {object}  listrTask Required.
 * @param       {object}  [observer]  Optional.
 * @param       {object}  [sharedData]  Optional.
 * @param       {string}  [message] Optional.
 * @param       {SfdxFalconResult}  [parentResult]  Optional.
 * @returns     {ListrExecutionOptions}
 * @description Returns a ListrExecutionOptions structure, populated with the values provided by the
 *              caller. Before returning, this function will perform DEBUG for the namespace that
 *              the caller provides.
 * @private
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function initObservableTaskResult(extDbgNs, listrContext, listrTask, observer, sharedData, parentResult, message) {
    // Create a Listr "Execution Options" data structure.
    const listrExecOpts = {
        listrContext: listrContext,
        listrTask: listrTask,
        observer: observer,
        sharedData: sharedData
    };
    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${extDbgNs}`, listrExecOpts, `listrExecOpts: `);
    // Initialize an SFDX-Falcon Result object.
    const observableTaskResult = new sfdx_falcon_result_1.SfdxFalconResult(extDbgNs, "LISTR" /* LISTR */, { startNow: true,
        bubbleError: false,
        bubbleFailure: false }); // Let the parent Result handle failures (no bubbling)
    // Set the initial Task Detail message.
    if (typeof observer === 'object' && typeof observer.next === 'function' && typeof message === 'string') {
        observer.next(`[0s] ${message}`);
    }
    // Set up Progress Notifications.
    const progressNotification = sfdx_falcon_notifications_1.FalconProgressNotifications.start(message, 1000, observableTaskResult, observer);
    // Initialize the Results Detail object for this LISTR observable task.
    const observableTaskResultDetail = {
        progressNotification: progressNotification,
        parentResult: parentResult,
        listrExecOpts: listrExecOpts,
        listrContext: listrContext,
        listrTask: listrTask,
        observer: observer,
        sharedData: sharedData,
        message: message
    };
    // Attach the Results Detail object to the LISTR result.
    observableTaskResult.setDetail(observableTaskResultDetail);
    // Return the Observable Task Result to the caller.
    return observableTaskResult;
}
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    packagedMetadataFetch
 * @param       {string}  aliasOrUsername Required. The alias or username associated with a current
 *              Salesforce CLI connected org.
 * @param       {string[]}  packageNames  Required. String array containing the names of all
 *              packages that should be retrieved.
 * @param       {string}  retrieveTargetDir Required. The root of the directory structure where
 *              the retrieved .zip or metadata files are put.
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that attempts to retrieve the metadata
 *              for the specified package from the specified org. The metadata will be saved to
 *              the local filesystem at the location specified by the caller.
 * @public
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function packagedMetadataFetch(aliasOrUsername, packageNames, retrieveTargetDir) {
    // Validate incoming arguments.
    validatePkgConversionArguments.apply(null, arguments);
    // Make sure the calling scope has access to Shared Data.
    validateSharedData.call(this);
    // Build and return an OTR (Observable Listr Task).
    return {
        title: 'Fetching Metadata Packages...',
        task: (listrContext, thisTask) => {
            return new rxjs_1.Observable(observer => {
                // Initialize an OTR (Observable Task Result).
                const otr = initObservableTaskResult(`${dbgNs}packagedMetadataFetch`, listrContext, thisTask, observer, this.sharedData, this.generatorResult, `Retrieving '${packageNames}' from ${aliasOrUsername} (this might take a few minutes)`);
                // Execute the Task Logic
                sfdxHelper.fetchMetadataPackages(aliasOrUsername, packageNames, retrieveTargetDir)
                    .then((successResult) => {
                    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}packagedMetadataFetch:successResult:`, successResult, `successResult: `);
                    // Save the UTILITY result to Shared Data and update the task title.
                    this.sharedData.pkgMetadataFetchResult = successResult;
                    thisTask.title += 'Done!';
                    finalizeObservableTaskResult(otr);
                })
                    .catch((failureResult) => {
                    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}packagedMetadataFetch:failureResult:`, failureResult, `failureResult: `);
                    // We get here if no connections were found.
                    thisTask.title += 'Failed';
                    finalizeObservableTaskResult(otr, failureResult);
                });
            });
        }
    };
}
exports.packagedMetadataFetch = packagedMetadataFetch;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    reValidateGitRemote
 * @param       {string}  gitRemoteUri
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that attempts to re-validate the presence of
 *              a Git Remote at the Git Remote URI provided.
 * @public
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function reValidateGitRemote(gitRemoteUri) {
    // Make sure the calling scope has access to Shared Data.
    validateSharedData.call(this);
    // Build and return a Listr Task.
    return {
        title: `Validating Access to the Git Remote...`,
        enabled: () => (typeof gitRemoteUri === 'string' && gitRemoteUri !== ''),
        skip: (listrContext) => {
            if (listrContext.gitInstalled !== true) {
                return true;
            }
        },
        task: (listrContext, thisTask) => {
            return new rxjs_1.Observable(observer => {
                // Initialize an OTR (Observable Task Result).
                const otr = initObservableTaskResult(`${dbgNs}reValidateGitRemote`, listrContext, thisTask, observer, this.sharedData, this.generatorResult, `Attempting to reach ${gitRemoteUri}`);
                // Execute the Task Logic
                gitHelper.checkGitRemoteStatus(gitRemoteUri, 3)
                    .then((successResult) => {
                    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}reValidateGitRemote:successResult:`, successResult, `successResult: `);
                    listrContext.gitRemoteIsValid = true;
                    thisTask.title += 'Done!';
                    finalizeObservableTaskResult(otr);
                })
                    .catch((errorResult) => {
                    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}reValidateGitRemote:errorResult:`, errorResult, `errorResult: `);
                    // Error code 2 (Git remote reachable but empty) is the ideal state.
                    // Consider that a success result.
                    if (errorResult.code === 2) {
                        listrContext.gitRemoteIsValid = true;
                        thisTask.title += 'Done!';
                        finalizeObservableTaskResult(otr);
                        return;
                    }
                    // Any non-zero error code other than 2 is a failure.
                    listrContext.gitRemoteIsValid = false;
                    thisTask.title += errorResult.message;
                    finalizeObservableTaskResult(otr, new sfdx_falcon_error_1.SfdxFalconError(`Git Remote is invalid. ${errorResult.message}. `, `GitRemoteError`, `${dbgNs}reValidateGitRemote`));
                });
            });
        }
    };
}
exports.reValidateGitRemote = reValidateGitRemote;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    scanConnectedOrgs
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that scans the orgs that are connected to
 *              (ie. authenticated to) the local SFDX environment. The raw list of these orgs is
 *              added to the Listr Context var so it's available to subsequent Listr tasks.
 * @public
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function scanConnectedOrgs() {
    // Make sure the calling scope has access to Shared Data.
    validateSharedData.call(this);
    // Build and return a Listr Task.
    return {
        title: 'Scanning Connected Orgs...',
        task: (listrContext, thisTask) => {
            return sfdxHelper.scanConnectedOrgs()
                .then(successResult => {
                // DEBUG
                sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}scanConnectedOrgs:successResult:`, successResult, `successResult: `);
                // Store the JSON result containing the list of orgs that are NOT scratch orgs in a class member.
                let rawSfdxOrgList;
                if (successResult.detail && typeof successResult.detail === 'object') {
                    if (successResult.detail.stdOutParsed) {
                        rawSfdxOrgList = successResult.detail.stdOutParsed['result']['nonScratchOrgs'];
                    }
                }
                // Make sure that there is at least ONE connnected org
                if (Array.isArray(rawSfdxOrgList) === false || rawSfdxOrgList.length < 1) {
                    throw new sfdx_falcon_error_1.SfdxFalconError(`No orgs have been authenticated to the Salesforce CLI. `
                        + `Please run force:auth:web:login to connect to an org.`, `NoConnectedOrgs`, `${dbgNs}scanConnectedOrgs`);
                }
                // Put the raw SFDX Org List into the Listr Context variable.
                this.sharedData.rawSfdxOrgList = rawSfdxOrgList;
                // Build a baseline list of SFDX Org Info objects based on thie raw list.
                this.sharedData.sfdxOrgInfoMap = sfdxHelper.buildSfdxOrgInfoMap(rawSfdxOrgList);
                // Change the title of the task.
                thisTask.title += 'Done!';
            })
                .catch(failureResult => {
                // We get here if no connections were found.
                sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}scanConnectedOrgs:failureResult:`, failureResult, `failureResult: `);
                thisTask.title += 'No Connections Found';
                throw failureResult;
            });
        }
    };
}
exports.scanConnectedOrgs = scanConnectedOrgs;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    sfdxInitTasks
 * @returns     {ListrObject}  A "runnable" Listr Object
 * @description Returns a Listr-compatible Task Object that contains a number of sub-tasks which
 *              inspect the connected orgs in the local SFDX environment and build Inquirer "choice
 *              lists" with them.
 * @public
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function sfdxInitTasks() {
    // Build and return a Listr Object.
    return new listr([
        {
            // PARENT_TASK: Local SFDX Configuration
            title: 'Inspecting Local SFDX Configuration',
            task: listrContext => {
                return new listr([
                    scanConnectedOrgs.call(this),
                    identifyDevHubs.call(this),
                    identifyEnvHubs.call(this),
                    identifyPkgOrgs.call(this),
                    buildDevHubAliasList.call(this),
                    buildEnvHubAliasList.call(this),
                    buildPkgOrgAliasList.call(this)
                ], 
                // SUBTASK OPTIONS: (SFDX Config Tasks)
                {
                    concurrent: false,
                    collapse: false,
                    renderer: falconUpdateRenderer
                });
            }
        }
    ], {
        // PARENT_TASK OPTIONS: (Git Validation/Initialization)
        concurrent: false,
        collapse: false,
        renderer: falconUpdateRenderer
    });
}
exports.sfdxInitTasks = sfdxInitTasks;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    stageProjectFiles
 * @param       {string}  targetDir
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that stages (git -A) ALL files in the target
 *              directory.
 * @public
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function stageProjectFiles(targetDir) {
    // Make sure the calling scope has access to Shared Data.
    validateSharedData.call(this);
    // Build and return a Listr Task.
    return {
        title: `Staging Files...`,
        enabled: () => (typeof targetDir === 'string' && targetDir !== ''),
        skip: (listrContext) => {
            if (listrContext.gitInstalled !== true) {
                return true;
            }
        },
        task: (listrContext, thisTask) => {
            return new rxjs_1.Observable(observer => {
                // Initialize an OTR (Observable Task Result).
                const otr = initObservableTaskResult(`${dbgNs}stageProjectFiles`, listrContext, thisTask, observer, this.sharedData, this.generatorResult, `Staging all new and modified files (git -A) in ${targetDir}`);
                // Define the Task Logic to be executed.
                const asyncTask = async () => {
                    const shellString = gitHelper.gitAdd(targetDir);
                    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}stageProjectFiles:shellString:`, shellString, `shellString: `);
                    return shellString;
                };
                // Execute the Task Logic.
                asyncTask()
                    .then(async (result) => {
                    await async_1.waitASecond(3);
                    thisTask.title += 'Done!';
                    listrContext.projectFilesStaged = true;
                    finalizeObservableTaskResult(otr);
                })
                    .catch(async (error) => {
                    await async_1.waitASecond(3);
                    thisTask.title += 'Failed';
                    listrContext.projectFilesStaged = false;
                    finalizeObservableTaskResult(otr, error);
                });
            });
        }
    };
}
exports.stageProjectFiles = stageProjectFiles;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    validateGitCloneArguments
 * @returns     {void}
 * @description Ensures that the arguments provided match an expected, ordered set.
 * @private
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function validateGitCloneArguments() {
    // Validate "gitRemoteUri".
    if (typeof arguments[0] !== 'string' || arguments[0] === '') {
        throw new sfdx_falcon_error_1.SfdxFalconError(`Expected gitRemoteUri to be a non-empty string but got type '${typeof arguments[0]}' instead.`, `TypeError`, `${dbgNs}validateGitCloneArguments`);
    }
    // Validate "targetDirectory".
    if (typeof arguments[1] !== 'string' || arguments[1] === '') {
        throw new sfdx_falcon_error_1.SfdxFalconError(`Expected targetDirectory to be a non-empty string but got type '${typeof arguments[1]}' instead.`, `TypeError`, `${dbgNs}validateGitCloneArguments`);
    }
    // Validate "gitCloneDirectory".
    if (typeof arguments[2] !== 'string') {
        throw new sfdx_falcon_error_1.SfdxFalconError(`Expected gitCloneDirectory to be a string but got type '${typeof arguments[2]}' instead.`, `TypeError`, `${dbgNs}validateGitCloneArguments`);
    }
}
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    validateGitRemote
 * @param       {string}  gitRemoteUri  Required. URI of the remote Git repository being validated.
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that validates the presence of and access to
 *              the Git remote at the provided Git Remote URI.
 * @public
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function validateGitRemote(gitRemoteUri = '') {
    // Validate incoming arguments.
    if (typeof gitRemoteUri !== 'string') {
        throw new sfdx_falcon_error_1.SfdxFalconError(`Expected gitRemoteUri to be string but got type '${typeof gitRemoteUri}' instead.`, `TypeError`, `${dbgNs}validateGitRemote`);
    }
    // Build and return the Listr task.
    return {
        title: 'Validating Git Remote...',
        enabled: listrContext => (gitRemoteUri && listrContext.gitInstalled === true),
        task: (listrContext, thisTask) => {
            return gitHelper.checkGitRemoteStatus(gitRemoteUri, 3)
                .then(result => {
                thisTask.title += 'Valid!';
                listrContext.wizardInitialized = true;
            })
                .catch(result => {
                thisTask.title += 'ERROR';
                if (result instanceof Error) {
                    throw new sfdx_falcon_error_1.SfdxFalconError('There was a problem with your Git Remote.', 'InvalidGitRemote', `${dbgNs}validateGitRemote`, result);
                }
                else {
                    throw new sfdx_falcon_error_1.SfdxFalconError(`There was a problem with your Git Remote: ${result.message}.`, 'InvalidGitRemote', `${dbgNs}validateGitRemote`);
                }
            });
        }
    };
}
exports.validateGitRemote = validateGitRemote;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    validateSharedData
 * @returns     {void}
 * @description Ensures that the calling scope has the special "sharedData" object.
 * @private
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function validateSharedData() {
    if (typeof this.sharedData !== 'object') {
        throw new sfdx_falcon_error_1.SfdxFalconError(`Expected this.sharedData to be an object available in the calling scope. Got type '${typeof this.sharedData}' instead. `
            + `You must execute listr-tasks functions using the syntax: functionName.call(this). `
            + `You must also ensure that the calling scope has defined an object named 'sharedData'.`, `InvalidSharedData`, `${dbgNs}validateSharedData`);
    }
}
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    validatePkgConversionArguments
 * @returns     {void}
 * @description Ensures that the arguments provided match an expected, ordered set.
 * @private
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function validatePkgConversionArguments() {
    // Validate "aliasOrUsername".
    if (typeof arguments[0] !== 'string' || arguments[0] === '') {
        throw new sfdx_falcon_error_1.SfdxFalconError(`Expected aliasOrUsername to be a non-empty string but got type '${typeof arguments[0]}' instead.`, `TypeError`, `${dbgNs}validatePkgConversionArguments`);
    }
    // Validate "packageNames" array.
    if (Array.isArray(arguments[1]) !== true || arguments[1].length < 1) {
        throw new sfdx_falcon_error_1.SfdxFalconError(`Expected packageNames to be a non-empty array but got type '${typeof arguments[1]}' instead.`, `TypeError`, `${dbgNs}validatePkgConversionArguments`);
    }
    // Validate "retrieveTargetDir".
    if (typeof arguments[2] !== 'string' || arguments[2] === '') {
        throw new sfdx_falcon_error_1.SfdxFalconError(`Expected retrieveTargetDir to be a non-empty string but got type '${typeof arguments[2]}' instead.`, `TypeError`, `${dbgNs}validatePkgConversionArguments`);
    }
}
//# sourceMappingURL=listr-tasks.js.map