import { Connection } from '@salesforce/core';
import { AnyJson } from '@salesforce/ts-types';
import { SfdxFalconResult } from '../sfdx-falcon-result';
import { AliasOrConnection } from '../sfdx-falcon-types';
import { MetadataPackage } from '../sfdx-falcon-types';
import { MetadataPackageVersion } from '../sfdx-falcon-types';
import { RawSfdxOrgInfo } from '../sfdx-falcon-types';
import { ResolvedConnection } from '../sfdx-falcon-types';
import { SfdxOrgInfoMap } from '../sfdx-falcon-types';
import { SfdxOrgInfoSetup } from '../sfdx-falcon-types';
/**
 * Interface. Represents the expected possible input and output of a generic Salesforce CLI call.
 */
export interface SfdxUtilityResultDetail {
    sfdxCommandString: string;
    stdOutParsed: AnyJson;
    stdOutBuffer: string;
    stdErrBuffer: string;
    error: Error;
}
/**
 * @class       SfdxOrgInfo
 * @summary     Stores information about orgs that are connected to the local Salesforce CLI.
 * @description ???
 * @public
 */
export declare class SfdxOrgInfo {
    readonly alias: string;
    readonly username: string;
    readonly orgId: string;
    readonly connectedStatus: string;
    private _isDevHub;
    private _isPkgOrg;
    private _isEnvHub;
    private _nsPrefix;
    private _packages;
    private _pkgVersionMap;
    private _metadataPackageResults;
    readonly isDevHub: boolean;
    readonly isEnvHub: boolean;
    readonly isPkgOrg: boolean;
    readonly latestManagedBetaPkgVersion: MetadataPackageVersion;
    readonly latestManagedReleasedPkgVersion: MetadataPackageVersion;
    readonly managedPkgId: string;
    readonly managedPkgName: string;
    readonly nsPrefix: string;
    readonly packages: MetadataPackage[];
    /**
     * @constructs  SfdxOrgInfo
     * @param       {SfdxOrgInfoSetup}  opts Required. Sets initial values.
     * @description Constructs a SfdxOrgInfo object.
     * @public
     */
    constructor(opts: SfdxOrgInfoSetup);
    /**
     * @method      determineEnvHubStatus
     * @returns     {Promise<boolean}
     * @description Performs a query against this object's related org to find out
     *              if it's an Environment Hub or not.
     * @public @async
     */
    determineEnvHubStatus(): Promise<boolean>;
    /**
     * @method      determinePkgOrgStatus
     * @returns     {Promise<boolean}
     * @description Performs a query against this object's related org to find out
     *              if it's a Packaging Org or not.
     * @public @async
     */
    determinePkgOrgStatus(): Promise<boolean>;
    /**
     * @method      getLatestPackageVersion
     * @param       {string}  metadataPackageId  Required. The Metadata Package ID
     *              (033) that the caller wants the latest package version for.
     * @param       {string}  [releaseState]  Optional. The "Release State" (eg.
     *              "Released" or "Beta") that the caller is interested in.
     * @returns     {MetadataPackageVersion}
     * @description Given a Metadata Package ID (033), returns the associated
     *              Metadata Package Version that was most recently uploaded. If
     *              the caller provides a target "Release State", ensure the chosen
     *              package version matches.
     * @public
     */
    getLatestPackageVersion(metadataPackageId: string, releaseState?: string): MetadataPackageVersion;
    /**
     * @method      extractPackages
     * @param       {QueryResult<MetadataPackage>} metadataPackages  Required. The
     *              output of a JSForce Tooling API query.
     * @returns     {MetadataPackage[]}
     * @description Given the results of a JSForce Tooling API query that fetches
     *              the package info from an org, extract the package and package
     *              version records and create a simplified MetadataPackage array.
     * @private
     */
    private extractPackages;
    /**
     * @method      mapPackageVersions
     * @param       {MetadataPackage[]} packageObjs Required. Array of Metadata
     *              Package objects.
     * @returns     {PackageVersionMap}
     * @description Given an array of MetadataPackage objects, extracts the
     *              Metadata Package Versions from each one and returns a map
     *              whose keys are Metadata Package IDs and whose values are arrays
     *              of MetadataPackageVersion objects sorted by major, minor, and
     *              patch version.
     * @private
     */
    private mapPackageVersions;
}
/**
 * @function    buildSfdxOrgInfoMap
 * @param       {RawSfdxOrgInfo[]}  rawSfdxOrgList  This should be the raw list of SFDX orgs that
 *              comes in the result of a call to force:org:list.
 * @returns     {SfdxOrgInfoMap} Map containing SfdxOrgInfo objects for EVERY org that
 *              was part of the raw SFDX org list. The Org Info's "username" value will be used
 *              as the key for this map.
 * @description Given a raw list of SFDX Org Information (like what you get from force:org:list),
 *              creates an SfdxOrgInfo object for each one.
 * @public
 */
export declare function buildSfdxOrgInfoMap(rawSfdxOrgList: RawSfdxOrgInfo[]): SfdxOrgInfoMap;
/**
 * @function    detectSalesforceCliError
 * @param       {unknown} thingToCheck  Required. Either a string buffer containing an
 *              stderr CLI response or a safeParse() JSON object that (hopefully) came from a
 *              Salesforce CLI command.
 * @returns     {boolean} Returns TRUE if the stdOutBuffer contains something that might be
 *              considered an error.  FALSE if otherwise.
 * @description Given a string buffer containing an stdout response, determines if that response
 *              should be considered a Salesforce CLI error. Please note that there could still be
 *              something wrong with the result even if this function returns FALSE.  It just means
 *              that stdOutBuffer did not contain something that could be interpreted as a
 *              Salesforce CLI Error.
 * @public
 */
export declare function detectSalesforceCliError(thingToCheck: unknown): boolean;
/**
 * @function    executeSfdxCommand
 * @param       {string}  sfdxCommandString Required. String containing an "sfdx force" command.
 * @param       {SfdxFalconResult}  utilityResult Required. Falcon Result used to track actions here.
 * @param       {object}  [messages]  Optional. Success, failure, and "mixed" messages.
 * @returns     {Promise<SfdxFalconResult>} Uses an SfdxShellResult to return data to the caller for
 *              both RESOLVE and REJECT.
 * @description Uses the Salesforce CLI to execute the "sfdx force" command provided by the caller.
 *              All data returned from the CLI command execution is wrapped up in an SfdxFalconResult
 *              object, with stdout and stderr contained within the RESULT's details object.
 * @public @async
 */
export declare function executeSfdxCommand(sfdxCommandString: string, utilityResult: SfdxFalconResult, messages?: object): Promise<SfdxFalconResult>;
/**
 * @function    fetchMetadataPackages
 * @param       {string}  aliasOrUsername Required. The alias or username associated with a current
 *              Salesforce CLI connected org.
 * @param       {string[]}  packageNames  Required. String array containing the names of all
 *              packages that should be retrieved.
 * @param       {string}  retrieveTargetDir Required. The root of the directory structure where
 *              the retrieved .zip or metadata files are put.
 * @returns     {Promise<SfdxFalconResult>} Uses an SfdxShellResult to return data to the caller for
 *              both RESOLVE and REJECT.
 * @description Uses the Salesforce CLI's force:mdapi:retrieve command to retrieve one or more
 *              metadata packages from the target org.
 * @public @async
 */
export declare function fetchMetadataPackages(aliasOrUsername: string, packageNames: string[], retrieveTargetDir: string): Promise<SfdxFalconResult>;
/**
 * @function    getConnection
 * @param       {string}  orgAlias  Required. The alias of the org to create a connection to.
 * @param       {string}  [apiVersion]  Optional. Expects format "[1-9][0-9].0", i.e. 42.0.
 * @returns     {Promise<Connection>} Resolves with an authenticated JSForce Connection object.
 * @description Given an SFDX alias, resolves with an authenticated JSForce Connection object
 * @public @async
 */
export declare function getConnection(aliasOrUsername: string, apiVersion?: string): Promise<Connection>;
/**
 * @function    getUsernameFromAlias
 * @param       {string}  sfdxAlias The local SFDX alias whose Salesforce Username should be found.
 * @returns     {Promise<string>}   Resolves to the username if the alias was found, NULL if not.
 * @description Given an SFDX org alias, return the Salesforce Username associated with the alias
 *              in the local environment the CLI is running in.
 * @public @async
 */
export declare function getUsernameFromAlias(sfdxAlias: string): Promise<string>;
/**
 * @function    identifyDevHubOrgs
 * @param       {SfdxOrgInfoMap} sfdxOrgInfoMap  This should be the baseline list of SFDX
 *              Org Info objects previously created by a call to buildSfdxOrgInfoMap().
 * @returns     {SfdxOrgInfo[]} Array containing only SfdxOrgInfo objects that point to
 *              Dev Hub orgs.
 * @description Given a list of SFDX Org Info objects previously created by a call to
 *              buildSfdxOrgInfoMap(), finds all the org connections that point to DevHub Orgs and
 *              returns them as an array of SfdxOrgInfo objects.
 * @public
 */
export declare function identifyDevHubOrgs(sfdxOrgInfoMap: SfdxOrgInfoMap): SfdxOrgInfo[];
/**
 * @function    identifyEnvHubOrgs
 * @param       {SfdxOrgInfoMap} sfdxOrgInfoMap  This should be the baseline list of SFDX
 *              Org Info objects previously created by a call to buildSfdxOrgInfoMap().
 * @returns     {Promise<SfdxOrgInfo[]>}  Resolves with an array containing only SfdxOrgInfo objects
 *              that point to Environment Hub Orgs.
 * @description Given a list of SFDX Org Info objects previously created by a call to
 *              buildSfdxOrgInfoMap(), finds all the org connections that point to Environment Hub
 *              Orgs and returns them as an array of SfdxOrgInfo objects.
 * @public @async
 */
export declare function identifyEnvHubOrgs(sfdxOrgInfoMap: SfdxOrgInfoMap): Promise<SfdxOrgInfo[]>;
/**
 * @function    identifyPkgOrgs
 * @param       {SfdxOrgInfoMap} sfdxOrgInfoMap  This should be the baseline list of SFDX
 *              Org Info objects previously created by a call to buildSfdxOrgInfoMap().
 * @returns     {Promise<SfdxOrgInfo[]>}  Resolves with an array containing only SfdxOrgInfo objects
 *              that point to Packaging Orgs.
 * @description Given a map of SFDX Org Info objects previously created by a call to
 *              buildSfdxOrgInfoMap(), finds all the org connections that point to Packaging Orgs and
 *              returns them as an array of SfdxOrgInfo objects.
 * @public @async
 */
export declare function identifyPkgOrgs(sfdxOrgInfoMap: SfdxOrgInfoMap): Promise<SfdxOrgInfo[]>;
/**
 * @function    mdapiConvert
 * @param       {string}  mdapiSourceRootDir  Required. ???
 * @param       {string}  sfdxSourceOutputDir Required. ???
 * @returns     {Promise<SfdxFalconResult>} Uses an SfdxShellResult to return data to the caller for
 *              both RESOLVE and REJECT.
 * @description Uses the Salesforce CLI's force:mdapi:convert command to convert MDAPI source to
 *              SFDX source.
 * @public @async
 */
export declare function mdapiConvert(mdapiSourceRootDir: string, sfdxSourceOutputDir: string): Promise<SfdxFalconResult>;
/**
 * @function    resolveConnection
 * @param       {AliasOrConnection} aliasOrConnection  Required. Either a string containing the
 *              Alias of the org being queried or an authenticated JSForce Connection object.
 * @returns     {Promise<ResolvedConnection>}  Resolves with an authenticated JSForce Connection.
 * @description Given an Alias/Username or a JSForce Connection, returns a valid JSForce Connection.
 * @public @async
 */
export declare function resolveConnection(aliasOrConnection: AliasOrConnection): Promise<ResolvedConnection>;
/**
 * @function    scanConnectedOrgs
 * @returns     {Promise<SfdxFalconResult>} Uses an SfdxFalconResult to return data to the caller for
 *              both RESOLVE and REJECT.
 * @description Calls force:org:list via an async shell command, then creates an array of
 *              SfdxOrgInfo objects by parsing the JSON response returned by the CLI command.
 *              Sends results back to caller as an SfdxFalconResult.
 * @public @async
 */
export declare function scanConnectedOrgs(): Promise<SfdxFalconResult>;
