"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
//─────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @file          modules/sfdx-falcon-util/sfdx.ts
 * @copyright     Vivek M. Chawla - 2018
 * @author        Vivek M. Chawla <@VivekMChawla>
 * @summary       Utility Module - SFDX
 * @description   Utility functions related to Salesforce DX and the Salesforce CLI
 * @version       1.0.0
 * @license       MIT
 */
//─────────────────────────────────────────────────────────────────────────────────────────────────┘
// Import External Modules
const core_1 = require("@salesforce/core"); // Why?
const core_2 = require("@salesforce/core"); // Why?
const core_3 = require("@salesforce/core"); // Why?
// Import Internal Modules
const sfdx_falcon_debug_1 = require("../sfdx-falcon-debug"); // Class. Specialized debug provider for SFDX-Falcon code.
const sfdx_falcon_error_1 = require("../sfdx-falcon-error"); // Class. Extends SfdxError to provide specialized error structures for SFDX-Falcon modules.
const sfdx_falcon_error_2 = require("../sfdx-falcon-error"); // Class. Extends SfdxFalconError to provide specialized error handling of error results returned from CLI commands run via shell exec.
const sfdx_falcon_error_3 = require("../sfdx-falcon-error"); // Class. Extends SfdxFalconError to provide specialized error handling of error results returned by failed shell commands
const sfdx_falcon_result_1 = require("../sfdx-falcon-result"); // Why?
// Import Utility Functions
const sfdx_falcon_util_1 = require("../sfdx-falcon-util"); // Function. Given any content to parse, returns a JavaScript object based on that content.
const async_1 = require("../sfdx-falcon-util/async"); // Function. Can be used to introduce a delay when called inside async functions with the "await" keyword.
//import {toolingApiQuery}        from  '../sfdx-falcon-util/jsforce';  // Function. Given an Org Alias or JSForce Connection, makes a REST call to the target org's tooling.
const jsforce_1 = require("../sfdx-falcon-util/jsforce"); // Function. Given an Org Alias or a JSForce Connection, tries to get an "Object Describe" back for the SignupRequest SObject.
const jsforce_2 = require("../sfdx-falcon-util/jsforce"); // Function. Given an Org Alias or a JSForce Connection, queries the related org and returns a QueryResult containing the MetadataPackage objects and their child objects.
// Requires
const shell = require('shelljs'); // Cross-platform shell access - use for setting up Git repo.
// Set the File Local Debug Namespace
const dbgNs = 'UTILITY:sfdx:';
//─────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @class       SfdxOrgInfo
 * @summary     Stores information about orgs that are connected to the local Salesforce CLI.
 * @description ???
 * @public
 */
//─────────────────────────────────────────────────────────────────────────────────────────────────┘
class SfdxOrgInfo {
    // Accessors
    get isDevHub() {
        return this._isDevHub ? true : false;
    }
    get isEnvHub() {
        return this._isEnvHub ? true : false;
    }
    get isPkgOrg() {
        return this._isPkgOrg ? true : false;
    }
    get latestManagedBetaPkgVersion() {
        return this.getLatestPackageVersion(this.managedPkgId, 'Beta');
    }
    get latestManagedReleasedPkgVersion() {
        return this.getLatestPackageVersion(this.managedPkgId, 'Released');
    }
    get managedPkgId() {
        for (const packageObj of this._packages) {
            if (packageObj.NamespacePrefix) {
                return packageObj.Id;
            }
        }
        return '';
    }
    get managedPkgName() {
        for (const packageObj of this._packages) {
            if (packageObj.NamespacePrefix) {
                return packageObj.Name;
            }
        }
        return '';
    }
    get nsPrefix() {
        return this._nsPrefix || '';
    }
    get packages() {
        return this._packages;
    }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @constructs  SfdxOrgInfo
     * @param       {SfdxOrgInfoSetup}  opts Required. Sets initial values.
     * @description Constructs a SfdxOrgInfo object.
     * @public
     */
    //───────────────────────────────────────────────────────────────────────────┘
    constructor(opts) {
        // Initialize core class members.
        this.alias = opts.alias;
        this.username = opts.username;
        this.orgId = opts.orgId;
        this.connectedStatus = opts.connectedStatus;
        // Initialize org identifcation members.
        this._isDevHub = opts.isDevHub ? true : false;
        this._isPkgOrg = false;
        this._isEnvHub = false;
        // Initialize package-related class members.
        this._metadataPackageResults = opts.metadataPackageResults;
        this._packages = this.extractPackages(this._metadataPackageResults);
        this._pkgVersionMap = this.mapPackageVersions(this._packages);
        // If there is at least one member in the packages array, mark this as a Packaging Org.
        this._isPkgOrg = (this._packages.length > 0);
        // If no alias was set, copy the username over as the alias.
        if (typeof this.alias !== 'string' || this.alias.length < 1) {
            this.alias = this.username;
        }
    }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      determineEnvHubStatus
     * @returns     {Promise<boolean}
     * @description Performs a query against this object's related org to find out
     *              if it's an Environment Hub or not.
     * @public @async
     */
    //───────────────────────────────────────────────────────────────────────────┘
    async determineEnvHubStatus() {
        // Check if the aliased user is able to get a describe of the SignupRequest object.
        const signupRequestDescribe = await jsforce_1.describeSignupRequest(this.alias);
        sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}determineEnvHubStatus:signupRequestDescribe:`, signupRequestDescribe, `signupRequestDescribe: `);
        // Make sure that SignupRequest is CREATABLE. Anything else means NOT an EnvHub.
        if (signupRequestDescribe.createable === true) {
            this._isEnvHub = true;
        }
        else {
            this._isEnvHub = false;
        }
        // Done! Let the caller know what the answer is.
        return this._isEnvHub;
    }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      determinePkgOrgStatus
     * @returns     {Promise<boolean}
     * @description Performs a query against this object's related org to find out
     *              if it's a Packaging Org or not.
     * @public @async
     */
    //───────────────────────────────────────────────────────────────────────────┘
    async determinePkgOrgStatus() {
        // Don't do anything if we already know this is a packaging org.
        if (this._packages.length > 0) {
            return this._isPkgOrg = true;
        }
        // Run Tooling API query.
        this._metadataPackageResults = await jsforce_2.getPackages(this.alias);
        sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}SfdxOrgInfo:determinePkgOrgStatus:`, this._metadataPackageResults, `this._metadataPackageResults: `);
        // Extract any packages from the results we just got.
        this._packages = this.extractPackages(this._metadataPackageResults);
        sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}SfdxOrgInfo:determinePkgOrgStatus:`, this._packages, `this._packages: `);
        // Create a Package Version Map.
        this._pkgVersionMap = this.mapPackageVersions(this._packages);
        sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}SfdxOrgInfo:determinePkgOrgStatus:`, this._pkgVersionMap, `this._pkgVersionMap: `);
        // Search the packages for a namespace.
        for (const packageObj of this._packages) {
            if (packageObj.NamespacePrefix) {
                this._nsPrefix = packageObj.NamespacePrefix;
                break;
            }
        }
        // If there is at least one member in the packages array, mark this as a Packaging Org.
        return this._isPkgOrg = (this._packages.length > 0);
    }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      getLatestPackageVersion
     * @param       {string}  metadataPackageId  Required. The Metadata Package ID
     *              (033) that the caller wants the latest package version for.
     * @param       {string}  [releaseState]  Optional. The "Release State" (eg.
     *              "Released" or "Beta") that the caller is interested in.
     * @returns     {MetadataPackageVersion}
     * @description Given a Metadata Package ID (033), returns the associated
     *              Metadata Package Version that was most recently uploaded. If
     *              the caller provides a target "Release State", ensure the chosen
     *              package version matches.
     * @public
     */
    //───────────────────────────────────────────────────────────────────────────┘
    getLatestPackageVersion(metadataPackageId, releaseState = '') {
        // Try to find an array of Package Versions for the provided Metadata Package ID.
        const metadataPackageVersions = this._pkgVersionMap.get(metadataPackageId);
        // If no Metadata Package Versions were found, return NULL.
        if (Array.isArray(metadataPackageVersions) !== true || metadataPackageVersions.length < 1) {
            return null;
        }
        // If the caller didn't specify a Release State to look for, return the first Package Version.
        // The Package Versions should ALREADY be sorted by major, minor, and patch version number
        // in DESCENDING order. The first element in the array is the LAST version uploaded.
        if (!releaseState) {
            return metadataPackageVersions.values[0];
        }
        // Iterate over the Metadata Package Versions till we find one with the RELEASED state.
        for (const metadataPackageVersion of metadataPackageVersions) {
            if (metadataPackageVersion.ReleaseState === releaseState) {
                return metadataPackageVersion;
            }
        }
        // We couldn't find a Package Version matching the caller's criteria. Return NULL.
        return null;
    }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      extractPackages
     * @param       {QueryResult<MetadataPackage>} metadataPackages  Required. The
     *              output of a JSForce Tooling API query.
     * @returns     {MetadataPackage[]}
     * @description Given the results of a JSForce Tooling API query that fetches
     *              the package info from an org, extract the package and package
     *              version records and create a simplified MetadataPackage array.
     * @private
     */
    //───────────────────────────────────────────────────────────────────────────┘
    extractPackages(metadataPackages) {
        // Debug incoming arguments
        sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}SfdxOrgInfo:extractPackages:arguments:`, arguments, `arguments: `);
        // If there isn't a Metadata Package Query Result, return an empty array.
        if (typeof metadataPackages === 'undefined' || Array.isArray(metadataPackages.records) !== true) {
            return [];
        }
        // Initialize a Metadata Package array.
        const packages = new Array();
        // Copy over core information, then extract any Metadata Package Version records.
        for (const metadataPackageRecord of metadataPackages.records) {
            const metadataPackage = {
                Id: metadataPackageRecord.Id,
                Name: metadataPackageRecord.Name,
                NamespacePrefix: metadataPackageRecord.NamespacePrefix,
                MetadataPackageVersions: metadataPackageRecord.MetadataPackageVersions
                    ?
                        metadataPackageRecord.MetadataPackageVersions['records']
                    :
                        []
            };
            // Add this to the Packages array.
            packages.push(metadataPackage);
        }
        // All done.
        return packages;
    }
    //───────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      mapPackageVersions
     * @param       {MetadataPackage[]} packageObjs Required. Array of Metadata
     *              Package objects.
     * @returns     {PackageVersionMap}
     * @description Given an array of MetadataPackage objects, extracts the
     *              Metadata Package Versions from each one and returns a map
     *              whose keys are Metadata Package IDs and whose values are arrays
     *              of MetadataPackageVersion objects sorted by major, minor, and
     *              patch version.
     * @private
     */
    //───────────────────────────────────────────────────────────────────────────┘
    mapPackageVersions(packageObjs) {
        // Initialize the Package Version Map.
        const pkgVersionMap = new Map();
        // Iterate over the array of Metadata Package objects.
        for (const packageObj of packageObjs) {
            // If this Package Object does NOT have any Metadata Package Versions, put an empty array and continue.
            if (Array.isArray(packageObj.MetadataPackageVersions) !== true || packageObj.MetadataPackageVersions.length < 1) {
                pkgVersionMap.set(packageObj.Id, []);
                continue;
            }
            // Sort the Metadata Package Versions array by version (major, minor, patch - DESC).
            packageObj.MetadataPackageVersions.sort((a, b) => {
                // Make sure we deal only with Numbers.
                const aMajorVersion = Number.isNaN(a.MajorVersion) ? 0 : Number(a.MajorVersion);
                const bMajorVersion = Number.isNaN(b.MajorVersion) ? 0 : Number(b.MajorVersion);
                const aMinorVersion = Number.isNaN(a.MinorVersion) ? 0 : Number(a.MinorVersion);
                const bMinorVersion = Number.isNaN(b.MinorVersion) ? 0 : Number(b.MinorVersion);
                const aPatchVersion = Number.isNaN(a.PatchVersion) ? 0 : Number(a.PatchVersion);
                const bPatchVersion = Number.isNaN(b.PatchVersion) ? 0 : Number(b.PatchVersion);
                // Sorting Result Rules: a<b: return -1; a==b: return 0; a>b: return 1
                // Compare Major Version.
                if (aMajorVersion < bMajorVersion)
                    return -1;
                if (aMajorVersion > bMajorVersion)
                    return 1;
                // Major Versions are equal. Compare Minor Versions.
                if (aMinorVersion < bMinorVersion)
                    return -1;
                if (aMinorVersion > bMinorVersion)
                    return 1;
                // Minor Versions are equal. Compare Patch Versions.
                if (aPatchVersion < bPatchVersion)
                    return -1;
                if (aPatchVersion > bPatchVersion)
                    return 1;
                // Major, Minor, and Patch Versions are all equal.
                return 0;
            });
            // Reverse the array so we get a DESCENDING sort order.
            packageObj.MetadataPackageVersions.reverse();
            // Place the sorted array in the Package Version Map.
            pkgVersionMap.set(packageObj.Id, packageObj.MetadataPackageVersions);
        }
        return pkgVersionMap;
    }
}
exports.SfdxOrgInfo = SfdxOrgInfo;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    buildSfdxOrgInfoMap
 * @param       {RawSfdxOrgInfo[]}  rawSfdxOrgList  This should be the raw list of SFDX orgs that
 *              comes in the result of a call to force:org:list.
 * @returns     {SfdxOrgInfoMap} Map containing SfdxOrgInfo objects for EVERY org that
 *              was part of the raw SFDX org list. The Org Info's "username" value will be used
 *              as the key for this map.
 * @description Given a raw list of SFDX Org Information (like what you get from force:org:list),
 *              creates an SfdxOrgInfo object for each one.
 * @public
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function buildSfdxOrgInfoMap(rawSfdxOrgList) {
    // Debug incoming arguments
    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}buildSfdxOrgInfoMap:arguments:`, arguments, `arguments: `);
    // Make sure that the caller passed us an Array.
    if (Array.isArray(rawSfdxOrgList) !== true) {
        throw new sfdx_falcon_error_1.SfdxFalconError(`Expected rawSfdxOrgList to an Array but got type '${typeof rawSfdxOrgList}' instead.`, `TypeError`, `${dbgNs}buildSfdxOrgInfoMap`);
    }
    // Initalize an array to hold the SfdxOrgInfo objects we're going to create.
    const sfdxOrgInfoMap = new Map();
    // Iterate over the raw list of orgs to create SfdxOrgInfo objects.
    for (const rawOrgInfo of rawSfdxOrgList) {
        // Only work with orgs that have a CONNECTED status.
        if (rawOrgInfo.connectedStatus === 'Connected') {
            // Create a new SfdxOrgInfo object and add it to the Map using the Username as the key.
            sfdxOrgInfoMap.set(rawOrgInfo.username, new SfdxOrgInfo({
                alias: rawOrgInfo.alias,
                username: rawOrgInfo.username,
                orgId: rawOrgInfo.orgId,
                connectedStatus: rawOrgInfo.connectedStatus,
                isDevHub: rawOrgInfo.isDevHub
            }));
        }
        else {
            sfdx_falcon_debug_1.SfdxFalconDebug.str(`${dbgNs}buildSfdxOrgInfoMap:`, `${rawOrgInfo.alias}(${rawOrgInfo.username})`, `ORG NOT CONNECTED!`);
        }
    }
    // DEBUG
    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}buildSfdxOrgInfoMap:sfdxOrgInfoMap:`, sfdxOrgInfoMap, `sfdxOrgInfoMap: `);
    // Return the SFDX Org Infos to the caller. Let them worry about putting it into Shared Data.
    return sfdxOrgInfoMap;
}
exports.buildSfdxOrgInfoMap = buildSfdxOrgInfoMap;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    detectSalesforceCliError
 * @param       {unknown} thingToCheck  Required. Either a string buffer containing an
 *              stderr CLI response or a safeParse() JSON object that (hopefully) came from a
 *              Salesforce CLI command.
 * @returns     {boolean} Returns TRUE if the stdOutBuffer contains something that might be
 *              considered an error.  FALSE if otherwise.
 * @description Given a string buffer containing an stdout response, determines if that response
 *              should be considered a Salesforce CLI error. Please note that there could still be
 *              something wrong with the result even if this function returns FALSE.  It just means
 *              that stdOutBuffer did not contain something that could be interpreted as a
 *              Salesforce CLI Error.
 * @public
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function detectSalesforceCliError(thingToCheck) {
    // Debug incoming arguments
    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}detectSalesforceCliError:arguments:`, arguments, `arguments: `);
    // Parse thingToCheck if it's a string, assign it directly if not.
    let possibleCliError;
    if (typeof thingToCheck === 'string') {
        possibleCliError = sfdx_falcon_util_1.safeParse(thingToCheck);
    }
    else {
        possibleCliError = thingToCheck;
    }
    // Debug
    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}detectSalesforceCliError:possibleCliError:`, possibleCliError, `possibleCliError: `);
    // If the Possible CLI Error "status" property is present AND has a non-zero value, then IT IS a Salesforce CLI Error.
    if (possibleCliError['status'] && possibleCliError['status'] !== 0) {
        return true; // This is definitely a Salesforce CLI Error
    }
    else {
        return false; // This is NOT a Salesforce CLI Error.
    }
}
exports.detectSalesforceCliError = detectSalesforceCliError;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    executeSfdxCommand
 * @param       {string}  sfdxCommandString Required. String containing an "sfdx force" command.
 * @param       {SfdxFalconResult}  utilityResult Required. Falcon Result used to track actions here.
 * @param       {object}  [messages]  Optional. Success, failure, and "mixed" messages.
 * @returns     {Promise<SfdxFalconResult>} Uses an SfdxShellResult to return data to the caller for
 *              both RESOLVE and REJECT.
 * @description Uses the Salesforce CLI to execute the "sfdx force" command provided by the caller.
 *              All data returned from the CLI command execution is wrapped up in an SfdxFalconResult
 *              object, with stdout and stderr contained within the RESULT's details object.
 * @public @async
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
async function executeSfdxCommand(sfdxCommandString, utilityResult, messages = {}) {
    // Extract the Detail object from the Utility Result.
    const utilityResultDetail = utilityResult.detail;
    // Override default messages if provided by the caller.
    const failureMessage = messages['failureMessage'] || 'Salesforce CLI Command Failed';
    const successMessage = messages['successMessage'] || 'Salesforce CLI Command Succeeded';
    const mixedMessage = messages['mixedMessage'] || 'Salesforce CLI Command failed but returned a Success Response';
    return new Promise((resolve, reject) => {
        // Declare a function-local string buffer to hold the stdio stream.
        let stdOutBuffer = '';
        let stdErrBuffer = '';
        // Set the FORCE_COLOR environment variable to 0.
        // This prevents the possibility of ANSI Escape codes polluting STDOUT
        shell.env['FORCE_COLOR'] = 0;
        // Set the SFDX_JSON_TO_STDOUT environment variable to TRUE.
        // This won't be necessary after CLI v45.  See CLI v44.2.0 release notes for more info.
        shell.env['SFDX_JSON_TO_STDOUT'] = 'true';
        // Set the SFDX_AUTOUPDATE_DISABLE environment variable to TRUE.
        // This may help prevent strange typescript compile errors when internal SFDX CLI commands are executed.
        shell.env['SFDX_AUTOUPDATE_DISABLE'] = 'true';
        // Run force:org:list asynchronously inside a child process.
        const childProcess = shell.exec(sfdxCommandString, { silent: true, async: true });
        // Capture stdout datastream. Data is piped in from stdout in small chunks, so prepare for multiple calls.
        childProcess.stdout.on('data', (stdOutDataStream) => {
            stdOutBuffer += stdOutDataStream;
        });
        // Capture the stderr datastream. Values should only come here if there was a shell error.
        // CLI warnings used to be sent to stderr as well, but as of CLI v45 all output should be going to stdout.
        childProcess.stderr.on('data', (stdErrDataStream) => {
            stdErrBuffer += stdErrDataStream;
        });
        // Handle Child Process "close". Fires only once the contents of stdout and stderr are read.
        childProcess.on('close', (code, signal) => {
            // Store BOTH stdout and stderr buffers (this helps track stderr WARNING messages)
            utilityResultDetail.stdOutBuffer = stdOutBuffer;
            utilityResultDetail.stdErrBuffer = stdErrBuffer;
            // Determine if the command succeded or failed.
            if (code !== 0) {
                if (detectSalesforceCliError(stdOutBuffer)) {
                    // We have a Salesforce CLI Error. Prepare ERROR detail using SfdxCliError.
                    utilityResultDetail.error = new sfdx_falcon_error_2.SfdxCliError(sfdxCommandString, stdOutBuffer, `${failureMessage}`, `${dbgNs}executeSfdxCommand`);
                }
                else {
                    // We have a shell Error. Prepare ERROR detail using ShellError.
                    utilityResultDetail.error = new sfdx_falcon_error_3.ShellError(sfdxCommandString, code, signal, stdErrBuffer, stdOutBuffer, `${dbgNs}executeSfdxCommand`);
                }
                // Process this as an ERROR result.
                utilityResult.error(utilityResultDetail.error);
                utilityResult.debugResult(`${failureMessage}`, `${dbgNs}executeSfdxCommand:`);
                // Process this as an ERROR result.
                reject(utilityResult);
            }
            else {
                //The code below can be used to simulate invalid JSON response that sometimes comes from the Salesforce CLI
                //stdOutBuffer = '\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1GProcessing... \\\u001b[2K\u001b[1GProcessing... |\u001b[2K\u001b[1GProcessing... /\u001b[2K\u001b[1GProcessing... -\u001b[2K\u001b[1G{"message":"The request to create a scratch org failed with error code: C-9999.","status":1,"stack":"RemoteOrgSignupFailed: The request to create a scratch org failed with error code: C-9999.\\n    at force.retrieve.then (/Users/vchawla/.local/share/sfdx/client/node_modules/salesforce-alm/dist/lib/scratchOrgInfoApi.js:333:25)\\n    at tryCatcher (/Users/vchawla/.local/share/sfdx/client/node_modules/bluebird/js/release/util.js:16:23)\\n    at Promise._settlePromiseFromHandler (/Users/vchawla/.local/share/sfdx/client/node_modules/bluebird/js/release/promise.js:510:31)\\n    at Promise._settlePromise (/Users/vchawla/.local/share/sfdx/client/node_modules/bluebird/js/release/promise.js:567:18)\\n    at Promise._settlePromise0 (/Users/vchawla/.local/share/sfdx/client/node_modules/bluebird/js/release/promise.js:612:10)\\n    at Promise._settlePromises (/Users/vchawla/.local/share/sfdx/client/node_modules/bluebird/js/release/promise.js:691:18)\\n    at Async._drainQueue (/Users/vchawla/.local/share/sfdx/client/node_modules/bluebird/js/release/async.js:138:16)\\n    at Async._drainQueues (/Users/vchawla/.local/share/sfdx/client/node_modules/bluebird/js/release/async.js:148:10)\\n    at Immediate.Async.drainQueues (/Users/vchawla/.local/share/sfdx/client/node_modules/bluebird/js/release/async.js:17:14)\\n    at runCallback (timers.js:789:20)\\n    at tryOnImmediate (timers.js:751:5)\\n    at processImmediate [as _immediateCallback] (timers.js:722:5)","name":"RemoteOrgSignupFailed","warnings":[]}\n'
                // Make sure we got back a valid JSON Response
                const stdOutJsonResponse = stdOutBuffer.substring(stdOutBuffer.indexOf('{'), stdOutBuffer.lastIndexOf('}') + 1);
                const parsedCliResponse = sfdx_falcon_util_1.safeParse(stdOutJsonResponse);
                // Unparseable responses from the CLI are SHELL ERRORS and should be rejected.
                if (parsedCliResponse['unparsed']) {
                    utilityResultDetail.error = new sfdx_falcon_error_3.ShellError(sfdxCommandString, code, signal, stdErrBuffer, stdOutBuffer, `${dbgNs}executeSfdxCommand`);
                    utilityResult.error(utilityResultDetail.error);
                    reject(utilityResult);
                }
                // Parseable responses might be CLI ERRORS and should be marked ERROR and rejected if so.
                if (detectSalesforceCliError(parsedCliResponse)) {
                    utilityResultDetail.error = new sfdx_falcon_error_2.SfdxCliError(sfdxCommandString, stdOutJsonResponse, `${failureMessage}`, `${dbgNs}executeSfdxCommand:`);
                    utilityResult.error(utilityResultDetail.error);
                    utilityResult.debugResult(`${mixedMessage}`, `${dbgNs}executeSfdxCommand:`);
                    reject(utilityResult);
                }
                // If we get here, the call was successful. Prepare the SUCCESS detail for this function's Result.
                utilityResultDetail.stdOutParsed = parsedCliResponse;
                // Regiser a SUCCESS result
                utilityResult.success();
                utilityResult.debugResult(`${successMessage}`, `${dbgNs}executeSfdxCommand:`);
                // Resolve with the successful SFDX-Falcon Result.
                resolve(utilityResult);
            }
        });
    });
}
exports.executeSfdxCommand = executeSfdxCommand;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    fetchMetadataPackages
 * @param       {string}  aliasOrUsername Required. The alias or username associated with a current
 *              Salesforce CLI connected org.
 * @param       {string[]}  packageNames  Required. String array containing the names of all
 *              packages that should be retrieved.
 * @param       {string}  retrieveTargetDir Required. The root of the directory structure where
 *              the retrieved .zip or metadata files are put.
 * @returns     {Promise<SfdxFalconResult>} Uses an SfdxShellResult to return data to the caller for
 *              both RESOLVE and REJECT.
 * @description Uses the Salesforce CLI's force:mdapi:retrieve command to retrieve one or more
 *              metadata packages from the target org.
 * @public @async
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
async function fetchMetadataPackages(aliasOrUsername, packageNames, retrieveTargetDir) {
    // Set the SFDX Command String to be used by this function.
    const sfdxCommandString = `sfdx force:mdapi:retrieve `
        + ` --targetusername ${aliasOrUsername}`
        + ` --packagenames "${packageNames.join('","')}"`
        + (packageNames.length === 1 ? ' --singlepackage' : '')
        + ` --retrievetargetdir ${retrieveTargetDir}`
        + ` --wait 10`
        + ` --loglevel debug`
        + ` --json`;
    // Introduce a small delay in case this is being used by an Observable Listr Task.
    await async_1.waitASecond(3);
    // Initialize a UTILITY Result for this function.
    const utilityResult = new sfdx_falcon_result_1.SfdxFalconResult(`sfdx:fetchMetadataPackages`, "UTILITY" /* UTILITY */);
    const utilityResultDetail = {
        sfdxCommandString: sfdxCommandString,
        stdOutParsed: null,
        stdOutBuffer: null,
        stdErrBuffer: null,
        error: null
    };
    utilityResult.detail = utilityResultDetail;
    utilityResult.debugResult('Utility Result Initialized', `${dbgNs}fetchMetadataPackages:`);
    // Define the success, failure, and "mixed" messages for the SFDX command execution.
    const messages = {
        failureMessage: 'Package Retrieval Failed',
        successMessage: 'Package Retrieval Succeeded',
        mixedMessage: 'Package retrieval failed but the CLI returned a Success Response'
    };
    // Execute the Salesforce CLI Command.
    return executeSfdxCommand(sfdxCommandString, utilityResult, messages);
}
exports.fetchMetadataPackages = fetchMetadataPackages;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    getConnection
 * @param       {string}  orgAlias  Required. The alias of the org to create a connection to.
 * @param       {string}  [apiVersion]  Optional. Expects format "[1-9][0-9].0", i.e. 42.0.
 * @returns     {Promise<Connection>} Resolves with an authenticated JSForce Connection object.
 * @description Given an SFDX alias, resolves with an authenticated JSForce Connection object
 * @public @async
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
async function getConnection(aliasOrUsername, apiVersion) {
    // Debug incoming arguments
    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}getConnection:arguments:`, arguments, `arguments: `);
    // Fetch the username associated with this alias.
    let username = await getUsernameFromAlias(aliasOrUsername);
    // If the alias didn't result in a username, assume that aliasOrUsername held a username, not a alias.
    if (typeof username === 'undefined') {
        username = aliasOrUsername;
    }
    // DEBUG
    sfdx_falcon_debug_1.SfdxFalconDebug.str(`${dbgNs}getConnection:username:`, username, `username: `);
    // Create an AuthInfo object for the username we have.
    const authInfo = await core_2.AuthInfo.create({ username: username });
    // Create and return a connection to the org attached to the username.
    const connection = await core_3.Connection.create({ authInfo: authInfo });
    // Set the API version (if specified by the caller).
    if (typeof apiVersion !== 'undefined') {
        sfdx_falcon_debug_1.SfdxFalconDebug.str(`${dbgNs}getConnection:apiVersion:`, apiVersion, `apiVersion: `);
        connection.setApiVersion(apiVersion);
    }
    // The connection is ready for use.
    return connection;
}
exports.getConnection = getConnection;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    getUsernameFromAlias
 * @param       {string}  sfdxAlias The local SFDX alias whose Salesforce Username should be found.
 * @returns     {Promise<string>}   Resolves to the username if the alias was found, NULL if not.
 * @description Given an SFDX org alias, return the Salesforce Username associated with the alias
 *              in the local environment the CLI is running in.
 * @public @async
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
async function getUsernameFromAlias(sfdxAlias) {
    const username = await core_1.Aliases.fetch(sfdxAlias);
    return username;
}
exports.getUsernameFromAlias = getUsernameFromAlias;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    identifyDevHubOrgs
 * @param       {SfdxOrgInfoMap} sfdxOrgInfoMap  This should be the baseline list of SFDX
 *              Org Info objects previously created by a call to buildSfdxOrgInfoMap().
 * @returns     {SfdxOrgInfo[]} Array containing only SfdxOrgInfo objects that point to
 *              Dev Hub orgs.
 * @description Given a list of SFDX Org Info objects previously created by a call to
 *              buildSfdxOrgInfoMap(), finds all the org connections that point to DevHub Orgs and
 *              returns them as an array of SfdxOrgInfo objects.
 * @public
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
function identifyDevHubOrgs(sfdxOrgInfoMap) {
    // Debug incoming arguments
    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}identifyDevHubOrgs:arguments:`, arguments, `arguments: `);
    // Make sure that the caller passed us a Map.
    if ((sfdxOrgInfoMap instanceof Map) !== true) {
        throw new sfdx_falcon_error_1.SfdxFalconError(`Expected sfdxOrgInfoMap to a Map but got a '${typeof sfdxOrgInfoMap !== 'undefined' ? sfdxOrgInfoMap.constructor.name : 'undefined'}' instead.`, `TypeError`, `${dbgNs}identifyDevHubOrgs`);
    }
    // Array of SfdxOrgInfo objects that will hold Dev Hubs.
    const devHubOrgInfos = new Array();
    // Iterate over the Org Info list and identify Developer Hub Orgs.
    for (const orgInfo of sfdxOrgInfoMap.values()) {
        if (orgInfo.isDevHub) {
            sfdx_falcon_debug_1.SfdxFalconDebug.str(`${dbgNs}identifyDevHubOrgs:`, `${orgInfo.alias}(${orgInfo.username})`, `DEVELOPER HUB FOUND: `);
            devHubOrgInfos.push(orgInfo);
        }
        else {
            sfdx_falcon_debug_1.SfdxFalconDebug.str(`${dbgNs}identifyDevHubOrgs:`, `${orgInfo.alias}(${orgInfo.username})`, `NOT A DEVELOPER HUB: `);
        }
    }
    // DEBUG
    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}identifyDevHubOrgs:devHubOrgInfos`, devHubOrgInfos, `devHubOrgInfos: `);
    // Return the list of Packaging Orgs to the caller
    return devHubOrgInfos;
}
exports.identifyDevHubOrgs = identifyDevHubOrgs;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    identifyEnvHubOrgs
 * @param       {SfdxOrgInfoMap} sfdxOrgInfoMap  This should be the baseline list of SFDX
 *              Org Info objects previously created by a call to buildSfdxOrgInfoMap().
 * @returns     {Promise<SfdxOrgInfo[]>}  Resolves with an array containing only SfdxOrgInfo objects
 *              that point to Environment Hub Orgs.
 * @description Given a list of SFDX Org Info objects previously created by a call to
 *              buildSfdxOrgInfoMap(), finds all the org connections that point to Environment Hub
 *              Orgs and returns them as an array of SfdxOrgInfo objects.
 * @public @async
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
async function identifyEnvHubOrgs(sfdxOrgInfoMap) {
    // Debug incoming arguments
    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}identifyEnvHubOrgs:arguments:`, arguments, `arguments: `);
    // Make sure that the caller passed us a Map.
    if ((sfdxOrgInfoMap instanceof Map) !== true) {
        throw new sfdx_falcon_error_1.SfdxFalconError(`Expected sfdxOrgInfoMap to a Map but got a '${typeof sfdxOrgInfoMap !== 'undefined' ? sfdxOrgInfoMap.constructor.name : 'undefined'}' instead.`, `TypeError`, `${dbgNs}identifyEnvHubOrgs`);
    }
    // Array of SfdxOrgInfo objects that will hold Dev Hubs.
    const envHubOrgInfos = new Array();
    // Iterate over the Org Info list and identify Environment Hub Orgs.
    for (const orgInfo of sfdxOrgInfoMap.values()) {
        if (await orgInfo.determineEnvHubStatus()) {
            sfdx_falcon_debug_1.SfdxFalconDebug.str(`${dbgNs}identifyEnvHubOrgs:`, `${orgInfo.alias}(${orgInfo.username})`, `ENVIRONMENT HUB FOUND: `);
            envHubOrgInfos.push(orgInfo);
        }
        else {
            sfdx_falcon_debug_1.SfdxFalconDebug.str(`${dbgNs}identifyEnvHubOrgs:`, `${orgInfo.alias}(${orgInfo.username})`, `NOT AN ENVIRONMENT HUB: `);
        }
    }
    // DEBUG
    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}identifyEnvHubOrgs:envHubOrgInfos`, envHubOrgInfos, `envHubOrgInfos: `);
    // Return the list of Packaging Orgs to the caller
    return envHubOrgInfos;
}
exports.identifyEnvHubOrgs = identifyEnvHubOrgs;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    identifyPkgOrgs
 * @param       {SfdxOrgInfoMap} sfdxOrgInfoMap  This should be the baseline list of SFDX
 *              Org Info objects previously created by a call to buildSfdxOrgInfoMap().
 * @returns     {Promise<SfdxOrgInfo[]>}  Resolves with an array containing only SfdxOrgInfo objects
 *              that point to Packaging Orgs.
 * @description Given a map of SFDX Org Info objects previously created by a call to
 *              buildSfdxOrgInfoMap(), finds all the org connections that point to Packaging Orgs and
 *              returns them as an array of SfdxOrgInfo objects.
 * @public @async
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
async function identifyPkgOrgs(sfdxOrgInfoMap) {
    // Debug incoming arguments
    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}identifyPkgOrgs:arguments:`, arguments, `arguments: `);
    // Make sure that the caller passed us a Map.
    if ((sfdxOrgInfoMap instanceof Map) !== true) {
        throw new sfdx_falcon_error_1.SfdxFalconError(`Expected sfdxOrgInfoMap to a Map but got a '${typeof sfdxOrgInfoMap !== 'undefined' ? sfdxOrgInfoMap.constructor.name : 'undefined'}' instead.`, `TypeError`, `${dbgNs}identifyPkgOrgs`);
    }
    // Array of SfdxOrgInfo objects that will hold Packaging Org info.
    const pkgOrgInfos = new Array();
    // Iterate over the Org Info list and identify Packaging Orgs.
    for (const orgInfo of sfdxOrgInfoMap.values()) {
        if (await orgInfo.determinePkgOrgStatus()) {
            sfdx_falcon_debug_1.SfdxFalconDebug.str(`${dbgNs}identifyPkgOrgs:`, `${orgInfo.alias}(${orgInfo.username})`, `PACKAGING ORG FOUND: `);
            pkgOrgInfos.push(orgInfo);
        }
        else {
            sfdx_falcon_debug_1.SfdxFalconDebug.str(`${dbgNs}identifyPkgOrgs:`, `${orgInfo.alias}(${orgInfo.username})`, `NOT A PACKAGING ORG: `);
        }
    }
    // DEBUG
    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}identifyPkgOrgs:pkgOrgInfos`, pkgOrgInfos, `pkgOrgInfos: `);
    // Return the list of Packaging Orgs to the caller
    return pkgOrgInfos;
}
exports.identifyPkgOrgs = identifyPkgOrgs;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    mdapiConvert
 * @param       {string}  mdapiSourceRootDir  Required. ???
 * @param       {string}  sfdxSourceOutputDir Required. ???
 * @returns     {Promise<SfdxFalconResult>} Uses an SfdxShellResult to return data to the caller for
 *              both RESOLVE and REJECT.
 * @description Uses the Salesforce CLI's force:mdapi:convert command to convert MDAPI source to
 *              SFDX source.
 * @public @async
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
async function mdapiConvert(mdapiSourceRootDir, sfdxSourceOutputDir) {
    // Set the SFDX Command String to be used by this function.
    const sfdxCommandString = `sfdx force:mdapi:convert `
        + ` --rootdir ${mdapiSourceRootDir}`
        + ` --outputdir ${sfdxSourceOutputDir}`
        + ` --loglevel debug`
        + ` --json`;
    // Introduce a small delay in case this is being used by an Observable Listr Task.
    await async_1.waitASecond(3);
    // Initialize a UTILITY Result for this function.
    const utilityResult = new sfdx_falcon_result_1.SfdxFalconResult(`sfdx:mdapiConvert`, "UTILITY" /* UTILITY */);
    const utilityResultDetail = {
        sfdxCommandString: sfdxCommandString,
        stdOutParsed: null,
        stdOutBuffer: null,
        stdErrBuffer: null,
        error: null
    };
    utilityResult.detail = utilityResultDetail;
    utilityResult.debugResult('Utility Result Initialized', `${dbgNs}mdapiConvert:`);
    // Define the success, failure, and "mixed" messages for the SFDX command execution.
    const messages = {
        failureMessage: 'MDAPI Source Conversion Failed',
        successMessage: 'MDAPI Source Conversion Succeeded',
        mixedMessage: 'MDAPI Source Conversion failed but the CLI returned a Success Response'
    };
    // Execute the Salesforce CLI Command.
    return executeSfdxCommand(sfdxCommandString, utilityResult, messages);
}
exports.mdapiConvert = mdapiConvert;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    resolveConnection
 * @param       {AliasOrConnection} aliasOrConnection  Required. Either a string containing the
 *              Alias of the org being queried or an authenticated JSForce Connection object.
 * @returns     {Promise<ResolvedConnection>}  Resolves with an authenticated JSForce Connection.
 * @description Given an Alias/Username or a JSForce Connection, returns a valid JSForce Connection.
 * @public @async
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
async function resolveConnection(aliasOrConnection) {
    // Input validation
    if (typeof aliasOrConnection !== 'string'
        && (typeof aliasOrConnection === 'object'
            && (aliasOrConnection instanceof core_3.Connection) !== true)) {
        throw new sfdx_falcon_error_1.SfdxFalconError(`Expected aliasOrConnection to be a string or Connection Object. Got '${typeof aliasOrConnection}' instead. `, `TypeError`, `${dbgNs}resolveConnection`);
    }
    let connection;
    let orgIdentifier;
    // Either get a new connection based on an alias or use one provided to us.
    if (typeof aliasOrConnection === 'string') {
        orgIdentifier = aliasOrConnection;
        connection = await getConnection(aliasOrConnection);
    }
    else {
        connection = aliasOrConnection;
        orgIdentifier = connection.getUsername();
    }
    // Return a ResolvedConnection object
    return {
        connection: connection,
        orgIdentifier: orgIdentifier
    };
}
exports.resolveConnection = resolveConnection;
// ────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    scanConnectedOrgs
 * @returns     {Promise<SfdxFalconResult>} Uses an SfdxFalconResult to return data to the caller for
 *              both RESOLVE and REJECT.
 * @description Calls force:org:list via an async shell command, then creates an array of
 *              SfdxOrgInfo objects by parsing the JSON response returned by the CLI command.
 *              Sends results back to caller as an SfdxFalconResult.
 * @public @async
 */
// ────────────────────────────────────────────────────────────────────────────────────────────────┘
async function scanConnectedOrgs() {
    // Set the SFDX Command String to be used by this function.
    const sfdxCommandString = `sfdx force:org:list --json`;
    // Initialize an UTILITY Result for this function.
    const utilityResult = new sfdx_falcon_result_1.SfdxFalconResult(`sfdx:executeSfdxCommand`, "UTILITY" /* UTILITY */);
    const utilityResultDetail = {
        sfdxCommandString: sfdxCommandString,
        stdOutParsed: null,
        stdOutBuffer: null,
        stdErrBuffer: null,
        error: null
    };
    utilityResult.detail = utilityResultDetail;
    utilityResult.debugResult('Utility Result Initialized', `${dbgNs}scanConnectedOrgs:`);
    // Define the success, failure, and "mixed" messages for the SFDX command execution.
    const messages = {
        failureMessage: 'Unable to scan Connected Orgs',
        successMessage: 'Scan Connected Orgs Succeeded',
        mixedMessage: 'Scan Connected Orgs Failed but the CLI Returned a Success Response'
    };
    // Execute the Salesforce CLI Command.
    return executeSfdxCommand(sfdxCommandString, utilityResult, messages);
}
exports.scanConnectedOrgs = scanConnectedOrgs;
//# sourceMappingURL=sfdx.js.map