/**
 * @interface   SfdxFalconKeyValueTableDataRow
 * @description Represents a single row of data for use in an SFDX-Falon Table.
 */
export interface SfdxFalconKeyValueTableDataRow {
    option: string;
    value: string;
    height?: number;
}
/**
 * @type        SfdxFalconTableData
 * @description Represents an array of SfdxFalconKeyValueTableDataRow object literals.
 */
export declare type SfdxFalconTableData = SfdxFalconKeyValueTableDataRow[];
/**
 * @interface   StatusMessage
 * @description Represents a Status Message that may eventually be displayed using a Falcon Table.
 */
export interface StatusMessage {
    title: string;
    message: string;
    type: 'error' | 'info' | 'success' | 'warning';
}
/**
 * @interface   TableColumn
 * @description Represents the settings for a column of data in a Falcon Table.
 */
export interface TableColumn {
    key: string;
    label?: string | (() => string);
    width: number;
    format(value: string, row: string): string;
    get(row: any[]): string;
}
/**
 * @interface   TableColumnKey
 * @description Represents the Key that identifies a column in a Falcon Table.
 */
export interface TableColumnKey {
    key: string;
    label?: string | (() => string);
}
/**
 * @interface   TableOptions
 * @description Represents the options that can be set when constructing a Falcon Table.
 */
export interface TableOptions {
    columns: Partial<TableColumn>[];
    colSep: string;
    headerAnsi: any;
    after(row: SfdxFalconKeyValueTableDataRow, options: TableOptions): void;
    printLine(row: any[]): void;
    printRow(row: any[]): void;
    printInverseRow(row: any[]): void;
    printHeader(row: any[]): void;
}
/**
 * @function    printStatusMessages
 * @param       {StatusMessage} statusMessages  Required. The status message to be printed to console.
 * @param       {number}  [padLength] Optional. The minimum character distance between the start of
 *              the message label and the separator character.
 * @param       {string}  [separator] Optional. Separates the Status Message title and message.
 * @returns     {void}
 * @description Given a single status message, outputs that message to the console using specialized
 *              formatting depending on the type of status message provided. Has the ability to
 *              ensure that multiple status messages printed back-to-back will have identical
 *              positioning of the separator as long as a value is provided for padLength.
 * @public
 */
export declare function printStatusMessage(statusMessage: StatusMessage, padLength?: number, separator?: string): void;
/**
 * @function    printStatusMessages
 * @param       {StatusMessage[]} statusMessages  Required. Status messages to be printed to console.
 * @param       {number}  [padLength] Optional. The minimum character distance between the start of
 *              the message label and the separator character.
 * @param       {string}  [separator] Optional. Separates the Status Message title and message.
 * @returns     {void}
 * @description Given an array of Status Message objects, prints a formatted Falcon Table to the
 *              console with color-coded text for each message, depending on message status.
 * @public
 */
export declare function printStatusMessages(statusMessages: StatusMessage[], separator?: string): void;
/**
 * @class       SfdxFalconKeyValueTable
 * @description Uses table creation code borrowed from the SFDX-Core UX library to make it easy to
 *              build "Key/Value" tables. These are marked by a single line header that uses an
 *              inverse font, a single space separator between cells, bold text for "Keys" and
 *              green text for "Values".
 * @public
 */
export declare class SfdxFalconKeyValueTable {
    private tableColumnKeys;
    private tableOptions;
    /**
     * @constructs  SfdxFalconKeyValueTable
     * @param       {TableColumnKey[]} [tableColumnKeys] Optional. Overrides Column keys/labels.
     * @param       {Partial<TableOptions>} [tableOptions] Optional. Allows override of the Table
     *              Options that are used to initialize/render the table.
     * @param       {boolean} [debugMode] Optional. Set to TRUE to enable debug output from inside
     *              SfdxFalconKeyValueTable.
     * @description Constructs an SfdxFalconKeyValueTable object.
     * @public
     */
    constructor(tableColumnKeys?: TableColumnKey[], tableOptions?: Partial<TableOptions>, debugMode?: boolean);
    /**
     * @method      render
     * @param       {SfdxFalconKeyValueTableDataRow[]} keyValueTableData Array of table data
     *              row objects. Each row must provided data using the keys "option" and "value".
     * @returns     {void}
     * @description Renders an SFDX-Falcon themed Key/Value table with the header "OPTIONS"/"DEFAULT"
     * @public
     */
    render(keyValueTableData: SfdxFalconKeyValueTableDataRow[]): void;
}
