"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
//─────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @file          modules/sfdx-falcon-util/ux.ts
 * @copyright     Vivek M. Chawla - 2018
 * @author        Vivek M. Chawla <@VivekMChawla>
 * @summary       Console UX utility helper library
 * @description   Exports classes that provide various console.log() based UX / display functions.
 * @version       1.0.0
 * @license       MIT
 */
//─────────────────────────────────────────────────────────────────────────────────────────────────┘
// Import External Modules
const _ = require("lodash");
// Import Internal Modules
const sfdx_falcon_debug_1 = require("../sfdx-falcon-debug"); // Class. Specialized debug provider for SFDX-Falcon code.
const sfdx_falcon_error_1 = require("../sfdx-falcon-error"); // Class. Extends SfdxError to provide specialized error structures for SFDX-Falcon modules.
// Requires
const stripAnsi = require('strip-ansi'); // Strips ANSI escape codes from strings.
const chalk = require('chalk'); // Utility for creating colorful console output.
const pad = require('pad'); // Provides consistent spacing when trying to align console output.
// Set the File Local Debug Namespace
const dbgNs = 'UTILITY:ux:';
//─────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    printStatusMessages
 * @param       {StatusMessage} statusMessages  Required. The status message to be printed to console.
 * @param       {number}  [padLength] Optional. The minimum character distance between the start of
 *              the message label and the separator character.
 * @param       {string}  [separator] Optional. Separates the Status Message title and message.
 * @returns     {void}
 * @description Given a single status message, outputs that message to the console using specialized
 *              formatting depending on the type of status message provided. Has the ability to
 *              ensure that multiple status messages printed back-to-back will have identical
 *              positioning of the separator as long as a value is provided for padLength.
 * @public
 */
//─────────────────────────────────────────────────────────────────────────────────────────────────┘
function printStatusMessage(statusMessage, padLength = 0, separator = ' : ') {
    // Validate input
    if (typeof statusMessage.title !== 'string') {
        throw new sfdx_falcon_error_1.SfdxFalconError(`Expected string for statusMessage.title but got '${typeof statusMessage.title}'`, `TypeError`, `${dbgNs}printStatusMessage`);
    }
    if (typeof statusMessage.message !== 'string') {
        throw new sfdx_falcon_error_1.SfdxFalconError(`Expected string for statusMessage.message but got '${typeof statusMessage.message}'`, `TypeError`, `${dbgNs}printStatusMessage`);
    }
    if (typeof separator !== 'string') {
        throw new sfdx_falcon_error_1.SfdxFalconError(`Expected string for separator but got '${typeof separator}'`, `TypeError`, `${dbgNs}printStatusMessage`);
    }
    if (isNaN(padLength)) {
        padLength = 0;
    }
    // Print the Status Message
    switch (statusMessage.type) {
        case 'error':
            console.log(chalk `{bold.red ${pad(statusMessage.title, padLength) + separator}}{bold ${statusMessage.message}}`);
            break;
        case 'info':
            console.log(chalk `{bold ${pad(statusMessage.title, padLength) + separator}}${statusMessage.message}`);
            break;
        case 'success':
            console.log(chalk `{bold ${pad(statusMessage.title, padLength) + separator}}{green ${statusMessage.message}}`);
            break;
        case 'warning':
            console.log(chalk `{bold ${pad(statusMessage.title, padLength) + separator}}{yellow ${statusMessage.message}}`);
            break;
        default:
            throw new sfdx_falcon_error_1.SfdxFalconError(`Invalid setting for statusMessage.type: '${statusMessage.type}'`, `InvalidMessageType`, `${dbgNs}printStatusMessage`);
    }
}
exports.printStatusMessage = printStatusMessage;
//─────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    printStatusMessages
 * @param       {StatusMessage[]} statusMessages  Required. Status messages to be printed to console.
 * @param       {number}  [padLength] Optional. The minimum character distance between the start of
 *              the message label and the separator character.
 * @param       {string}  [separator] Optional. Separates the Status Message title and message.
 * @returns     {void}
 * @description Given an array of Status Message objects, prints a formatted Falcon Table to the
 *              console with color-coded text for each message, depending on message status.
 * @public
 */
//─────────────────────────────────────────────────────────────────────────────────────────────────┘
function printStatusMessages(statusMessages, separator = ' : ') {
    // Validate input
    if (Array.isArray(statusMessages) === false) {
        throw new sfdx_falcon_error_1.SfdxFalconError(`Expected array for statusMessages but got '${typeof statusMessages}'`, `TypeError`, `${dbgNs}printStatusMessages`);
    }
    if (typeof separator !== 'string') {
        throw new sfdx_falcon_error_1.SfdxFalconError(`Expected string for separator but got '${typeof separator}'`, `TypeError`, `${dbgNs}printStatusMessages`);
    }
    // Calculate the length of the longest StatusMessage Title
    let longestTitle = 0;
    for (const statusMessage of statusMessages) {
        if (typeof statusMessage.title !== 'undefined') {
            longestTitle = Math.max(statusMessage.title.length, longestTitle);
        }
    }
    // Set the header text for the final status messages. Padding to length
    // of longest title, then adding separator length - 1 will only provide
    // ideal results if there is one blank space to the right of the specified
    // separator string.
    const statusHeader = pad('Final Status', longestTitle + separator.length - 1);
    // Print a header for the Status Messages
    console.log(chalk `{inverse ${statusHeader}}`);
    // Print all of the Satus Messages
    for (const statusMessage of statusMessages) {
        printStatusMessage(statusMessage, longestTitle, separator);
    }
}
exports.printStatusMessages = printStatusMessages;
//─────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @function    renderTable
 * @param       {SfdxFalconKeyValueTableDataRow[]}  data  Required.
 * @param       {Partial<TableOptions>} [tableOptionsOverride]  Optional.
 * @returns     {void}
 * @description Table rendering code borrowed from oclif's cli-ux module. Original code can be found
 *              at https://github.com/oclif/cli-ux/blob/master/src/styled/table.ts
 * @private
 */
//─────────────────────────────────────────────────────────────────────────────────────────────────┘
function renderTable(data, tableOptionsOverride = {}) {
    const tableOptions = Object.assign({ 
        // DEFINE default table tableOptions
        colSep: '  ', after: () => { }, headerAnsi: _.identity, printLine: (s) => console.log(s), printRow: function (cells) {
            this.printLine(cells.join(this.colSep).trimRight()); // tslint:disable-line: no-any
        }, printInverseRow: function (cells) {
            for (let i = 0; i < cells.length; i++) {
                process.stdout.write(chalk.inverse(cells[i]));
                if (i < cells.length - 1) {
                    process.stdout.write(this.colSep);
                }
            }
            process.stdout.write('\n');
        }, printHeader: function (cells) {
            this.printRow(cells.map(_.ary(this.headerAnsi, 1)));
            this.printRow(cells.map(hdr => hdr.replace(/./g, '─')));
        } }, tableOptionsOverride, { 
        // BEGIN columns definition. This must be the LAST element of the TableOptions definition
        columns: (tableOptionsOverride.columns || []).map(c => (Object.assign({ format: (value) => {
                return value != null ? value.toString() : '';
            }, width: 0, label: function () {
                return this.key.toString();
            }, get: function (row) {
                let value; // tslint:disable-line: no-any
                const path = _.result(this, 'key'); // tslint:disable-line: no-any
                if (!path) {
                    value = row;
                }
                else {
                    value = _.get(row, path);
                }
                return this.format(value, row); // tslint:disable-line: no-any
            } }, c))) });
    sfdx_falcon_debug_1.SfdxFalconDebug.obj(`${dbgNs}renderTable:`, tableOptions, `tableOptions: `);
    function calcWidth(cell) {
        const lines = stripAnsi(cell).split(/[\r\n]+/);
        const lineLengths = lines.map(_.property('length'));
        return Math.max.apply(Math, lineLengths);
    }
    function pad(string, length) {
        const visibleLength = stripAnsi(string).length;
        const diff = length - visibleLength;
        return string + ' '.repeat(Math.max(0, diff));
    }
    function render() {
        let columns = tableOptions.columns; // tslint:disable-line: no-any
        if (typeof columns[0] === 'string') {
            columns = columns.map((key) => ({ key: key })); // tslint:disable-line: no-any
        }
        for (const row of data) {
            row.height = 1;
            for (const col of columns) {
                const cell = col.get(row); // tslint:disable-line: no-any
                col.width = Math.max(_.result(col, 'label').length, col.width || 0, calcWidth(cell));
                row.height = Math.max(row.height || 0, cell.split(/[\r\n]+/).length);
            }
        }
        if (tableOptions.printHeader) {
            tableOptions.printHeader(columns.map(col => {
                const label = _.result(col, 'label');
                return pad(label, col.width || label.length);
            }));
        }
        function getNthLineOfCell(n, row, col) {
            const lines = col.get(row).split(/[\r\n]+/);
            return pad(lines[n] || '', col.width);
        }
        for (const row of data) {
            for (let i = 0; i < (row.height || 0); i++) {
                const cells = columns.map(_.partial(getNthLineOfCell, i, row));
                tableOptions.printRow(cells);
            }
            tableOptions.after(row, tableOptions);
        }
    }
    render();
}
//─────────────────────────────────────────────────────────────────────────────────────────────────┐
/**
 * @class       SfdxFalconKeyValueTable
 * @description Uses table creation code borrowed from the SFDX-Core UX library to make it easy to
 *              build "Key/Value" tables. These are marked by a single line header that uses an
 *              inverse font, a single space separator between cells, bold text for "Keys" and
 *              green text for "Values".
 * @public
 */
//─────────────────────────────────────────────────────────────────────────────────────────────────┘
class SfdxFalconKeyValueTable {
    //───────────────────────────────────────────────────────────────────────────────────────────────┐
    /**
     * @constructs  SfdxFalconKeyValueTable
     * @param       {TableColumnKey[]} [tableColumnKeys] Optional. Overrides Column keys/labels.
     * @param       {Partial<TableOptions>} [tableOptions] Optional. Allows override of the Table
     *              Options that are used to initialize/render the table.
     * @param       {boolean} [debugMode] Optional. Set to TRUE to enable debug output from inside
     *              SfdxFalconKeyValueTable.
     * @description Constructs an SfdxFalconKeyValueTable object.
     * @public
     */
    //───────────────────────────────────────────────────────────────────────────────────────────────┘
    constructor(tableColumnKeys, tableOptions, debugMode) {
        // Define the table columns. Use defaults if not properly specified by caller.
        if (typeof tableColumnKeys === 'undefined' || tableColumnKeys.length !== 2) {
            // By default, the keys should be "option" and "value".
            this.tableColumnKeys = [
                {
                    key: 'option',
                    label: 'OPTIONS'
                },
                {
                    key: 'value',
                    label: 'VALUES'
                }
            ];
        }
        // Define table options. Use the spread operator to add any overrides
        // specified by the caller.  Note that the overrides of printHeader
        // and printRow are what makes this a "Key/Value" table.
        this.tableOptions = Object.assign({ colSep: ' ', printHeader: function (cells) {
                const headerCells = cells.map(_.ary(this.headerAnsi, 1));
                for (let i = 0; i < headerCells.length; i++) {
                    process.stdout.write(chalk.inverse(headerCells[i]));
                    if (i < headerCells.length - 1) {
                        process.stdout.write(this.colSep);
                    }
                }
                process.stdout.write('\n');
            }, printRow: function (cells) {
                process.stdout.write(chalk.bold(cells[0]));
                process.stdout.write(this.colSep);
                process.stdout.write(chalk.green(cells[1]));
                process.stdout.write('\n');
            } }, tableOptions, { 
            // Define the columns key here to make sure that OUR definition is the last one set.
            columns: this.tableColumnKeys });
    }
    //───────────────────────────────────────────────────────────────────────────────────────────────┐
    /**
     * @method      render
     * @param       {SfdxFalconKeyValueTableDataRow[]} keyValueTableData Array of table data
     *              row objects. Each row must provided data using the keys "option" and "value".
     * @returns     {void}
     * @description Renders an SFDX-Falcon themed Key/Value table with the header "OPTIONS"/"DEFAULT"
     * @public
     */
    //───────────────────────────────────────────────────────────────────────────────────────────────┘
    render(keyValueTableData) {
        renderTable(keyValueTableData, this.tableOptions);
    }
}
exports.SfdxFalconKeyValueTable = SfdxFalconKeyValueTable;
//# sourceMappingURL=ux.js.map