/**
 * @file          modules/sfdx-falcon-yeoman-command/index.ts
 * @copyright     Vivek M. Chawla - 2018
 * @author        Vivek M. Chawla <@VivekMChawla>
 * @summary       Exports SfdxFalconYeomanCommand for use with custom Salesforce CLI commands.
 * @description   Exports an abstract class that adds support for running a Yeoman Generator inside
 *                of custom-built Salesforce CLI commands.
 * @version       1.0.0
 * @license       MIT
 */
import { AnyJson } from '@salesforce/ts-types';
import { SfdxFalconCommand } from '../sfdx-falcon-command';
import { SfdxFalconCommandType } from '../sfdx-falcon-command';
import { SfdxFalconResult } from '../sfdx-falcon-result';
import { GeneratorStatus } from '../sfdx-falcon-util/yeoman';
/**
 * @interface   GeneratorOptions
 * @description Specifies options used when spinning up an SFDX-Falcon Yeoman environment.
 */
export interface GeneratorOptions {
    commandName?: string;
    commandResult?: SfdxFalconResult;
    generatorType?: string;
    generatorStatus?: GeneratorStatus;
    generatorResult?: SfdxFalconResult;
    [key: string]: AnyJson | GeneratorStatus | SfdxFalconResult | string | number;
}
/**
 * @class       SfdxFalconYeomanCommand
 * @extends     SfdxFalconCommand
 * @summary     Abstract base class class for building Salesforce CLI commands that use Yeoman.
 * @description Classes that extend SfdxYeomanCommand will be able to run any Generator defined
 *              in the src/generators directory.  The file name in src/generators should match the
 *              generatorType string passed into runYeomanGenerator().  For example, if
 *              generatorType==="my-generator", then there MUST be a TS script file located at
 *              src/generators/my-generator.ts.
 * @public @abstract
 */
export declare abstract class SfdxFalconYeomanCommand extends SfdxFalconCommand {
    /**
     * @function    onError
     * @param       {unknown} rejectedPromise Required. Result of the failed
     *              Yeoman Generator.
     * @param       {boolean} [showErrorDebug]  Optional. Determines if extended
     *              debugging output the Error Result can be shown.
     * @param       {boolean} [promptUser] Optional. Determines if the user will
     *              be prompted to display debug info. If FALSE, debug info will
     *              be shown without requiring additional user input.
     * @returns     {Promise<void>}
     * @description Handles the output of a failed Yeoman Generator run.
     * @protected @async
     */
    protected onError(rejectedPromise: unknown, showErrorDebug?: boolean, promptUser?: boolean): Promise<void>;
    /**
     * @function    onSuccess
     * @param       {unknown} resolvedPromise Required. This should be an
     *              SfdxFalconResult object returned from runYeomanGenerator()
     * @returns     {Promise<void>}
     * @description Handles the output of a successful Yeoman Generator run.
     * @protected @asnyc
     */
    protected onSuccess(resolvedPromise: unknown): Promise<void>;
    /**
     * @function    runYeomanGenerator
     * @param       {GeneratorOptions}  generatorOptions  Required. Options that
     *              specify how the Yeoman Generator should run.
     * @returns     {Promise<SfdxFalconResult>}  Returns a promise that resolves
     *              and rejects with an SFDX-Falcon Result. The output of this
     *              function is intended to be consumed by the onSuccess() and
     *              onError() methods.
     * @description Runs the specified Yeoman generator using the given options.
     * @protected @async
     */
    protected runYeomanGenerator(generatorOptions: GeneratorOptions): Promise<SfdxFalconResult>;
    /**
     * @function    sfdxFalconCommandInit
     * @param       {string}  commandName Required. Name of the command.
     * @param       {SfdxFalconCommandType} commandType Required. Type of command.
     * @returns     {void}
     * @description Initializes various SfdxFalconYeomandCommand structures before
     *              calling the same init function from SfdxFalconCommand.
     * @protected
     */
    protected sfdxFalconCommandInit(commandName: string, commandType: SfdxFalconCommandType): void;
}
