import * as Generator from 'yeoman-generator';
import { SfdxFalconInterview } from '../sfdx-falcon-interview';
import { SfdxFalconResult } from '../sfdx-falcon-result';
import { SfdxFalconKeyValueTable } from '../sfdx-falcon-util/ux';
import { SfdxFalconTableData } from '../sfdx-falcon-util/ux';
import { GeneratorStatus } from '../sfdx-falcon-util/yeoman';
import { GeneratorOptions } from '../sfdx-falcon-yeoman-command';
import { SfdxFalconProjectConfig } from '../sfdx-falcon-types';
/**
 * @class       SfdxFalconYeomanGenerator
 * @extends     Generator
 * @summary     Abstract base class class for building Yeoman Generators for SFDX-Falcon commands.
 * @description Classes that extend SfdxFalconYeomanGenerator must provide a type parameter to
 *              ensure that the "xAnswers" family of member variables has the appropriate interface
 *              type which defines the answers that are relevant to a concrete child class.
 * @public @abstract
 */
export declare abstract class SfdxFalconYeomanGenerator<T extends object> extends Generator {
    protected cliCommandName: string;
    protected pluginVersion: string;
    protected sfdcApiVersion: string;
    protected successMessage: string;
    protected failureMessage: string;
    protected warningMessage: string;
    protected openingMessage: string;
    protected generatorStatus: GeneratorStatus;
    protected generatorResult: SfdxFalconResult;
    protected generatorType: string;
    protected installComplete: boolean;
    protected falconTable: SfdxFalconKeyValueTable;
    protected userInterview: SfdxFalconInterview<T>;
    protected defaultAnswers: T;
    protected finalAnswers: T;
    protected metaAnswers: T;
    protected confirmationQuestion: string;
    protected sharedData: object;
    /**
     * @constructs  SfdxFalconYeomanGenerator
     * @param       {string|string[]} args Required. Not used (as far as I know).
     * @param       {GeneratorOptions}  opts Required. Sets generator options.
     * @description Constructs a SfdxFalconYeomanGenerator object.
     * @public
     */
    constructor(args: string | string[], opts: GeneratorOptions);
    protected abstract _buildInterview(): SfdxFalconInterview<T>;
    protected abstract _buildInterviewAnswersTableData(userAnswers: T): Promise<SfdxFalconTableData>;
    /**
     * @method      _cloneRepository
     * @returns     {Promise<string>} Local path into which the Git Repository
     *              was cloned. If the clone operation is unsuccessful, this
     *              will return an empty string.
     * @description Clones a remote Git Repository per information specified
     *              during by the command and/or during their interview. Returns
     *              the local path to which the Git Repository was cloned, or
     *              an empty string otherwise.
     * @protected @async
     */
    protected _cloneRepository(): Promise<string>;
    /**
     * @method      _default_initializing
     * @returns     {Promise<void>}
     * @description STEP ONE in the Yeoman run-loop.  Uses Yeoman's "initializing"
     *              run-loop priority.  This is a "default" implementation and
     *              should work for most SFDX-Falcon use cases. It must be called
     *              from inside the initializing() method of the child class.
     * @protected @async
     */
    protected _default_initializing(): Promise<void>;
    /**
     * @method      _default_prompting
     * @returns     {Promise<void>}
     * @description STEP TWO in the Yeoman run-loop. Interviews the User to get
     *              information needed by the "writing" and "installing" phases.
     *              This is a "default" implementation and should work for most
     *              SFDX-Falcon use cases.
     * @protected @async
     */
    protected _default_prompting(): Promise<void>;
    /**
     * @method      _default_configuring
     * @returns     {void}
     * @description STEP THREE in the Yeoman run-loop. Perform any pre-install
     *              configuration steps based on the answers provided by the User.
     *              This is a "default" implementation and should work for most
     *              SFDX-Falcon use cases.
     * @protected
     */
    protected _default_configuring(): void;
    /**
     * @method      _default_end
     * @returns     {void}
     * @description STEP SIX in the Yeoman run-loop. This is the FINAL step that
     *              Yeoman runs and it gives us a chance to do any post-Yeoman
     *              updates and/or cleanup.
     * @protected
     */
    protected _default_end(): void;
    /**
     * @method      _executeInitializationTasks
     * @returns     {Promise<void>}
     * @description Runs a series of initialization tasks using the Listr UX/Task
     *              Runner module.  Listr provides a framework for executing tasks
     *              while also providing an attractive, realtime display of task
     *              status (running, successful, failed, etc.).
     * @protected @async
     */
    protected _executeInitializationTasks(): Promise<void>;
    /**
     * @method      _finalizeGitActions
     * @param       {string}  destinationRoot Required.
     * @param       {boolean} isInitializingGit Required.
     * @param       {string}  gitRemoteUri  Required.
     * @param       {string}  projectAlias  Required.
     * @returns     {Promise<void>}
     * @description Intended to run after _finalizeProjectCreation() during the
     *              Yeoman "writing" phase.  Initializes local Git repo, and will
     *              even try to attach a Git remote if specified by the user.
     * @protected @async
     */
    protected _finalizeGitActions(destinationRoot: string, isInitializingGit: boolean, gitRemoteUri: string, projectAlias: string): Promise<void>;
    /**
     * @method      _finalizeProjectCloning
     * @returns     {boolean} Returns FALSE if the project was aborted.
     * @description Intended to run after the Yeoman "writing" phase.  Has logic
     *              that ensures the Generator wasn't aborted, and then carries
     *              out finalization tasks that are generic to "cloning"
     *              Generators. Returns a boolean so the calling class can decide
     *              whether or not to perform additional actions.
     * @protected
     */
    protected _finalizeProjectCloning(): boolean;
    /**
     * @method      _finalizeProjectCreation
     * @returns     {boolean} Returns FALSE if the project was aborted.
     * @description Intended to run after the Yeoman "writing" phase.  Has logic
     *              that ensures the Generator wasn't aborted, and then carries
     *              out finalization tasks that are generic to "creation"
     *              Generators. Returns a boolean so the calling class can decide
     *              whether or not to perform additional actions.
     * @protected
     */
    protected _finalizeProjectCreation(): boolean;
    /**
     * @method      _resolveFalconProjectConfig
     * @returns     {Promise<SfdxFalconProjectConfig>}  Falcon Project config
     *              JSON for the project that resides at the path provided.
     * @description Given a local filepath, tries to resolve an SFDX Project at
     *              that location, then extracts the SFDX-Falcon specific config
     *              from it.
     * @protected @async
     */
    protected _resolveFalconProjectConfig(projectPath: string): Promise<SfdxFalconProjectConfig>;
}
